@@ -4,18 +4,15 @@
 
 "use strict";
 
-// Pass an empty scope object to the import to prevent "leaked window property"
-// errors in tests.
-var Preferences = ChromeUtils.import(
-  "resource://gre/modules/Preferences.jsm",
-  {}
-).Preferences;
-var TelemetryReportingPolicy = ChromeUtils.import(
-  "resource://gre/modules/TelemetryReportingPolicy.jsm",
-  {}
-).TelemetryReportingPolicy;
+var { Preferences } = ChromeUtils.importESModule(
+  "resource://gre/modules/Preferences.sys.mjs"
+);
+var { TelemetryReportingPolicy } = ChromeUtils.importESModule(
+  "resource://gre/modules/TelemetryReportingPolicy.sys.mjs"
+);
 
 const PREF_BRANCH = "datareporting.policy.";
+const PREF_FIRST_RUN = "toolkit.telemetry.reportingpolicy.firstRun";
 const PREF_BYPASS_NOTIFICATION =
   PREF_BRANCH + "dataSubmissionPolicyBypassNotification";
 const PREF_CURRENT_POLICY_VERSION = PREF_BRANCH + "currentPolicyVersion";
@@ -23,24 +20,29 @@ const PREF_ACCEPTED_POLICY_VERSION =
   PREF_BRANCH + "dataSubmissionPolicyAcceptedVersion";
 const PREF_ACCEPTED_POLICY_DATE =
   PREF_BRANCH + "dataSubmissionPolicyNotifiedTime";
+const PREF_TOS_ROLLOUT_POPULATION =
+  "browser.preonboarding.onTrainRolloutPopulation";
+const PREF_TOS_ROLLOUT_ENROLLED =
+  "browser.preonboarding.enrolledInOnTrainRollout";
+
+const PREF_TELEMETRY_LOG_LEVEL = "toolkit.telemetry.log.level";
 
 const TEST_POLICY_VERSION = 37;
 
 function fakeShowPolicyTimeout(set, clear) {
-  let reportingPolicy = ChromeUtils.import(
-    "resource://gre/modules/TelemetryReportingPolicy.jsm",
-    null
+  let reportingPolicy = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryReportingPolicy.sys.mjs"
   ).Policy;
   reportingPolicy.setShowInfobarTimeout = set;
   reportingPolicy.clearShowInfobarTimeout = clear;
 }
 
-function sendSessionRestoredNotification() {
-  let reportingPolicyImpl = ChromeUtils.import(
-    "resource://gre/modules/TelemetryReportingPolicy.jsm",
-    null
-  ).TelemetryReportingPolicyImpl;
-  reportingPolicyImpl.observe(null, "sessionstore-windows-restored", null);
+async function sendSessionRestoredNotification() {
+  let reportingPolicy = ChromeUtils.importESModule(
+    "resource://gre/modules/TelemetryReportingPolicy.sys.mjs"
+  ).Policy;
+
+  await reportingPolicy.fakeSessionRestoreNotification();
 }
 
 /**
@@ -56,10 +58,10 @@ function promiseNextTick() {
  * @return {Promise} Resolved when the notification is displayed.
  */
 function promiseWaitForAlertActive(aNotificationBox) {
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   aNotificationBox.stack.addEventListener(
     "AlertActive",
-    function() {
+    function () {
       deferred.resolve();
     },
     { once: true }
@@ -73,12 +75,12 @@ function promiseWaitForAlertActive(aNotificationBox) {
  * @return {Promise} Resolved when the notification is closed.
  */
 function promiseWaitForNotificationClose(aNotification) {
-  let deferred = PromiseUtils.defer();
+  let deferred = Promise.withResolvers();
   waitForNotificationClose(aNotification, deferred.resolve);
   return deferred.promise;
 }
 
-function triggerInfoBar(expectedTimeoutMs) {
+async function triggerInfoBar(expectedTimeoutMs) {
   let showInfobarCallback = null;
   let timeoutMs = null;
   fakeShowPolicyTimeout(
@@ -88,7 +90,7 @@ function triggerInfoBar(expectedTimeoutMs) {
     },
     () => {}
   );
-  sendSessionRestoredNotification();
+  await sendSessionRestoredNotification();
   Assert.ok(!!showInfobarCallback, "Must have a timer callback.");
   if (expectedTimeoutMs !== undefined) {
     Assert.equal(timeoutMs, expectedTimeoutMs, "Timeout should match");
@@ -96,9 +98,9 @@ function triggerInfoBar(expectedTimeoutMs) {
   showInfobarCallback();
 }
 
-var checkInfobarButton = async function(aNotification) {
+var checkInfobarButton = async function (aNotification) {
   // Check that the button on the data choices infobar does the right thing.
-  let buttons = aNotification.getElementsByTagName("button");
+  let buttons = aNotification.buttonContainer.getElementsByTagName("button");
   Assert.equal(
     buttons.length,
     1,
@@ -106,21 +108,31 @@ var checkInfobarButton = async function(aNotification) {
   );
   let button = buttons[0];
 
+  let openPrefsPromise = BrowserTestUtils.waitForLocationChange(
+    gBrowser,
+    "about:preferences#privacy"
+  );
+
   // Click on the button.
   button.click();
 
   // Wait for the preferences panel to open.
-  await promiseNextTick();
+  await openPrefsPromise;
 };
 
-add_task(async function setup() {
+add_setup(async function () {
+  const isFirstRun = Preferences.get(PREF_FIRST_RUN, true);
   const bypassNotification = Preferences.get(PREF_BYPASS_NOTIFICATION, true);
   const currentPolicyVersion = Preferences.get(PREF_CURRENT_POLICY_VERSION, 1);
 
   // Register a cleanup function to reset our preferences.
   registerCleanupFunction(() => {
+    Preferences.set(PREF_FIRST_RUN, isFirstRun);
     Preferences.set(PREF_BYPASS_NOTIFICATION, bypassNotification);
     Preferences.set(PREF_CURRENT_POLICY_VERSION, currentPolicyVersion);
+    Preferences.reset(PREF_TELEMETRY_LOG_LEVEL);
+    Preferences.reset(PREF_TOS_ROLLOUT_ENROLLED);
+    Preferences.reset(PREF_TOS_ROLLOUT_POPULATION);
 
     return closeAllNotifications();
   });
@@ -129,6 +141,12 @@ add_task(async function setup() {
   Preferences.set(PREF_BYPASS_NOTIFICATION, false);
   // Set the current policy version.
   Preferences.set(PREF_CURRENT_POLICY_VERSION, TEST_POLICY_VERSION);
+  // Ensure this isn't the first run, because then we open the first run page.
+  Preferences.set(PREF_FIRST_RUN, false);
+  TelemetryReportingPolicy.testUpdateFirstRun();
+  // Do not trigger ToS modal for Linux, Mac, and MSIX builds
+  Preferences.set(PREF_TOS_ROLLOUT_ENROLLED, false);
+  Preferences.set(PREF_TOS_ROLLOUT_POPULATION, 0);
 });
 
 function clearAcceptedPolicy() {
@@ -137,12 +155,7 @@ function clearAcceptedPolicy() {
   Preferences.reset(PREF_ACCEPTED_POLICY_DATE);
 }
 
-add_task(async function test_single_window() {
-  clearAcceptedPolicy();
-
-  // Close all the notifications, then try to trigger the data choices infobar.
-  await closeAllNotifications();
-
+function assertCoherentInitialState() {
   // Make sure that we have a coherent initial state.
   Assert.equal(
     Preferences.get(PREF_ACCEPTED_POLICY_VERSION, 0),
@@ -158,6 +171,16 @@ add_task(async function test_single_window() {
     !TelemetryReportingPolicy.testIsUserNotified(),
     "User not notified about datareporting policy."
   );
+}
+
+add_task(async function test_single_window() {
+  TelemetryReportingPolicy.reset();
+  clearAcceptedPolicy();
+
+  // Close all the notifications, then try to trigger the data choices infobar.
+  await closeAllNotifications();
+
+  assertCoherentInitialState();
 
   let alertShownPromise = promiseWaitForAlertActive(gNotificationBox);
   Assert.ok(
@@ -166,8 +189,10 @@ add_task(async function test_single_window() {
   );
 
   // Wait for the infobar to be displayed.
-  triggerInfoBar(10 * 1000);
+  await triggerInfoBar(10 * 1000);
+
   await alertShownPromise;
+  await promiseNextTick();
 
   Assert.equal(
     gNotificationBox.allNotifications.length,
@@ -211,6 +236,7 @@ add_task(async function test_single_window() {
   );
 });
 
+/* See bug 1571932
 add_task(async function test_multiple_windows() {
   clearAcceptedPolicy();
 
@@ -226,21 +252,7 @@ add_task(async function test_multiple_windows() {
     "2nd window has a global notification box."
   );
 
-  // Make sure that we have a coherent initial state.
-  Assert.equal(
-    Preferences.get(PREF_ACCEPTED_POLICY_VERSION, 0),
-    0,
-    "No version should be set on init."
-  );
-  Assert.equal(
-    Preferences.get(PREF_ACCEPTED_POLICY_DATE, 0),
-    0,
-    "No date should be set on init."
-  );
-  Assert.ok(
-    !TelemetryReportingPolicy.testIsUserNotified(),
-    "User not notified about datareporting policy."
-  );
+  assertCoherentInitialState();
 
   let showAlertPromises = [
     promiseWaitForAlertActive(gNotificationBox),
@@ -253,7 +265,7 @@ add_task(async function test_multiple_windows() {
   );
 
   // Wait for the infobars.
-  triggerInfoBar(10 * 1000);
+  await triggerInfoBar(10 * 1000);
   await Promise.all(showAlertPromises);
 
   // Both notification were displayed. Close one and check that both gets closed.
@@ -289,4 +301,4 @@ add_task(async function test_multiple_windows() {
     -1,
     "Date pref set."
   );
-});
+});*/
