@@ -8,6 +8,7 @@ var idnDomain =
 var tests = [
   {
     name: "normal domain",
+    // eslint-disable-next-line @microsoft/sdl/no-insecure-url
     location: "http://test1.example.org/",
     hostForDisplay: "test1.example.org",
     hasSubview: true,
@@ -15,6 +16,7 @@ var tests = [
   {
     name: "view-source",
     location: "view-source:http://example.com/",
+    // eslint-disable-next-line @microsoft/sdl/no-insecure-url
     newURI: "http://example.com/",
     hostForDisplay: "example.com",
     hasSubview: true,
@@ -27,12 +29,14 @@ var tests = [
   },
   {
     name: "IDN subdomain",
+    // eslint-disable-next-line @microsoft/sdl/no-insecure-url
     location: "http://sub1.xn--hxajbheg2az3al.xn--jxalpdlp/",
     hostForDisplay: "sub1." + idnDomain,
     hasSubview: true,
   },
   {
     name: "subdomain with port",
+    // eslint-disable-next-line @microsoft/sdl/no-insecure-url
     location: "http://sub1.test1.example.org:8000/",
     hostForDisplay: "sub1.test1.example.org",
     hasSubview: true,
@@ -71,13 +75,16 @@ var tests = [
   },
   {
     name: "chrome:",
-    location: "chrome://global/skin/in-content/info-pages.css",
-    hostForDisplay: "chrome://global/skin/in-content/info-pages.css",
+    location: "chrome://global/content/mozilla.html",
+    hostForDisplay: "chrome://global/content/mozilla.html",
     hasSubview: false,
   },
 ];
 
 add_task(async function test() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["browser.urlbar.scotchBonnet.enableOverride", false]],
+  });
   ok(gIdentityHandler, "gIdentityHandler should exist");
 
   await BrowserTestUtils.openNewForegroundTab(gBrowser);
@@ -118,12 +125,15 @@ async function runTest(i, forward) {
     false,
     currentTest.location
   );
-  BrowserTestUtils.loadURI(gBrowser.selectedBrowser, currentTest.location);
+  BrowserTestUtils.startLoadingURIString(
+    gBrowser.selectedBrowser,
+    currentTest.location
+  );
   await loaded;
   await popupHidden;
   ok(
     !gIdentityHandler._identityPopup ||
-      BrowserTestUtils.is_hidden(gIdentityHandler._identityPopup),
+      BrowserTestUtils.isHidden(gIdentityHandler._identityPopup),
     "Control Center is hidden"
   );
 
@@ -153,7 +163,7 @@ async function runTest(i, forward) {
   info("Waiting for the Control Center to be shown");
   await popupShown;
   ok(
-    !BrowserTestUtils.is_hidden(gIdentityHandler._identityPopup),
+    !BrowserTestUtils.isHidden(gIdentityHandler._identityPopup),
     "Control Center is visible"
   );
   let displayedHost = currentTest.hostForDisplay || currentTest.location;
