@@ -1,35 +1,50 @@
-ChromeUtils.defineModuleGetter(
-  this,
-  "AddonTestUtils",
-  "resource://testing-common/AddonTestUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  AddonTestUtils: "resource://testing-common/AddonTestUtils.sys.mjs",
+  ExtensionsUI: "resource:///modules/ExtensionsUI.sys.mjs",
+});
 
 const BASE = getRootDirectory(gTestPath).replace(
   "chrome://mochitests/content/",
   "https://example.com/"
 );
 
-var { ExtensionsUI } = ChromeUtils.import(
-  "resource:///modules/ExtensionsUI.jsm"
-);
-XPCOMUtils.defineLazyGetter(this, "Management", () => {
+ChromeUtils.defineLazyGetter(this, "Management", () => {
   // eslint-disable-next-line no-shadow
-  const { Management } = ChromeUtils.import(
-    "resource://gre/modules/Extension.jsm"
+  const { Management } = ChromeUtils.importESModule(
+    "resource://gre/modules/Extension.sys.mjs"
   );
   return Management;
 });
 
-ChromeUtils.import(
-  "resource://testing-common/CustomizableUITestUtils.jsm",
-  this
+let { CustomizableUITestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/CustomizableUITestUtils.sys.mjs"
 );
 let gCUITestUtils = new CustomizableUITestUtils(window);
 
-const { PermissionTestUtils } = ChromeUtils.import(
-  "resource://testing-common/PermissionTestUtils.jsm"
+const { PermissionTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/PermissionTestUtils.sys.mjs"
 );
 
+let extL10n = null;
+/**
+ * @param {string} id
+ * @param {object} [args]
+ * @returns {string}
+ */
+function formatExtValue(id, args) {
+  if (!extL10n) {
+    extL10n = new Localization(
+      [
+        "toolkit/global/extensions.ftl",
+        "toolkit/global/extensionPermissions.ftl",
+        "branding/brand.ftl",
+      ],
+      true
+    );
+  }
+  return extL10n.formatValueSync(id, args);
+}
+
 /**
  * Wait for the given PopupNotification to display
  *
@@ -59,8 +74,8 @@ function promisePopupNotificationShown(name) {
 }
 
 function promiseAppMenuNotificationShown(id) {
-  const { AppMenuNotifications } = ChromeUtils.import(
-    "resource://gre/modules/AppMenuNotifications.jsm"
+  const { AppMenuNotifications } = ChromeUtils.importESModule(
+    "resource://gre/modules/AppMenuNotifications.sys.mjs"
   );
   return new Promise(resolve => {
     function popupshown() {
@@ -186,44 +201,7 @@ function triggerPageOptionsAction(win, action) {
 }
 
 function isDefaultIcon(icon) {
-  // These are basically the same icon, but code within webextensions
-  // generates references to the former and generic add-ons manager code
-  // generates referces to the latter.
-  return (
-    icon == "chrome://browser/content/extension.svg" ||
-    icon == "chrome://mozapps/skin/extensions/extensionGeneric.svg"
-  );
-}
-
-/**
- * Check the contents of an individual permission string.
- * This function is fairly specific to the use here and probably not
- * suitable for re-use elsewhere...
- *
- * @param {string} string
- *        The string value to check (i.e., pulled from the DOM)
- * @param {string} key
- *        The key in browser.properties for the localized string to
- *        compare with.
- * @param {string|null} param
- *        Optional string to substitute for %S in the localized string.
- * @param {string} msg
- *        The message to be emitted as part of the actual test.
- */
-function checkPermissionString(string, key, param, msg) {
-  let localizedString = param
-    ? gBrowserBundle.formatStringFromName(key, [param])
-    : gBrowserBundle.GetStringFromName(key);
-
-  // If this is a parameterized string and the parameter isn't given,
-  // just do a simple comparison of the text before and after the %S
-  if (localizedString.includes("%S")) {
-    let i = localizedString.indexOf("%S");
-    ok(string.startsWith(localizedString.slice(0, i)), msg);
-    ok(string.endsWith(localizedString.slice(i + 2)), msg);
-  } else {
-    is(string, localizedString, msg);
-  }
+  return icon == "chrome://mozapps/skin/extensions/extensionGeneric.svg";
 }
 
 /**
@@ -240,16 +218,16 @@ function checkPermissionString(string, key, param, msg) {
  * @param {array} permissions
  *        The expected entries in the permissions list.  Each element
  *        in this array is itself a 2-element array with the string key
- *        for the item (e.g., "webextPerms.description.foo") and an
+ *        for the item (e.g., "webext-perms-description-foo") and an
  *        optional formatting parameter.
  * @param {boolean} sideloaded
  *        Whether the notification is for a sideloaded extenion.
  */
 function checkNotification(panel, checkIcon, permissions, sideloaded) {
   let icon = panel.getAttribute("icon");
+  let learnMoreLink = panel.querySelector(".popup-notification-learnmore-link");
   let ul = document.getElementById("addon-webext-perm-list");
   let singleDataEl = document.getElementById("addon-webext-perm-single-entry");
-  let learnMoreLink = document.getElementById("addon-webext-perm-info");
 
   if (checkIcon instanceof RegExp) {
     ok(
@@ -262,51 +240,91 @@ function checkNotification(panel, checkIcon, permissions, sideloaded) {
     is(icon, checkIcon, "Notification icon is correct");
   }
 
-  let description = panel.querySelector(".popup-notification-description")
-    .textContent;
-  let expectedDescription = "webextPerms.header";
+  let description = panel.querySelector(
+    ".popup-notification-description"
+  ).textContent;
+  let descL10nId = "webext-perms-header";
   if (permissions.length) {
-    expectedDescription += "WithPerms";
+    descL10nId = "webext-perms-header-with-perms";
   }
   if (sideloaded) {
-    expectedDescription = "webextPerms.sideloadHeader";
+    descL10nId = "webext-perms-sideload-header";
   }
-  checkPermissionString(
-    description,
-    expectedDescription,
-    undefined,
-    `Description is the expected one`
-  );
-  is(
-    learnMoreLink.hidden,
-    !permissions.length,
-    "Permissions learn more is hidden if there are no permissions"
+  const exp = formatExtValue(descL10nId, { extension: "<>" }).split("<>");
+  ok(description.startsWith(exp.at(0)), "Description is the expected one");
+  ok(description.endsWith(exp.at(-1)), "Description is the expected one");
+
+  const hasPBCheckbox = !!(
+    singleDataEl.querySelector("checkbox") || ul.querySelector("checkbox")
   );
 
-  if (!permissions.length) {
+  is(
+    BrowserTestUtils.isHidden(learnMoreLink),
+    !permissions.length && !hasPBCheckbox,
+    "Permissions learn more is hidden if there are no permissions and no private browsing checkbox"
+  );
+
+  if (!permissions.length && !hasPBCheckbox) {
+    ok(ul.hidden, "Permissions list is hidden");
+    ok(singleDataEl.hidden, "Expect a single permission entry to be hidden");
+  } else if (!permissions.length) {
     ok(ul.hidden, "Permissions list is hidden");
-    ok(singleDataEl.hidden, "Single permission data entry is hidden");
     ok(
-      !(ul.childElementCount || singleDataEl.textContent),
-      "Permission list and single permission element have no entries"
+      !singleDataEl.hidden,
+      "Expect a single permission entry for the private browsing checkbox to not be hidden"
     );
-  } else if (permissions.length === 1) {
-    ok(ul.hidden, "Permissions list is hidden");
+    ok(
+      singleDataEl.querySelector("checkbox"),
+      "Expect a checkbox inside the single permission entry"
+    );
+    ok(singleDataEl.textContent, "Single entry text content should not empty");
+    is(ul.childElementCount, 0, "Permission list should have no entries");
+  } else if (permissions.length === 1 && hasPBCheckbox) {
+    ok(!ul.hidden, "Permissions list to not be hidden");
+    is(ul.childElementCount, 2, "Expect 2 entries in the permissions list");
+    is(
+      ul.children[0].textContent,
+      formatExtValue(permissions[0]),
+      "First Permission entry is correct"
+    );
+    const lastEntry = ul.children[permissions.length];
+    ok(
+      lastEntry.classList.contains("webext-perm-privatebrowsing"),
+      "Expect last permissions list entry to be the private browsing checkbox"
+    );
+    ok(
+      lastEntry.querySelector("checkbox"),
+      "Expect a checkbox inside the last permissions list entry"
+    );
+  } else if (permissions.length === 1 && !hasPBCheckbox) {
+    ok(ul.hidden, "Permissions list to be hidden");
     ok(!ul.childElementCount, "Permission list has no entries");
-    ok(singleDataEl.textContent, "Single permission data label has been set");
   } else {
     ok(singleDataEl.hidden, "Single permission data entry is hidden");
     ok(
       !singleDataEl.textContent,
       "Single permission data label has not been set"
     );
+    ok(!ul.hidden, "Permissions list to not be hidden");
     for (let i in permissions) {
       let [key, param] = permissions[i];
-      checkPermissionString(
-        ul.children[i].textContent,
-        key,
-        param,
-        `Permission number ${i + 1} is correct`
+      const expected = formatExtValue(key, param);
+      // If the permissions list entry has a label child element then
+      // we expect the permission string to be set as the label element
+      // value (in particular this is the case when the permission dialog
+      // is going to show multiple host permissions as a single permission
+      // entry and a nested ul listing all those domains).
+      const permDescriptionEl = ul.children[i].querySelector("label")
+        ? ul.children[i].firstElementChild.value
+        : ul.children[i].textContent;
+      is(permDescriptionEl, expected, `Permission number ${i + 1} is correct`);
+    }
+
+    if (hasPBCheckbox) {
+      const lastEntry = ul.children[permissions.length];
+      ok(
+        lastEntry.classList.contains("webext-perm-privatebrowsing"),
+        "Expect last permissions list entry to be the private browsing checkbox"
       );
     }
   }
@@ -326,7 +344,7 @@ function checkNotification(panel, checkIcon, permissions, sideloaded) {
  *
  * @returns {Promise}
  */
-async function testInstallMethod(installFn, telemetryBase) {
+async function testInstallMethod(installFn) {
   const PERMS_XPI = "browser_webext_permissions.xpi";
   const NO_PERMS_XPI = "browser_webext_nopermissions.xpi";
   const ID = "permissions@test.mozilla.org";
@@ -338,10 +356,6 @@ async function testInstallMethod(installFn, telemetryBase) {
     ],
   });
 
-  if (telemetryBase !== undefined) {
-    hookExtensionsTelemetry();
-  }
-
   let testURI = makeURI("https://example.com/");
   PermissionTestUtils.add(testURI, "install", Services.perms.ALLOW_ACTION);
   registerCleanupFunction(() => PermissionTestUtils.remove(testURI, "install"));
@@ -383,17 +397,20 @@ async function testInstallMethod(installFn, telemetryBase) {
 
     let panel = await promisePopupNotificationShown("addon-webext-permissions");
     if (filename == PERMS_XPI) {
+      const hostPermissions = [
+        ["webext-perms-host-description-multiple-domains", { domainCount: 2 }],
+      ];
+
       // The icon should come from the extension, don't bother with the precise
       // path, just make sure we've got a jar url pointing to the right path
       // inside the jar.
       checkNotification(panel, /^jar:file:\/\/.*\/icon\.png$/, [
-        ["webextPerms.hostDescription.wildcard", "wildcard.domain"],
-        ["webextPerms.hostDescription.oneSite", "singlehost.domain"],
-        ["webextPerms.description.nativeMessaging"],
+        ...hostPermissions,
+        ["webext-perms-description-nativeMessaging"],
         // The below permissions are deliberately in this order as permissions
         // are sorted alphabetically by the permission string to match AMO.
-        ["webextPerms.description.history"],
-        ["webextPerms.description.tabs"],
+        ["webext-perms-description-history"],
+        ["webext-perms-description-tabs"],
       ]);
     } else if (filename == NO_PERMS_XPI) {
       checkNotification(panel, isDefaultIcon, []);
@@ -406,9 +423,8 @@ async function testInstallMethod(installFn, telemetryBase) {
       } catch (err) {}
     } else {
       // Look for post-install notification
-      let postInstallPromise = promiseAppMenuNotificationShown(
-        "addon-installed"
-      );
+      let postInstallPromise =
+        promiseAppMenuNotificationShown("addon-installed");
       panel.button.click();
 
       // Press OK on the post-install notification
@@ -445,16 +461,6 @@ async function testInstallMethod(installFn, telemetryBase) {
   //    the extension to clean up.)
   await runOnce(PERMS_XPI, false);
 
-  if (telemetryBase !== undefined) {
-    // Should see 2 canceled installs followed by 1 successful install
-    // for this method.
-    expectTelemetry([
-      `${telemetryBase}Rejected`,
-      `${telemetryBase}Rejected`,
-      `${telemetryBase}Accepted`,
-    ]);
-  }
-
   await SpecialPowers.popPrefEnv();
 }
 
@@ -464,6 +470,7 @@ async function testInstallMethod(installFn, telemetryBase) {
 // updates applied automatically or not.
 async function interactiveUpdateTest(autoUpdate, checkFn) {
   AddonTestUtils.initMochitest(this);
+  Services.fog.testResetFOG();
 
   const ID = "update2@tests.mozilla.org";
   const FAKE_INSTALL_SOURCE = "fake-install-source";
@@ -527,7 +534,10 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
 
   // Navigate away from the starting page to force about:addons to load
   // in a new tab during the tests below.
-  BrowserTestUtils.loadURI(gBrowser.selectedBrowser, "about:robots");
+  BrowserTestUtils.startLoadingURIString(
+    gBrowser.selectedBrowser,
+    "about:mozilla"
+  );
   await BrowserTestUtils.browserLoaded(gBrowser.selectedBrowser);
 
   // Install version 1.0 of the test extension
@@ -537,7 +547,7 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
   ok(addon, "Addon was installed");
   is(addon.version, "1.0", "Version 1 of the addon is installed");
 
-  let win = await BrowserOpenAddonsMgr("addons://list/extension");
+  let win = await BrowserAddonUI.openAddonsMgr("addons://list/extension");
 
   await waitAboutAddonsViewLoaded(win.document);
 
@@ -549,7 +559,8 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
   // Click the cancel button, wait to see the cancel event
   let cancelPromise = promiseInstallEvent(addon, "onInstallCancelled");
   panel.secondaryButton.click();
-  await cancelPromise;
+  const cancelledByUser = await cancelPromise;
+  is(cancelledByUser, true, "Install cancelled by user");
 
   addon = await AddonManager.getAddonByID(ID);
   is(addon.version, "1.0", "Should still be running the old version");
@@ -581,25 +592,36 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
     }
   );
 
+  const expectedSteps = [
+    // First update is cancelled on the permission prompt.
+    "started",
+    "download_started",
+    "download_completed",
+    "permissions_prompt",
+    "cancelled",
+    // Second update is expected to be completed.
+    "started",
+    "download_started",
+    "download_completed",
+    "permissions_prompt",
+    "completed",
+  ];
+
   Assert.deepEqual(
+    expectedSteps,
     collectedUpdateEvents.map(evt => evt.extra.step),
-    [
-      // First update is cancelled on the permission prompt.
-      "started",
-      "download_started",
-      "download_completed",
-      "permissions_prompt",
-      "cancelled",
-      // Second update is expected to be completed.
-      "started",
-      "download_started",
-      "download_completed",
-      "permissions_prompt",
-      "completed",
-    ],
     "Got the expected sequence on update telemetry events"
   );
 
+  let gleanEvents = AddonTestUtils.getAMGleanEvents("update");
+  Services.fog.testResetFOG();
+
+  Assert.deepEqual(
+    expectedSteps,
+    gleanEvents.map(e => e.step),
+    "Got the expected sequence on update Glean events."
+  );
+
   ok(
     collectedUpdateEvents.every(evt => evt.extra.addon_id === ID),
     "Every update telemetry event should have the expected addon_id extra var"
@@ -617,6 +639,19 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
     "Every update telemetry event should have the update_from extra var 'user'"
   );
 
+  for (let e of gleanEvents) {
+    is(e.addon_id, ID, "Glean event has the expected addon_id.");
+    is(e.source, FAKE_INSTALL_SOURCE, "Glean event has the expected source.");
+    is(e.updated_from, "user", "Glean event has the expected updated_from.");
+
+    if (e.step === "permissions_prompt") {
+      Assert.greater(parseInt(e.num_strings), 0, "Expected num_strings.");
+    }
+    if (e.step === "download_completed") {
+      Assert.greater(parseInt(e.download_time), 0, "Valid download_time.");
+    }
+  }
+
   let hasPermissionsExtras = collectedUpdateEvents
     .filter(evt => {
       return evt.extra.step === "permissions_prompt";
@@ -655,23 +690,18 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
 // Individual tests can store a cleanup function in the testCleanup global
 // to ensure it gets called before the final check is performed.
 let testCleanup;
-add_task(async function() {
+add_setup(async function head_setup() {
   let addons = await AddonManager.getAllAddons();
   let existingAddons = new Set(addons.map(a => a.id));
 
-  registerCleanupFunction(async function() {
+  registerCleanupFunction(async function () {
     if (testCleanup) {
       await testCleanup();
       testCleanup = null;
     }
 
     for (let addon of await AddonManager.getAllAddons()) {
-      // Builtin search extensions may have been installed by SearchService
-      // during the test run, ignore those.
-      if (
-        !existingAddons.has(addon.id) &&
-        !(addon.isBuiltin && addon.id.endsWith("@search.mozilla.org"))
-      ) {
+      if (!existingAddons.has(addon.id)) {
         ok(
           false,
           `Addon ${addon.id} was left installed at the end of the test`
@@ -681,26 +711,3 @@ add_task(async function() {
     }
   });
 });
-
-let collectedTelemetry = [];
-function hookExtensionsTelemetry() {
-  let originalHistogram = ExtensionsUI.histogram;
-  ExtensionsUI.histogram = {
-    add(value) {
-      collectedTelemetry.push(value);
-    },
-  };
-  registerCleanupFunction(() => {
-    is(
-      collectedTelemetry.length,
-      0,
-      "No unexamined telemetry after test is finished"
-    );
-    ExtensionsUI.histogram = originalHistogram;
-  });
-}
-
-function expectTelemetry(values) {
-  Assert.deepEqual(values, collectedTelemetry);
-  collectedTelemetry = [];
-}
