@@ -2,26 +2,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "AppMenuNotifications",
-  "resource://gre/modules/AppMenuNotifications.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "NewTabUtils",
-  "resource://gre/modules/NewTabUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "PanelMultiView",
-  "resource:///modules/PanelMultiView.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "ToolbarPanelHub",
-  "resource://activity-stream/lib/ToolbarPanelHub.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  AppMenuNotifications: "resource://gre/modules/AppMenuNotifications.sys.mjs",
+  ASRouter: "resource:///modules/asrouter/ASRouter.sys.mjs",
+  MenuMessage: "resource:///modules/asrouter/MenuMessage.sys.mjs",
+  NewTabUtils: "resource://gre/modules/NewTabUtils.sys.mjs",
+  PanelMultiView: "resource:///modules/PanelMultiView.sys.mjs",
+});
 
 /**
  * Maintains the state and dispatches events for the main menu panel.
@@ -32,6 +19,12 @@ const PanelUI = {
   get kEvents() {
     return ["popupshowing", "popupshown", "popuphiding", "popuphidden"];
   },
+
+  /// Notification events used for overwriting notification actions
+  get kNotificationEvents() {
+    return ["buttoncommand", "secondarybuttoncommand", "learnmoreclick"];
+  },
+
   /**
    * Used for lazily getting and memoizing elements from the document. Lazy
    * getters are set in init, and memoizing happens after the first retrieval.
@@ -103,7 +96,7 @@ const PanelUI = {
     this.overflowFixedList.hidden = false;
     // Also unhide the separator. We use CSS to hide/show it based on the panel's content.
     this.overflowFixedList.previousElementSibling.hidden = false;
-    CustomizableUI.registerMenuPanel(
+    CustomizableUI.registerPanelNode(
       this.overflowFixedList,
       CustomizableUI.AREA_FIXED_OVERFLOW_PANEL
     );
@@ -123,7 +116,7 @@ const PanelUI = {
       // Need to do fresh let-bindings per iteration
       let getKey = k;
       let id = v;
-      this.__defineGetter__(getKey, function() {
+      this.__defineGetter__(getKey, function () {
         delete this[getKey];
         return (this[getKey] = document.getElementById(id));
       });
@@ -143,10 +136,16 @@ const PanelUI = {
       this.panel.addEventListener(event, this);
     }
 
-    PanelMultiView.getViewNode(document, "PanelUI-helpView").addEventListener(
-      "ViewShowing",
-      this._onHelpViewShow
-    );
+    let helpView = PanelMultiView.getViewNode(document, "PanelUI-helpView");
+    helpView.addEventListener("ViewShowing", this._onHelpViewShow);
+    helpView.addEventListener("command", this._onHelpCommand);
+    this._onLibraryCommand = this._onLibraryCommand.bind(this);
+    PanelMultiView.getViewNode(
+      document,
+      "appMenu-libraryView"
+    ).addEventListener("command", this._onLibraryCommand);
+    this.mainView.addEventListener("command", this);
+    this.mainView.addEventListener("ViewShowing", this._onMainViewShow);
     this._eventListenersAdded = true;
   },
 
@@ -154,10 +153,14 @@ const PanelUI = {
     for (let event of this.kEvents) {
       this.panel.removeEventListener(event, this);
     }
+    let helpView = PanelMultiView.getViewNode(document, "PanelUI-helpView");
+    helpView.removeEventListener("ViewShowing", this._onHelpViewShow);
+    helpView.removeEventListener("command", this._onHelpCommand);
     PanelMultiView.getViewNode(
       document,
-      "PanelUI-helpView"
-    ).removeEventListener("ViewShowing", this._onHelpViewShow);
+      "appMenu-libraryView"
+    ).removeEventListener("command", this._onLibraryCommand);
+    this.mainView.removeEventListener("command", this);
     this._eventListenersAdded = false;
   },
 
@@ -168,6 +171,9 @@ const PanelUI = {
       for (let event of this.kEvents) {
         this.notificationPanel.removeEventListener(event, this);
       }
+      for (let event of this.kNotificationEvents) {
+        this.notificationPanel.removeEventListener(event, this);
+      }
     }
 
     Services.obs.removeObserver(this, "fullscreen-nav-toolbox");
@@ -181,9 +187,6 @@ const PanelUI = {
     this.menuButton.removeEventListener("mousedown", this);
     this.menuButton.removeEventListener("keypress", this);
     CustomizableUI.removeListener(this);
-    if (this.whatsNewPanel) {
-      this.whatsNewPanel.removeEventListener("ViewShowing", this);
-    }
   },
 
   /**
@@ -225,6 +228,14 @@ const PanelUI = {
         return;
       }
 
+      if (ASRouter.initialized) {
+        await ASRouter.sendTriggerMessage({
+          browser: gBrowser.selectedBrowser,
+          id: "menuOpened",
+          context: { source: MenuMessage.SOURCES.APP_MENU },
+        });
+      }
+
       let domEvent = null;
       if (aEvent && aEvent.type != "command") {
         domEvent = aEvent;
@@ -234,7 +245,7 @@ const PanelUI = {
       await PanelMultiView.openPopup(this.panel, anchor, {
         triggerEvent: domEvent,
       });
-    })().catch(Cu.reportError);
+    })().catch(console.error);
   },
 
   /**
@@ -293,6 +304,7 @@ const PanelUI = {
         this._updatePanelButton(aEvent.target);
         if (aEvent.type == "popuphidden") {
           CustomizableUI.removePanelCloseListeners(this.panel);
+          MenuMessage.hideAppMenuMessage(gBrowser.selectedBrowser);
         }
         break;
       case "mousedown":
@@ -317,10 +329,60 @@ const PanelUI = {
       case "activate":
         this.updateNotifications();
         break;
-      case "ViewShowing":
-        if (aEvent.target == this.whatsNewPanel) {
-          this.onWhatsNewPanelShowing();
-        }
+      case "command":
+        this.onCommand(aEvent);
+        break;
+      case "buttoncommand":
+        this._onNotificationButtonEvent(aEvent, "buttoncommand");
+        break;
+      case "secondarybuttoncommand":
+        this._onNotificationButtonEvent(aEvent, "secondarybuttoncommand");
+        break;
+      case "learnmoreclick":
+        // Don't fall back to PopupNotifications.
+        aEvent.preventDefault();
+        break;
+    }
+  },
+
+  // Note that we listen for bubbling command events. In the case where the
+  // button that the user clicks has a command attribute, those events are
+  // redirected to the relevant command element, and we never see them in
+  // here. Bear this in mind if you want to write code that applies to
+  // all commands, for which this wouldn't work well.
+  onCommand(aEvent) {
+    let { target } = aEvent;
+    switch (target.id) {
+      case "appMenu-update-banner":
+        this._onBannerItemSelected(aEvent);
+        break;
+      case "appMenu-fxa-label2":
+        gSync.toggleAccountPanel(target, aEvent);
+        break;
+      case "appMenu-bookmarks-button":
+        BookmarkingUI.showSubView(target);
+        break;
+      case "appMenu-history-button":
+        this.showSubView("PanelUI-history", target);
+        break;
+      case "appMenu-passwords-button":
+        LoginHelper.openPasswordManager(window, { entryPoint: "Mainmenu" });
+        break;
+      case "appMenu-fullscreen-button2":
+        // Note that we're custom-handling the hiding of the panel to make
+        // sure it disappears before entering fullscreen. Otherwise it can
+        // end up moving around on the screen during the fullscreen transition.
+        target.closest("panel").hidePopup();
+        setTimeout(() => BrowserCommands.fullScreen(), 0);
+        break;
+      case "appMenu-settings-button":
+        openPreferences();
+        break;
+      case "appMenu-more-button2":
+        this.showMoreToolsPanel(target);
+        break;
+      case "appMenu-help-button2":
+        this.showSubView("PanelUI-helpView", target);
         break;
     }
   },
@@ -391,7 +453,6 @@ const PanelUI = {
    * @param aEvent the event triggering the view showing.
    */
   async showSubView(aViewId, aAnchor, aEvent) {
-    let domEvent = null;
     if (aEvent) {
       // On Mac, ctrl-click will send a context menu event from the widget, so
       // we don't want to bring up the panel when ctrl key is pressed.
@@ -409,53 +470,28 @@ const PanelUI = {
       ) {
         return;
       }
-      if (aEvent.type == "command" && aEvent.inputSource != null) {
-        // Synthesize a new DOM mouse event to pass on the inputSource.
-        domEvent = document.createEvent("MouseEvent");
-        domEvent.initNSMouseEvent(
-          "click",
-          true,
-          true,
-          null,
-          0,
-          aEvent.screenX,
-          aEvent.screenY,
-          0,
-          0,
-          false,
-          false,
-          false,
-          false,
-          0,
-          aEvent.target,
-          0,
-          aEvent.inputSource
-        );
-      } else if (aEvent.mozInputSource != null || aEvent.type == "keypress") {
-        domEvent = aEvent;
-      }
     }
 
     this._ensureEventListenersAdded();
 
     let viewNode = PanelMultiView.getViewNode(document, aViewId);
     if (!viewNode) {
-      Cu.reportError("Could not show panel subview with id: " + aViewId);
+      console.error("Could not show panel subview with id: ", aViewId);
       return;
     }
 
     if (!aAnchor) {
-      Cu.reportError(
-        "Expected an anchor when opening subview with id: " + aViewId
+      console.error(
+        "Expected an anchor when opening subview with id: ",
+        aViewId
       );
       return;
     }
 
-    this.ensureWhatsNewInitialized(viewNode);
     this.ensurePanicViewInitialized(viewNode);
 
     let container = aAnchor.closest("panelmultiview");
-    if (container) {
+    if (container && !viewNode.hasAttribute("disallowSubView")) {
       container.showSubView(aViewId, aAnchor);
     } else if (!aAnchor.open) {
       aAnchor.open = true;
@@ -492,7 +528,11 @@ const PanelUI = {
       viewNode.classList.add("cui-widget-panelview", "PanelUI-subView");
 
       let viewShown = false;
-      let panelRemover = () => {
+      let panelRemover = event => {
+        // Avoid bubbled events triggering the panel closing.
+        if (event && event.target != tempPanel) {
+          return;
+        }
         viewNode.classList.remove("cui-widget-panelview");
         if (viewShown) {
           CustomizableUI.removePanelCloseListeners(tempPanel);
@@ -515,11 +555,11 @@ const PanelUI = {
 
       try {
         viewShown = await PanelMultiView.openPopup(tempPanel, anchor, {
-          position: "bottomcenter topright",
-          triggerEvent: domEvent,
+          position: "bottomright topright",
+          triggerEvent: aEvent,
         });
       } catch (ex) {
-        Cu.reportError(ex);
+        console.error(ex);
       }
 
       if (viewShown) {
@@ -531,24 +571,6 @@ const PanelUI = {
     }
   },
 
-  /**
-   * Sets up the event listener for when the What's New panel is shown.
-   *
-   * @param {panelview} panelView The What's New panelview.
-   */
-  ensureWhatsNewInitialized(panelView) {
-    if (panelView.id != "PanelUI-whatsNew" || panelView._initialized) {
-      return;
-    }
-
-    if (!this.whatsNewPanel) {
-      this.whatsNewPanel = panelView;
-    }
-
-    panelView._initialized = true;
-    panelView.addEventListener("ViewShowing", this);
-  },
-
   /**
    * Adds FTL before appending the panic view markup to the main DOM.
    *
@@ -567,17 +589,6 @@ const PanelUI = {
     panelView._initialized = true;
   },
 
-  /**
-   * When the What's New panel is showing, we fetch the messages to show.
-   */
-  onWhatsNewPanelShowing() {
-    ToolbarPanelHub.renderMessages(
-      window,
-      document,
-      "PanelUI-whatsNew-message-container"
-    );
-  },
-
   /**
    * NB: The enable- and disableSingleSubviewPanelAnimations methods only
    * affect the hiding/showing animations of single-subview panels (tempPanel
@@ -603,7 +614,7 @@ const PanelUI = {
     }
   },
 
-  onWidgetAfterDOMChange(aNode, aNextNode, aContainer, aWasRemoval) {
+  onWidgetAfterDOMChange(aNode, aNextNode, aContainer) {
     if (aContainer == this.overflowFixedList) {
       this.updateOverflowStatus();
     }
@@ -636,21 +647,29 @@ const PanelUI = {
     }
   },
 
-  _onHelpViewShow(aEvent) {
+  _onMainViewShow(event) {
+    let panelview = event.target;
+    let messageId = panelview.getAttribute(
+      MenuMessage.SHOWING_FXA_MENU_MESSAGE_ATTR
+    );
+    if (messageId) {
+      MenuMessage.recordMenuMessageTelemetry(
+        "IMPRESSION",
+        MenuMessage.SOURCES.APP_MENU,
+        messageId
+      );
+      let message = ASRouter.getMessageById(messageId);
+      ASRouter.addImpression(message);
+    }
+  },
+
+  _onHelpViewShow() {
     // Call global menu setup function
     buildHelpMenu();
 
     let helpMenu = document.getElementById("menu_HelpPopup");
     let items = this.getElementsByTagName("vbox")[0];
-    let attrs = [
-      "command",
-      "oncommand",
-      "onclick",
-      "key",
-      "disabled",
-      "accesskey",
-      "label",
-    ];
+    let attrs = ["command", "onclick", "key", "disabled", "accesskey", "label"];
 
     // Remove all buttons from the view
     while (items.firstChild) {
@@ -679,7 +698,7 @@ const PanelUI = {
       // their localization IDs are set on "appmenu-data-l10n-id" attributes.
       let l10nId = node.getAttribute("appmenu-data-l10n-id");
       if (l10nId) {
-        button.setAttribute("data-l10n-id", l10nId);
+        document.l10n.setAttributes(button, l10nId);
       }
 
       if (node.id) {
@@ -706,6 +725,63 @@ const PanelUI = {
     items.appendChild(fragment);
   },
 
+  _onHelpCommand(aEvent) {
+    switch (aEvent.target.id) {
+      case "appMenu_menu_openHelp":
+        openHelpLink("firefox-help");
+        break;
+      case "appMenu_menu_layout_debugger":
+        toOpenWindowByType(
+          "mozapp:layoutdebug",
+          "chrome://layoutdebug/content/layoutdebug.xhtml"
+        );
+        break;
+      case "appMenu_feedbackPage":
+        openFeedbackPage();
+        break;
+      case "appMenu_helpSafeMode":
+        safeModeRestart();
+        break;
+      case "appMenu_troubleShooting":
+        openTroubleshootingPage();
+        break;
+      case "appMenu_menu_HelpPopup_reportPhishingtoolmenu":
+        openUILink(gSafeBrowsing.getReportURL("Phish"), aEvent, {
+          triggeringPrincipal:
+            Services.scriptSecurityManager.createNullPrincipal({}),
+        });
+        break;
+      case "appMenu_menu_HelpPopup_reportPhishingErrortoolmenu":
+        gSafeBrowsing.reportFalseDeceptiveSite();
+        break;
+      case "appMenu_helpSwitchDevice":
+        openSwitchingDevicesPage();
+        break;
+      case "appMenu_aboutName":
+        openAboutDialog();
+        break;
+      case "appMenu_helpPolicySupport":
+        openTrustedLinkIn(Services.policies.getSupportMenu().URL.href, "tab");
+        break;
+    }
+  },
+
+  _onLibraryCommand(aEvent) {
+    let button = aEvent.target;
+    let { BookmarkingUI, DownloadsPanel } = button.ownerGlobal;
+    switch (button.id) {
+      case "appMenu-library-bookmarks-button":
+        BookmarkingUI.showSubView(button);
+        break;
+      case "appMenu-library-history-button":
+        this.showSubView("PanelUI-history", button);
+        break;
+      case "appMenu-library-downloads-button":
+        DownloadsPanel.showDownloadsHistory();
+        break;
+    }
+  },
+
   _hidePopup() {
     if (!this._notificationPanel) {
       return;
@@ -881,7 +957,7 @@ const PanelUI = {
         el.removeAttribute("data-lazy-l10n-id");
       });
 
-    this.notificationPanel.openPopup(anchor, "bottomcenter topright");
+    this.notificationPanel.openPopup(anchor, "bottomright topright");
   },
 
   _clearNotificationPanel() {
@@ -908,16 +984,16 @@ const PanelUI = {
       for (let event of this.kEvents) {
         this._notificationPanel.addEventListener(event, this);
       }
+      for (let event of this.kNotificationEvents) {
+        this._notificationPanel.addEventListener(event, this);
+      }
     }
     return this._notificationPanel;
   },
 
   get mainView() {
     if (!this._mainView) {
-      this._mainView = PanelMultiView.getViewNode(
-        document,
-        "appMenu-protonMainView"
-      );
+      this._mainView = PanelMultiView.getViewNode(document, "appMenu-mainView");
     }
     return this._mainView;
   },
@@ -926,7 +1002,7 @@ const PanelUI = {
     if (!this._addonNotificationContainer) {
       this._addonNotificationContainer = PanelMultiView.getViewNode(
         document,
-        "appMenu-proton-addon-banners"
+        "appMenu-addon-banners"
       );
     }
 
@@ -949,14 +1025,6 @@ const PanelUI = {
     let popupnotification = document.getElementById(popupnotificationID);
 
     popupnotification.setAttribute("id", popupnotificationID);
-    popupnotification.setAttribute(
-      "buttoncommand",
-      "PanelUI._onNotificationButtonEvent(event, 'buttoncommand');"
-    );
-    popupnotification.setAttribute(
-      "secondarybuttoncommand",
-      "PanelUI._onNotificationButtonEvent(event, 'secondarybuttoncommand');"
-    );
 
     if (notification.options.message) {
       let desc = this._formatDescriptionMessage(notification);
@@ -971,6 +1039,12 @@ const PanelUI = {
       popupnotification.setAttribute("icon", notification.options.popupIconURL);
       popupnotification.setAttribute("hasicon", true);
     }
+    if (notification.options.learnMoreURL) {
+      popupnotification.setAttribute(
+        "learnmoreurl",
+        notification.options.learnMoreURL
+      );
+    }
 
     popupnotification.notification = notification;
     popupnotification.show();
@@ -999,8 +1073,18 @@ const PanelUI = {
       this._panelBannerItem = this.mainView.querySelector(".panel-banner-item");
     }
 
-    let l10nId = "appmenuitem-banner-" + notification.id;
-    document.l10n.setAttributes(this._panelBannerItem, l10nId);
+    const messageIDs = {
+      "update-downloading": "appmenuitem-banner-update-downloading",
+      "update-available": "appmenuitem-banner-update-available",
+      "update-manual": "appmenuitem-banner-update-manual",
+      "update-unsupported": "appmenuitem-banner-update-unsupported",
+      "update-restart": "appmenuitem-banner-update-restart",
+    };
+
+    document.l10n.setAttributes(
+      this._panelBannerItem,
+      messageIDs[notification.id]
+    );
 
     this._panelBannerItem.setAttribute("notificationid", notification.id);
     this._panelBannerItem.hidden = false;
@@ -1019,6 +1103,8 @@ const PanelUI = {
   },
 
   _onNotificationButtonEvent(event, type) {
+    event.preventDefault();
+
     let notificationEl = getNotificationFromElement(event.originalTarget);
 
     if (!notificationEl) {
