@@ -4,13 +4,12 @@
 
 "use strict";
 
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+const { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  schema: "resource:///modules/policies/schema.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  schema: "resource:///modules/policies/schema.sys.mjs",
 });
 
 function col(text, className) {
@@ -26,8 +25,7 @@ function col(text, className) {
 function link(text) {
   let column = document.createElement("td");
   let a = document.createElement("a");
-  a.href =
-    "https://github.com/mozilla/policy-templates/blob/master/README.md#" + text;
+  a.href = "https://mozilla.github.io/policy-templates/#" + text.toLowerCase();
   a.target = "_blank";
   let content = document.createTextNode(text);
   a.appendChild(content);
@@ -251,14 +249,14 @@ function generateErrors() {
   const consoleEvents = storage.getEvents();
   const prefixes = [
     "Enterprise Policies",
-    "JsonSchemaValidator.jsm",
-    "Policies.jsm",
-    "GPOParser.jsm",
+    "JsonSchemaValidator",
+    "Policies",
+    "WindowsGPOParser",
     "Enterprise Policies Child",
-    "BookmarksPolicies.jsm",
-    "ProxyPolicies.jsm",
+    "BookmarksPolicies",
+    "ProxyPolicies",
     "WebsiteFilter Policy",
-    "macOSPoliciesParser.jsm",
+    "macOSPoliciesParser",
   ];
 
   let new_cont = document.getElementById("errorsContent");
@@ -275,7 +273,7 @@ function generateErrors() {
   }
   if (!flag) {
     let errors_tab = document.getElementById("category-errors");
-    errors_tab.style.display = "none";
+    errors_tab.hidden = true;
   }
 }
 
@@ -286,17 +284,24 @@ function generateDocumentation() {
   // map specific policies to a different string ID, to allow updates to
   // existing descriptions
   let string_mapping = {
-    DisableSetDesktopBackground: "DisableSetAsDesktopBackground",
+    BackgroundAppUpdate: "BackgroundAppUpdate2",
     Certificates: "CertificatesDescription",
-    SanitizeOnShutdown: "SanitizeOnShutdown2",
+    DisableFirefoxAccounts: "DisableFirefoxAccounts1",
+    DisableMasterPasswordCreation: "DisablePrimaryPasswordCreation",
+    DisablePocket: "DisablePocket2",
+    DisableSetDesktopBackground: "DisableSetAsDesktopBackground",
+    FirefoxHome: "FirefoxHome2",
     Permissions: "Permissions2",
-    PrimaryPassword: "MasterPassword",
+    SanitizeOnShutdown: "SanitizeOnShutdown2",
+    SecurityDevices: "SecurityDevices2",
+    SkipTermsOfUse: "SkipTermsOfUse2",
+    WindowsSSO: "Windows10SSO",
   };
 
   for (let policyName in schema.properties) {
     let main_tbody = document.createElement("tbody");
     main_tbody.classList.add("collapsible");
-    main_tbody.addEventListener("click", function() {
+    main_tbody.addEventListener("click", function () {
       let content = this.nextElementSibling;
       content.classList.toggle("content");
     });
@@ -304,7 +309,7 @@ function generateDocumentation() {
     row.appendChild(link(policyName));
     let descriptionColumn = col("");
     let stringID = string_mapping[policyName] || policyName;
-    descriptionColumn.setAttribute("data-l10n-id", `policy-${stringID}`);
+    document.l10n.setAttributes(descriptionColumn, `policy-${stringID}`);
     row.appendChild(descriptionColumn);
     main_tbody.appendChild(row);
     let sec_tbody = document.createElement("tbody");
@@ -350,7 +355,7 @@ function generateDocumentation() {
 }
 
 let gInited = false;
-window.onload = function() {
+window.onload = function () {
   if (gInited) {
     return;
   }
@@ -361,11 +366,9 @@ window.onload = function() {
   generateErrors();
   generateDocumentation();
 
-  // Event delegation on #categories element
-  let menu = document.getElementById("categories");
-  for (let category of menu.children) {
-    category.addEventListener("click", () => show(category));
-  }
+  // Event delegation on #categories-nav element
+  let menu = document.getElementById("categories-nav");
+  menu.addEventListener("change-view", e => show(e.target));
 
   if (location.hash) {
     let sectionButton = document.getElementById(
@@ -376,7 +379,7 @@ window.onload = function() {
     }
   }
 
-  window.addEventListener("hashchange", function() {
+  window.addEventListener("hashchange", function () {
     if (location.hash) {
       let sectionButton = document.getElementById(
         "category-" + location.hash.substring(1)
@@ -399,10 +402,6 @@ function show(button) {
   content.classList.add("active");
   content.hidden = false;
 
-  let current_button = document.querySelector("[selected=true]");
-  current_button.removeAttribute("selected");
-  button.setAttribute("selected", "true");
-
   let title = document.getElementById("sectionTitle");
   title.textContent = button.textContent;
   location.hash = category;
