@@ -6,21 +6,17 @@
 
 "use strict";
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "BrowserUIUtils",
-  "resource:///modules/BrowserUIUtils.jsm"
-);
 ChromeUtils.defineESModuleGetters(this, {
+  BrowserUIUtils: "resource:///modules/BrowserUIUtils.sys.mjs",
+  CustomizableUI: "resource:///modules/CustomizableUI.sys.mjs",
   DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
   ExtensionControlledPopup:
     "resource:///modules/ExtensionControlledPopup.sys.mjs",
   PrivateBrowsingUtils: "resource://gre/modules/PrivateBrowsingUtils.sys.mjs",
-  PromiseUtils: "resource://gre/modules/PromiseUtils.sys.mjs",
   SessionStore: "resource:///modules/sessionstore/SessionStore.sys.mjs",
 });
 
-XPCOMUtils.defineLazyGetter(this, "strBundle", function () {
+ChromeUtils.defineLazyGetter(this, "strBundle", function () {
   return Services.strings.createBundle(
     "chrome://global/locale/extensions.properties"
   );
@@ -32,16 +28,18 @@ const TAB_HIDE_CONFIRMED_TYPE = "tabHideNotification";
 
 const TAB_ID_NONE = -1;
 
-XPCOMUtils.defineLazyGetter(this, "tabHidePopup", () => {
+ChromeUtils.defineLazyGetter(this, "tabHidePopup", () => {
   return new ExtensionControlledPopup({
     confirmedType: TAB_HIDE_CONFIRMED_TYPE,
-    anchorId: "alltabs-button",
     popupnotificationId: "extension-tab-hide-notification",
     descriptionId: "extension-tab-hide-notification-description",
     descriptionMessageId: "tabHideControlled.message",
     getLocalizedDescription: (doc, message, addonDetails) => {
       let image = doc.createXULElement("image");
-      image.setAttribute("class", "extension-controlled-icon alltabs-icon");
+      image.classList.add("extension-controlled-icon", "alltabs-icon");
+      if (!doc.getElementById("alltabs-button")?.closest("#TabsToolbar")) {
+        image.classList.add("alltabs-icon-generic");
+      }
       return BrowserUIUtils.getLocalizedFragment(
         doc,
         message,
@@ -49,7 +47,6 @@ XPCOMUtils.defineLazyGetter(this, "tabHidePopup", () => {
         image
       );
     },
-    learnMoreMessageId: "tabHideControlled.learnMore",
     learnMoreLink: "extension-hiding-tabs",
   });
 });
@@ -88,7 +85,7 @@ let tabListener = {
     }
   },
 
-  onLocationChange(browser, webProgress, request, locationURI, flags) {
+  onLocationChange(browser, webProgress) {
     if (webProgress.isTopLevel) {
       let { gBrowser } = browser.ownerGlobal;
       let nativeTab = gBrowser.getTabForBrowser(browser);
@@ -132,7 +129,7 @@ let tabListener = {
       if (promise) {
         return promise;
       }
-      deferred = PromiseUtils.defer();
+      deferred = Promise.withResolvers();
       if (
         !this.initializingTabs.has(nativeTab) &&
         (nativeTab.linkedBrowser.innerWindowID ||
@@ -155,15 +152,19 @@ const allAttrs = new Set([
   "mutedInfo",
   "sharingState",
   "title",
+  "autoDiscardable",
 ]);
 const allProperties = new Set([
   "attention",
   "audible",
+  "autoDiscardable",
   "discarded",
   "favIconUrl",
+  "groupId",
   "hidden",
   "isArticle",
   "mutedInfo",
+  "openerTabId",
   "pinned",
   "sharingState",
   "status",
@@ -260,13 +261,17 @@ this.tabs = class extends ExtensionAPIPersistent {
     }),
     onMoved({ fire }) {
       let { tabManager } = this.extension;
+      /**
+       * @param {CustomEvent} event
+       */
       let moveListener = event => {
         let nativeTab = event.originalTarget;
+        let { previousTabState, currentTabState } = event.detail;
         if (tabManager.canAccessTab(nativeTab)) {
           fire.async(tabTracker.getId(nativeTab), {
             windowId: windowTracker.getId(nativeTab.ownerGlobal),
-            fromIndex: event.detail,
-            toIndex: nativeTab._tPos,
+            fromIndex: previousTabState.tabIndex,
+            toIndex: currentTabState.tabIndex,
           });
         }
       };
@@ -360,7 +365,7 @@ this.tabs = class extends ExtensionAPIPersistent {
         return windowId;
       }
 
-      function matchFilters(tab, changed) {
+      function matchFilters(tab) {
         if (!filterProps) {
           return true;
         }
@@ -400,7 +405,10 @@ this.tabs = class extends ExtensionAPIPersistent {
       let listener = event => {
         // Ignore any events prior to TabOpen
         // and events that are triggered while tabs are swapped between windows.
-        if (event.originalTarget.initializingTab) {
+        if (
+          event.originalTarget.initializingTab ||
+          event.originalTarget.ownerGlobal.gBrowserInit?.isAdoptingTab()
+        ) {
           return;
         }
         if (!extension.canAccessWindow(event.originalTarget.ownerGlobal)) {
@@ -424,6 +432,12 @@ this.tabs = class extends ExtensionAPIPersistent {
           ) {
             needed.push("audible");
           }
+          if (
+            changed.includes("undiscardable") &&
+            filter.properties.has("autoDiscardable")
+          ) {
+            needed.push("autoDiscardable");
+          }
           if (changed.includes("label") && filter.properties.has("title")) {
             needed.push("title");
           }
@@ -452,6 +466,15 @@ this.tabs = class extends ExtensionAPIPersistent {
           needed.push("discarded");
         } else if (event.type == "TabBrowserDiscarded") {
           needed.push("discarded");
+        } else if (event.type === "TabGrouped") {
+          needed.push("groupId");
+        } else if (event.type === "TabUngrouped") {
+          if (event.originalTarget.group) {
+            // If there is still a group, that means that the group changed,
+            // so TabGrouped will also fire. Ignore to avoid duplicate events.
+            return;
+          }
+          needed.push("groupId");
         } else if (event.type == "TabShow") {
           needed.push("hidden");
         } else if (event.type == "TabHide") {
@@ -498,6 +521,11 @@ this.tabs = class extends ExtensionAPIPersistent {
         }
       };
 
+      let openerTabIdChangeListener = (_, { nativeTab, openerTabId }) => {
+        let tab = tabManager.getWrapper(nativeTab);
+        fireForTab(tab, { openerTabId }, nativeTab);
+      };
+
       let listeners = new Map();
       if (filter.properties.has("status") || filter.properties.has("url")) {
         listeners.set("status", statusListener);
@@ -513,6 +541,10 @@ this.tabs = class extends ExtensionAPIPersistent {
         listeners.set("TabBrowserInserted", listener);
         listeners.set("TabBrowserDiscarded", listener);
       }
+      if (filter.properties.has("groupId")) {
+        listeners.set("TabGrouped", listener);
+        listeners.set("TabUngrouped", listener);
+      }
       if (filter.properties.has("hidden")) {
         listeners.set("TabShow", listener);
         listeners.set("TabHide", listener);
@@ -526,6 +558,10 @@ this.tabs = class extends ExtensionAPIPersistent {
         tabTracker.on("tab-isarticle", isArticleChangeListener);
       }
 
+      if (filter.properties.has("openerTabId")) {
+        tabTracker.on("tab-openerTabId", openerTabIdChangeListener);
+      }
+
       return {
         unregister() {
           for (let [name, listener] of listeners) {
@@ -535,6 +571,10 @@ this.tabs = class extends ExtensionAPIPersistent {
           if (filter.properties.has("isArticle")) {
             tabTracker.off("tab-isarticle", isArticleChangeListener);
           }
+
+          if (filter.properties.has("openerTabId")) {
+            tabTracker.off("tab-openerTabId", openerTabIdChangeListener);
+          }
         },
         convert(_fire, _context) {
           fire = _fire;
@@ -658,7 +698,7 @@ this.tabs = class extends ExtensionAPIPersistent {
         onReplaced: new EventManager({
           context,
           name: "tabs.onReplaced",
-          register: fire => {
+          register: () => {
             return () => {};
           },
         }).api(),
@@ -678,7 +718,7 @@ this.tabs = class extends ExtensionAPIPersistent {
         }).api(),
 
         create(createProperties) {
-          return new Promise((resolve, reject) => {
+          return new Promise(resolve => {
             let window =
               createProperties.windowId !== null
                 ? windowTracker.getWindow(createProperties.windowId, context)
@@ -690,7 +730,7 @@ this.tabs = class extends ExtensionAPIPersistent {
             }
             let { gBrowserInit } = window;
             if (!gBrowserInit || !gBrowserInit.delayedStartupFinished) {
-              let obs = (finishedWindow, topic, data) => {
+              let obs = finishedWindow => {
                 if (finishedWindow != window) {
                   return;
                 }
@@ -903,6 +943,9 @@ this.tabs = class extends ExtensionAPIPersistent {
           if (updateProperties.active) {
             tabbrowser.selectedTab = nativeTab;
           }
+          if (updateProperties.autoDiscardable !== null) {
+            nativeTab.undiscardable = !updateProperties.autoDiscardable;
+          }
           if (updateProperties.highlighted !== null) {
             if (updateProperties.highlighted) {
               if (!nativeTab.selected && !nativeTab.multiselected) {
@@ -931,14 +974,7 @@ this.tabs = class extends ExtensionAPIPersistent {
             }
           }
           if (updateProperties.openerTabId !== null) {
-            let opener = tabTracker.getTab(updateProperties.openerTabId);
-            if (opener.ownerDocument !== nativeTab.ownerDocument) {
-              return Promise.reject({
-                message:
-                  "Opener tab must be in the same window as the tab being updated",
-              });
-            }
-            tabTracker.setOpener(nativeTab, opener);
+            tabTracker.setOpener(nativeTab, updateProperties.openerTabId);
           }
           if (updateProperties.successorTabId !== null) {
             let successor = null;
@@ -1018,7 +1054,13 @@ this.tabs = class extends ExtensionAPIPersistent {
               ? windowTracker.getTopWindow(context)
               : windowTracker.getWindow(windowId, context);
 
-          let tab = tabManager.wrapTab(window.gBrowser.selectedTab);
+          let tab = tabManager.getWrapper(window.gBrowser.selectedTab);
+          if (
+            !extension.hasPermission("<all_urls>") &&
+            !tab.hasActiveTabPermission
+          ) {
+            throw new ExtensionError("Missing activeTab permission");
+          }
           await tabListener.awaitTabReady(tab.nativeTab);
 
           let zoom = window.ZoomManager.getZoomForBrowser(
@@ -1129,7 +1171,7 @@ this.tabs = class extends ExtensionAPIPersistent {
             // the current set of pinned tabs. Unpinned tabs, likewise, can only
             // be moved to a position after the current set of pinned tabs.
             // Attempts to move a tab to an illegal position are ignored.
-            let numPinned = gBrowser._numPinnedTabs;
+            let numPinned = gBrowser.pinnedTabCount;
             let ok = nativeTab.pinned
               ? insertionPoint <= numPinned
               : insertionPoint >= numPinned;
@@ -1139,11 +1181,13 @@ this.tabs = class extends ExtensionAPIPersistent {
 
             if (isSameWindow) {
               // If the window we are moving is the same, just move the tab.
-              gBrowser.moveTabTo(nativeTab, insertionPoint);
+              gBrowser.moveTabTo(nativeTab, { tabIndex: insertionPoint });
             } else {
               // If the window we are moving the tab in is different, then move the tab
               // to the new window.
-              nativeTab = gBrowser.adoptTab(nativeTab, insertionPoint, false);
+              nativeTab = gBrowser.adoptTab(nativeTab, {
+                tabIndex: insertionPoint,
+              });
             }
             lastInsertionMap.set(window, nativeTab._tPos);
             tabsMoved.push(nativeTab);
@@ -1356,7 +1400,11 @@ this.tabs = class extends ExtensionAPIPersistent {
           }
           filename = DownloadPaths.sanitize(filename);
 
-          picker.init(activeTab.ownerGlobal, title, Ci.nsIFilePicker.modeSave);
+          picker.init(
+            activeTab.ownerGlobal.browsingContext,
+            title,
+            Ci.nsIFilePicker.modeSave
+          );
           picker.appendFilter("PDF", "*.pdf");
           picker.defaultExtension = "pdf";
           picker.defaultString = filename;
@@ -1581,6 +1629,17 @@ this.tabs = class extends ExtensionAPIPersistent {
           }
           if (hidden.length) {
             let win = Services.wm.getMostRecentWindow("navigator:browser");
+
+            // Before showing the hidden tabs warning,
+            // move alltabs-button to somewhere visible if it isn't already.
+            if (!CustomizableUI.widgetIsLikelyVisible("alltabs-button", win)) {
+              CustomizableUI.addWidgetToArea(
+                "alltabs-button",
+                CustomizableUI.verticalTabsEnabled
+                  ? CustomizableUI.AREA_NAVBAR
+                  : CustomizableUI.AREA_TABSTRIP
+              );
+            }
             tabHidePopup.open(win, extension.id);
           }
           return hidden;
@@ -1613,12 +1672,119 @@ this.tabs = class extends ExtensionAPIPersistent {
 
         goForward(tabId) {
           let nativeTab = getTabOrActive(tabId);
-          nativeTab.linkedBrowser.goForward();
+          nativeTab.linkedBrowser.goForward(false);
         },
 
         goBack(tabId) {
           let nativeTab = getTabOrActive(tabId);
-          nativeTab.linkedBrowser.goBack();
+          nativeTab.linkedBrowser.goBack(false);
+        },
+
+        group(options) {
+          let nativeTabs = getNativeTabsFromIDArray(options.tabIds);
+          let window = windowTracker.getWindow(
+            options.createProperties?.windowId ?? Window.WINDOW_ID_CURRENT,
+            context
+          );
+          const windowIsPrivate = PrivateBrowsingUtils.isWindowPrivate(window);
+          for (const nativeTab of nativeTabs) {
+            if (
+              PrivateBrowsingUtils.isWindowPrivate(nativeTab.ownerGlobal) !==
+              windowIsPrivate
+            ) {
+              if (windowIsPrivate) {
+                throw new ExtensionError(
+                  "Cannot move non-private tabs to private window"
+                );
+              }
+              throw new ExtensionError(
+                "Cannot move private tabs to non-private window"
+              );
+            }
+          }
+          function unpinTabsBeforeGrouping() {
+            for (const nativeTab of nativeTabs) {
+              nativeTab.ownerGlobal.gBrowser.unpinTab(nativeTab);
+            }
+          }
+          let group;
+          if (options.groupId == null) {
+            // By default, tabs are appended after all other tabs in the
+            // window. But if we are grouping tabs within a window, ideally the
+            // tabs should just be grouped without moving positions.
+            // TODO bug 1939214: when addTabGroup inserts tabs at the front as
+            // needed (instead of always appending), simplify this logic.
+            const tabInWin = nativeTabs.find(t => t.ownerGlobal === window);
+            let insertBefore = tabInWin;
+            if (tabInWin?.group) {
+              if (tabInWin.group.tabs[0] === tabInWin) {
+                // When tabInWin is at the front of a tab group, insert before
+                // the tab group (instead of after it).
+                insertBefore = tabInWin.group;
+              } else {
+                insertBefore = insertBefore.group.nextElementSibling;
+              }
+            }
+            unpinTabsBeforeGrouping();
+            group = window.gBrowser.addTabGroup(nativeTabs, { insertBefore });
+            // Note: group is never null, because the only condition for which
+            // it could be null is when all tabs are pinned, and we are already
+            // explicitly unpinning them before moving.
+          } else {
+            group = window.gBrowser.getTabGroupById(
+              getInternalTabGroupIdForExtTabGroupId(options.groupId)
+            );
+            if (!group) {
+              throw new ExtensionError(`No group with id: ${options.groupId}`);
+            }
+            unpinTabsBeforeGrouping();
+            // When moving tabs within the same window, try to maintain their
+            // relative positions.
+            const tabsBefore = [];
+            const tabsAfter = [];
+            const firstTabInGroup = group.tabs[0];
+            for (const nativeTab of nativeTabs) {
+              if (
+                nativeTab.ownerGlobal === window &&
+                nativeTab._tPos < firstTabInGroup._tPos
+              ) {
+                tabsBefore.push(nativeTab);
+              } else {
+                tabsAfter.push(nativeTab);
+              }
+            }
+            if (tabsBefore.length) {
+              window.gBrowser.moveTabsBefore(tabsBefore, firstTabInGroup);
+            }
+            if (tabsAfter.length) {
+              group.addTabs(tabsAfter);
+            }
+          }
+          return getExtTabGroupIdForInternalTabGroupId(group.id);
+        },
+
+        ungroup(tabIds) {
+          const nativeTabs = getNativeTabsFromIDArray(tabIds);
+          // Ungroup tabs while trying to preserve the relative order of tabs
+          // within the tab strip as much as possible. This is not always
+          // possible, e.g. when a tab group is only partially ungrouped.
+          const ungroupOrder = new DefaultMap(() => []);
+          for (const nativeTab of nativeTabs) {
+            if (nativeTab.group) {
+              ungroupOrder.get(nativeTab.group).push(nativeTab);
+            }
+          }
+          for (const [group, tabs] of ungroupOrder) {
+            // Preserve original order of ungrouped tabs.
+            tabs.sort((a, b) => a._tPos - b._tPos);
+            if (tabs[0] === tabs[0].group.tabs[0]) {
+              // The tab is the front of the tab group, so insert before
+              // current tab group to preserve order.
+              tabs[0].ownerGlobal.gBrowser.moveTabsBefore(tabs, group);
+            } else {
+              tabs[0].ownerGlobal.gBrowser.moveTabsAfter(tabs, group);
+            }
+          }
         },
       },
     };
