@@ -2,7 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from ../../../base/content/utilityOverlay.js */
+/* import-globals-from /browser/base/content/utilityOverlay.js */
 
 var gSetBackground = {
   _position: AppConstants.platform == "macosx" ? "STRETCH" : "",
@@ -34,6 +34,18 @@ var gSetBackground = {
       document
         .getElementById("SetDesktopBackgroundDialog")
         .getButton("accept").hidden = true;
+
+      document
+        .getElementById("setDesktopBackground")
+        .addEventListener("command", () => this.setDesktopBackground());
+
+      document
+        .getElementById("showDesktopPreferences")
+        .addEventListener("command", () => {
+          this._shell
+            .QueryInterface(Ci.nsIMacShellService)
+            .showDesktopPreferences();
+        });
     } else {
       let multiMonitors = false;
       if (AppConstants.platform == "linux") {
@@ -46,21 +58,27 @@ var gSetBackground = {
         multiMonitors = monitors.length > 1;
       }
 
-      if (
-        !multiMonitors ||
-        AppConstants.isPlatformAndVersionAtMost("win", 6.1)
-      ) {
-        // Hide span option if < Win8 since that's when it was introduced.
+      if (!multiMonitors) {
+        // Hide span option on single monitor systems.
         document.getElementById("spanPosition").hidden = true;
       }
+
+      document
+        .getElementById("menuPosition")
+        .addEventListener("command", () => this.updatePosition());
+      document
+        .getElementById("desktopColor")
+        .addEventListener("change", event =>
+          this.updateColor(event.currentTarget.value)
+        );
     }
 
-    document.addEventListener("dialogaccept", function() {
+    document.addEventListener("dialogaccept", function () {
       gSetBackground.setDesktopBackground();
     });
     // make sure that the correct dimensions will be used
     setTimeout(
-      function(self) {
+      function (self) {
         self.init(window.arguments[0], window.arguments[1]);
       },
       0,
@@ -198,7 +216,7 @@ var gSetBackground = {
 };
 
 if (AppConstants.platform != "macosx") {
-  gSetBackground._initColor = function() {
+  gSetBackground._initColor = function () {
     var color = this._shell.desktopBackgroundColor;
 
     const rMask = 4294901760;
@@ -213,13 +231,13 @@ if (AppConstants.platform != "macosx") {
     colorpicker.value = this._backgroundColor;
   };
 
-  gSetBackground.updateColor = function(aColor) {
+  gSetBackground.updateColor = function (aColor) {
     this._backgroundColor = aColor;
     this._canvas.style.backgroundColor = aColor;
   };
 
   // Converts a color string in the format "#RRGGBB" to an integer.
-  gSetBackground._hexStringToLong = function(aString) {
+  gSetBackground._hexStringToLong = function (aString) {
     return (
       (parseInt(aString.substring(1, 3), 16) << 16) |
       (parseInt(aString.substring(3, 5), 16) << 8) |
@@ -227,7 +245,7 @@ if (AppConstants.platform != "macosx") {
     );
   };
 
-  gSetBackground._rgbToHex = function(aR, aG, aB) {
+  gSetBackground._rgbToHex = function (aR, aG, aB) {
     return (
       "#" +
       [aR, aG, aB]
@@ -237,7 +255,7 @@ if (AppConstants.platform != "macosx") {
     );
   };
 } else {
-  gSetBackground.observe = function(aSubject, aTopic, aData) {
+  gSetBackground.observe = function (aSubject, aTopic) {
     if (aTopic == "shell:desktop-background-changed") {
       document.getElementById("setDesktopBackground").hidden = true;
       document.getElementById("showDesktopPreferences").hidden = false;
@@ -245,8 +263,6 @@ if (AppConstants.platform != "macosx") {
       Services.obs.removeObserver(this, "shell:desktop-background-changed");
     }
   };
-
-  gSetBackground.showDesktopPrefs = function() {
-    this._shell.QueryInterface(Ci.nsIMacShellService).showDesktopPreferences();
-  };
 }
+
+window.addEventListener("load", () => gSetBackground.load());
