@@ -59,8 +59,7 @@ public class ContentBlocking {
               "googpub-phish-proto",
               "goog-malware-proto",
               "goog-unwanted-proto",
-              "goog-harmful-proto",
-              "goog-passwordwhite-proto")
+              "goog-harmful-proto")
           .updateUrl(
               "https://safebrowsing.googleapis.com/v4/threatListUpdates:fetch?$ct=application/x-protobuf&key=%GOOGLE_SAFEBROWSING_API_KEY%&$httpMethod=POST")
           .getHashUrl(
@@ -181,17 +180,6 @@ public class ContentBlocking {
         return this;
       }
 
-      /**
-       * Set the cookie lifetime.
-       *
-       * @param lifetime The enforced cookie lifetime. Use one of the {@link CookieLifetime} flags.
-       * @return The Builder instance.
-       */
-      public @NonNull Builder cookieLifetime(final @CBCookieLifetime int lifetime) {
-        getSettings().setCookieLifetime(lifetime);
-        return this;
-      }
-
       /**
        * Set the ETP behavior level.
        *
@@ -205,6 +193,18 @@ public class ContentBlocking {
         return this;
       }
 
+      /**
+       * Set whether or not email tracker blocking is enabled in private mode.
+       *
+       * @param enabled A boolean indicating whether or not email tracker blocking should be enabled
+       *     in private mode.
+       * @return The builder instance.
+       */
+      public @NonNull Builder emailTrackerBlockingPrivateMode(final boolean enabled) {
+        getSettings().setEmailTrackerBlockingPrivateBrowsing(enabled);
+        return this;
+      }
+
       /**
        * Set whether or not strict social tracking protection is enabled. This will block resources
        * from loading if they are on the social tracking protection list, rather than just blocking
@@ -232,6 +232,111 @@ public class ContentBlocking {
         getSettings().setCookiePurging(enabled);
         return this;
       }
+
+      /**
+       * Set the Cookie Banner Handling Mode.
+       *
+       * @param mode The mode of the Cookie Banner Handling one of the {@link CBCookieBannerMode}.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder cookieBannerHandlingMode(final @CBCookieBannerMode int mode) {
+        getSettings().setCookieBannerMode(mode);
+        return this;
+      }
+
+      /**
+       * When set to true, enable the use of global CookieBannerRules.
+       *
+       * @param enabled A boolean indicating whether to enable the use of global CookieBannerRules.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder cookieBannerGlobalRulesEnabled(final boolean enabled) {
+        getSettings().setCookieBannerGlobalRulesEnabled(enabled);
+        return this;
+      }
+
+      /**
+       * When set to true, enable the use of global CookieBannerRules in sub-frames.
+       *
+       * @param enabled A boolean indicating whether to enable the use of global CookieBannerRules
+       *     in sub-frames.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder cookieBannerGlobalRulesSubFramesEnabled(final boolean enabled) {
+        getSettings().setCookieBannerGlobalRulesSubFramesEnabled(enabled);
+        return this;
+      }
+
+      /**
+       * When set to true, query parameter stripping is enabled in normal mode.
+       *
+       * @param enabled A boolean indicating whether to query parameter stripping enabled in normal
+       *     mode.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder queryParameterStrippingEnabled(final boolean enabled) {
+        getSettings().setQueryParameterStrippingEnabled(enabled);
+        return this;
+      }
+
+      /**
+       * When set to true, query parameter stripping is enabled in private mode.
+       *
+       * @param enabled A boolean indicating whether to query parameter stripping enabled in private
+       *     mode.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder queryParameterStrippingPrivateBrowsingEnabled(final boolean enabled) {
+        getSettings().setQueryParameterStrippingPrivateBrowsingEnabled(enabled);
+        return this;
+      }
+
+      /**
+       * The allowed list for the query parameter stripping feature.
+       *
+       * @param list an array of identifiers for query parameter's stripping feature.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder queryParameterStrippingAllowList(final @NonNull String... list) {
+        getSettings().setQueryParameterStrippingAllowList(list);
+        return this;
+      }
+
+      /**
+       * The strip list for the query parameter stripping feature.
+       *
+       * @param list an array of identifiers for the strip list of the query parameter's stripping
+       *     feature.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder queryParameterStrippingStripList(final @NonNull String... list) {
+        getSettings().setQueryParameterStrippingStripList(list);
+        return this;
+      }
+
+      /**
+       * Set the Cookie Banner Handling Mode for private browsing.
+       *
+       * @param mode The mode of the Cookie Banner Handling one of the {@link CBCookieBannerMode}.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder cookieBannerHandlingModePrivateBrowsing(
+          final @CBCookieBannerMode int mode) {
+        getSettings().setCookieBannerModePrivateBrowsing(mode);
+        return this;
+      }
+
+      /**
+       * When set to true, cookie banners are detected and detection events are dispatched, but they
+       * will not be handled.
+       *
+       * @param enabled A boolean indicating whether to enable cookie banner detect only mode.
+       * @return The Builder instance.
+       */
+      public @NonNull Builder cookieBannerHandlingDetectOnlyMode(final boolean enabled) {
+        getSettings().setCookieBannerDetectOnlyMode(enabled);
+        return this;
+      }
     }
 
     /* package */ final Pref<String> mAt =
@@ -256,19 +361,19 @@ public class ContentBlocking {
     /* package */ final Pref<String> mStList =
         new Pref<String>(
             "urlclassifier.features.socialtracking.annotate.blacklistTables",
-            ContentBlocking.catToStListPref(AntiTracking.NONE));
+            ContentBlocking.catToPref(AntiTracking.NONE, AntiTracking.STP, STP));
 
     /* package */ final Pref<Boolean> mSbMalware =
         new Pref<Boolean>("browser.safebrowsing.malware.enabled", true);
     /* package */ final Pref<Boolean> mSbPhishing =
         new Pref<Boolean>("browser.safebrowsing.phishing.enabled", true);
     /* package */ final Pref<Integer> mCookieBehavior =
-        new Pref<Integer>("network.cookie.cookieBehavior", CookieBehavior.ACCEPT_NON_TRACKERS);
+        new Pref<Integer>(
+            "network.cookie.cookieBehavior", CookieBehavior.ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS);
     /* package */ final Pref<Integer> mCookieBehaviorPrivateMode =
         new Pref<Integer>(
-            "network.cookie.cookieBehavior.pbmode", CookieBehavior.ACCEPT_NON_TRACKERS);
-    /* package */ final Pref<Integer> mCookieLifetime =
-        new Pref<Integer>("network.cookie.lifetimePolicy", CookieLifetime.NORMAL);
+            "network.cookie.cookieBehavior.pbmode",
+            CookieBehavior.ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS);
     /* package */ final Pref<Boolean> mCookiePurging =
         new Pref<Boolean>("privacy.purge_trackers.enabled", false);
 
@@ -277,6 +382,46 @@ public class ContentBlocking {
     /* package */ final Pref<Boolean> mEtpStrict =
         new Pref<Boolean>("privacy.annotate_channels.strict_list.enabled", false);
 
+    /* package */ final Pref<Integer> mCbhMode =
+        new Pref<Integer>(
+            "cookiebanners.service.mode", CookieBannerMode.COOKIE_BANNER_MODE_DISABLED);
+    /* package */ final Pref<Integer> mCbhModePrivateBrowsing =
+        new Pref<Integer>(
+            "cookiebanners.service.mode.privateBrowsing",
+            CookieBannerMode.COOKIE_BANNER_MODE_REJECT);
+
+    /* package */ final Pref<Boolean> mChbDetectOnlyMode =
+        new Pref<Boolean>("cookiebanners.service.detectOnly", false);
+    /* package */
+    final Pref<Boolean> mCbhGlobalRulesEnabled =
+        new Pref<Boolean>("cookiebanners.service.enableGlobalRules", false);
+
+    final Pref<Boolean> mCbhGlobalRulesSubFramesEnabled =
+        new Pref<Boolean>("cookiebanners.service.enableGlobalRules.subFrames", false);
+
+    /* package */ final Pref<Boolean> mQueryParameterStrippingEnabled =
+        new Pref<Boolean>("privacy.query_stripping.enabled", false);
+
+    /* package */ final Pref<Boolean> mQueryParameterStrippingPrivateBrowsingEnabled =
+        new Pref<Boolean>("privacy.query_stripping.enabled.pbmode", false);
+
+    /* package */ final Pref<String> mQueryParameterStrippingAllowList =
+        new Pref<>("privacy.query_stripping.allow_list", "");
+
+    /* package */ final Pref<String> mQueryParameterStrippingStripList =
+        new Pref<>("privacy.query_stripping.strip_list", "");
+
+    /* package */ final Pref<Boolean> mEtb =
+        new Pref<Boolean>("privacy.trackingprotection.emailtracking.enabled", false);
+
+    /* package */ final Pref<Boolean> mEtbPrivateBrowsing =
+        new Pref<Boolean>("privacy.trackingprotection.emailtracking.pbmode.enabled", false);
+
+    /* package */ final Pref<String> mEtbList =
+        new Pref<String>(
+            "urlclassifier.features.emailtracking.blocklistTables",
+            ContentBlocking.catToPref(AntiTracking.NONE, AntiTracking.EMAIL, EMAIL));
+
     /* package */ final Pref<String> mSafeBrowsingMalwareTable =
         new Pref<>(
             "urlclassifier.malwareTable",
@@ -403,6 +548,50 @@ public class ContentBlocking {
       return ContentBlocking.prefToLists(mSafeBrowsingMalwareTable.get());
     }
 
+    /**
+     * Sets the allowed list for the query parameter stripping feature.
+     *
+     * @param list an array of identifiers for the allowed list of the query parameter's stripping
+     *     feature.
+     * @return this {@link Settings} instance.
+     */
+    public @NonNull Settings setQueryParameterStrippingAllowList(final @NonNull String... list) {
+      mQueryParameterStrippingAllowList.commit(ContentBlocking.listsToPref(list));
+      return this;
+    }
+
+    /**
+     * Get the allowed list for the query parameter stripping feature.
+     *
+     * @return an array of identifiers for the allowed list for the query parameter stripping
+     *     feature.
+     */
+    public @NonNull String[] getQueryParameterStrippingAllowList() {
+      return ContentBlocking.prefToLists(mQueryParameterStrippingAllowList.get());
+    }
+
+    /**
+     * Sets the strip list for the query parameter stripping feature.
+     *
+     * @param list an array of identifiers for the strip list of the query parameter's stripping
+     *     feature.
+     * @return this {@link Settings} instance.
+     */
+    public @NonNull Settings setQueryParameterStrippingStripList(final @NonNull String... list) {
+      mQueryParameterStrippingStripList.commit(ContentBlocking.listsToPref(list));
+      return this;
+    }
+
+    /**
+     * Get the strip list for the query parameter stripping feature
+     *
+     * @return an array of identifiers for the allowed list for the query parameter stripping
+     *     feature.
+     */
+    public @NonNull String[] getQueryParameterStrippingStripList() {
+      return ContentBlocking.prefToLists(mQueryParameterStrippingStripList.get());
+    }
+
     /**
      * Sets the table for SafeBrowsing Malware.
      *
@@ -432,7 +621,10 @@ public class ContentBlocking {
       mFpList.commit(ContentBlocking.catToFpListPref(cat));
 
       mSt.commit(ContentBlocking.catToStPref(cat));
-      mStList.commit(ContentBlocking.catToStListPref(cat));
+      mStList.commit(ContentBlocking.catToPref(cat, AntiTracking.STP, STP));
+
+      mEtb.commit(ContentBlocking.catToEtbPref(cat));
+      mEtbList.commit(ContentBlocking.catToPref(cat, AntiTracking.EMAIL, EMAIL));
       return this;
     }
 
@@ -488,7 +680,8 @@ public class ContentBlocking {
       return ContentBlocking.atListToAtCat(mAt.get())
           | ContentBlocking.cmListToAtCat(mCmList.get())
           | ContentBlocking.fpListToAtCat(mFpList.get())
-          | ContentBlocking.stListToAtCat(mStList.get());
+          | ContentBlocking.stListToAtCat(mStList.get())
+          | ContentBlocking.etbListToAtCat(mEtbList.get());
     }
 
     /**
@@ -568,27 +761,6 @@ public class ContentBlocking {
       return this;
     }
 
-    /**
-     * Get the assigned cookie lifetime.
-     *
-     * @return The assigned lifetime, as one of {@link CookieLifetime} flags.
-     */
-    @SuppressLint("WrongConstant")
-    public @CBCookieLifetime int getCookieLifetime() {
-      return mCookieLifetime.get();
-    }
-
-    /**
-     * Set the cookie lifetime.
-     *
-     * @param lifetime The enforced cookie lifetime. Use one of the {@link CookieLifetime} flags.
-     * @return This Settings instance.
-     */
-    public @NonNull Settings setCookieLifetime(final @CBCookieLifetime int lifetime) {
-      mCookieLifetime.commit(lifetime);
-      return this;
-    }
-
     /**
      * Get whether or not cookie purging is enabled.
      *
@@ -612,6 +784,175 @@ public class ContentBlocking {
       return this;
     }
 
+    /**
+     * Set the Cookie Banner Handling Mode to the new provided {@link CBCookieBannerMode} value.
+     *
+     * @param mode Integer indicating the new mode.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setCookieBannerMode(final @CBCookieBannerMode int mode) {
+      mCbhMode.commit(mode);
+      return this;
+    }
+
+    /**
+     * When set to true, cookie banners are detected and detection events are dispatched, but they
+     * will not be handled. Requires the service to be enabled for the desired mode via
+     * setCookieBannerMode.
+     *
+     * @param enabled A boolean indicating whether to enable cookie banners.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setCookieBannerDetectOnlyMode(final boolean enabled) {
+      mChbDetectOnlyMode.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Enables/disables the use of global CookieBannerRules, which apply to all sites. This enable
+     * handling of CMPs across sites without the use of site-specific rules.
+     *
+     * @param enabled A boolean indicating whether or not to enable.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setCookieBannerGlobalRulesEnabled(final boolean enabled) {
+      mCbhGlobalRulesEnabled.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Indicates if global CookieBannerRules is enabled or not.
+     *
+     * @return Indicates if global CookieBannerRule is enabled or disabled.
+     */
+    public boolean getCookieBannerGlobalRulesEnabled() {
+      return mCbhGlobalRulesEnabled.get();
+    }
+
+    /**
+     * Whether global rules are allowed to run in sub-frames. Running query selectors in every
+     * sub-frame may negatively impact performance, but is required for some CMPs.
+     *
+     * @param enabled A boolean indicating whether or not to enable.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setCookieBannerGlobalRulesSubFramesEnabled(final boolean enabled) {
+      mCbhGlobalRulesSubFramesEnabled.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Indicates if email tracker blocking is enabled in private mode.
+     *
+     * @return Indicates if email tracker blocking is enabled or disabled in private mode.
+     */
+    public @NonNull Boolean getEmailTrackerBlockingPrivateBrowsingEnabled() {
+      return mEtbPrivateBrowsing.get();
+    }
+
+    /**
+     * Sets whether email tracker blocking is enabled in private mode.
+     *
+     * @param enabled A boolean indicating whether or not to enable.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setEmailTrackerBlockingPrivateBrowsing(final boolean enabled) {
+      mEtbPrivateBrowsing.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Sets whether query parameter stripping is enabled in normal mode.
+     *
+     * @param enabled A boolean indicating whether or not to enable.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setQueryParameterStrippingEnabled(final boolean enabled) {
+      mQueryParameterStrippingEnabled.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Indicates if query parameter stripping is enabled in normal mode.
+     *
+     * @return Indicates if query parameter stripping is enabled or disabled in normal mode.
+     */
+    public boolean getQueryParameterStrippingEnabled() {
+      return mQueryParameterStrippingEnabled.get();
+    }
+
+    /**
+     * Sets Whether query parameter stripping is enabled in private mode.
+     *
+     * @param enabled A boolean indicating whether or not to enable in private mode.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setQueryParameterStrippingPrivateBrowsingEnabled(
+        final boolean enabled) {
+      mQueryParameterStrippingPrivateBrowsingEnabled.commit(enabled);
+      return this;
+    }
+
+    /**
+     * Indicates if query parameter stripping is enabled in private mode.
+     *
+     * @return Indicates if global CookieBannerRules is enabled or disabled in sub-frames.
+     */
+    public boolean getQueryParameterStrippingPrivateBrowsingEnabled() {
+      return mQueryParameterStrippingPrivateBrowsingEnabled.get();
+    }
+
+    /**
+     * Indicates if global CookieBannerRules is enabled or not in sub-frames.
+     *
+     * @return Indicates if global CookieBannerRules is enabled or disabled in sub-frames.
+     */
+    public boolean getCookieBannerGlobalRulesSubFramesEnabled() {
+      return mCbhGlobalRulesSubFramesEnabled.get();
+    }
+
+    /**
+     * Indicates if cookie banner handling detect only mode is enabled.
+     *
+     * @return boolean indicating if the cookie banner handling detect only mode setting is enabled.
+     */
+    public boolean getCookieBannerDetectOnlyMode() {
+      return mChbDetectOnlyMode.get();
+    }
+
+    /**
+     * Gets the current cookie banner handling mode.
+     *
+     * @return int the current cookie banner handling mode, one of the {@link CBCookieBannerMode}.
+     */
+    @SuppressLint("WrongConstant")
+    public @CBCookieBannerMode int getCookieBannerMode() {
+      return mCbhMode.get();
+    }
+
+    /**
+     * Set the Cookie Banner Handling Mode for private browsing to the new provided {@link
+     * CBCookieBannerMode} value.
+     *
+     * @param mode Integer indicating the new mode.
+     * @return This Settings instance.
+     */
+    public @NonNull Settings setCookieBannerModePrivateBrowsing(
+        final @CBCookieBannerMode int mode) {
+      mCbhModePrivateBrowsing.commit(mode);
+      return this;
+    }
+
+    /**
+     * Gets the current cookie banner handling mode for private browsing.
+     *
+     * @return int the current cookie banner handling mode, one of the {@link CBCookieBannerMode}.
+     */
+    @SuppressLint("WrongConstant")
+    public @CBCookieBannerMode int getCookieBannerModePrivateBrowsing() {
+      return mCbhModePrivateBrowsing.get();
+    }
+
     public static final Parcelable.Creator<Settings> CREATOR =
         new Parcelable.Creator<Settings>() {
           @Override
@@ -1143,11 +1484,14 @@ public class ContentBlocking {
     /** Block trackers on the Social Tracking Protection list. */
     public static final int STP = 1 << 8;
 
+    /** Block email trackers */
+    public static final int EMAIL = 1 << 9;
+
     /** Block ad, analytic, social and test trackers. */
     public static final int DEFAULT = AD | ANALYTIC | SOCIAL | TEST;
 
     /** Block all known trackers. May cause issues with some web sites. */
-    public static final int STRICT = DEFAULT | CONTENT | CRYPTOMINING | FINGERPRINTING;
+    public static final int STRICT = DEFAULT | CONTENT | CRYPTOMINING | FINGERPRINTING | EMAIL;
 
     protected AntiTracking() {}
   }
@@ -1166,6 +1510,7 @@ public class ContentBlocking {
         AntiTracking.DEFAULT,
         AntiTracking.STRICT,
         AntiTracking.STP,
+        AntiTracking.EMAIL,
         AntiTracking.NONE
       })
   public @interface CBAntiTracking {}
@@ -1238,30 +1583,15 @@ public class ContentBlocking {
 
   @Retention(RetentionPolicy.SOURCE)
   @IntDef({
-    CookieBehavior.ACCEPT_ALL, CookieBehavior.ACCEPT_FIRST_PARTY,
-    CookieBehavior.ACCEPT_NONE, CookieBehavior.ACCEPT_VISITED,
-    CookieBehavior.ACCEPT_NON_TRACKERS
+    CookieBehavior.ACCEPT_ALL,
+    CookieBehavior.ACCEPT_FIRST_PARTY,
+    CookieBehavior.ACCEPT_NONE,
+    CookieBehavior.ACCEPT_VISITED,
+    CookieBehavior.ACCEPT_NON_TRACKERS,
+    CookieBehavior.ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS
   })
   public @interface CBCookieBehavior {}
 
-  // Sync values with nsICookieService.idl.
-  public static class CookieLifetime {
-    /** Accept default cookie lifetime. */
-    public static final int NORMAL = 0;
-
-    /** Downgrade cookie lifetime to this runtime's lifetime. */
-    public static final int RUNTIME = 2;
-
-    /** Limit cookie lifetime to N days. Defaults to 90 days. */
-    public static final int DAYS = 3;
-
-    protected CookieLifetime() {}
-  }
-
-  @Retention(RetentionPolicy.SOURCE)
-  @IntDef({CookieLifetime.NORMAL, CookieLifetime.RUNTIME, CookieLifetime.DAYS})
-  public @interface CBCookieLifetime {}
-
   @Retention(RetentionPolicy.SOURCE)
   @IntDef({EtpLevel.NONE, EtpLevel.DEFAULT, EtpLevel.STRICT})
   public @interface CBEtpLevel {}
@@ -1353,7 +1683,8 @@ public class ContentBlocking {
           ContentBlocking.atListToAtCat(matchedList)
               | ContentBlocking.cmListToAtCat(matchedList)
               | ContentBlocking.fpListToAtCat(matchedList)
-              | ContentBlocking.stListToAtCat(matchedList),
+              | ContentBlocking.stListToAtCat(matchedList)
+              | ContentBlocking.etbListToAtCat(matchedList),
           ContentBlocking.errorToSbCat(error),
           ContentBlocking.geckoCatToCbCat(category),
           blocking);
@@ -1399,6 +1730,7 @@ public class ContentBlocking {
   private static final String FINGERPRINTING = "base-fingerprinting-track-digest256";
   private static final String STP =
       "social-tracking-protection-facebook-digest256,social-tracking-protection-linkedin-digest256,social-tracking-protection-twitter-digest256";
+  private static final String EMAIL = "base-email-track-digest256";
 
   /* package */ static @CBSafeBrowsing int sbMalwareToSbCat(final boolean enabled) {
     return enabled
@@ -1410,11 +1742,11 @@ public class ContentBlocking {
     return enabled ? SafeBrowsing.PHISHING : SafeBrowsing.NONE;
   }
 
-  /* package */ static boolean catToSbMalware(@CBAntiTracking final int cat) {
+  /* package */ static boolean catToSbMalware(@CBSafeBrowsing final int cat) {
     return (cat & (SafeBrowsing.MALWARE | SafeBrowsing.UNWANTED | SafeBrowsing.HARMFUL)) != 0;
   }
 
-  /* package */ static boolean catToSbPhishing(@CBAntiTracking final int cat) {
+  /* package */ static boolean catToSbPhishing(@CBSafeBrowsing final int cat) {
     return (cat & SafeBrowsing.PHISHING) != 0;
   }
 
@@ -1484,17 +1816,25 @@ public class ContentBlocking {
     return (cat & AntiTracking.STP) != 0;
   }
 
-  /* package */ static String catToStListPref(@CBAntiTracking final int cat) {
-    final StringBuilder builder = new StringBuilder();
+  /* package */ static boolean catToEtbPref(@CBAntiTracking final int cat) {
+    return (cat & AntiTracking.EMAIL) != 0;
+  }
 
-    if ((cat & AntiTracking.STP) != 0) {
-      builder.append(STP).append(",");
-    }
-    if (builder.length() == 0) {
+  /**
+   * Generic method for converting a category of anti-tracking to a Pref.
+   *
+   * @param cat Int representing the enabled anti-tracking blockers.
+   * @param tbCat Int representing the category mask to check for.
+   * @param catPrefString String to return if [cat] contains [tbCat].
+   * @return Pref string if [cat] contains [tbCat] otherwise empty string.
+   */
+  /* package */ static String catToPref(
+      @CBAntiTracking final int cat, final int tbCat, final String catPrefString) {
+    if ((cat & tbCat) != 0) {
+      return catPrefString;
+    } else {
       return "";
     }
-    // Trim final ','.
-    return builder.substring(0, builder.length() - 1);
   }
 
   /* package */ static @CBAntiTracking int atListToAtCat(final String list) {
@@ -1543,6 +1883,17 @@ public class ContentBlocking {
     return cat;
   }
 
+  /* package */ static @CBAntiTracking int etbListToAtCat(final String list) {
+    int cat = AntiTracking.NONE;
+    if (list == null) {
+      return cat;
+    }
+    if (list.indexOf(EMAIL) != -1) {
+      cat |= AntiTracking.EMAIL;
+    }
+    return cat;
+  }
+
   /* package */ static @CBSafeBrowsing int errorToSbCat(final long error) {
     // Match flags with XPCOM ErrorList.h.
     if (error == 0x805D001FL) {
@@ -1603,4 +1954,27 @@ public class ContentBlocking {
     // TODO: There are more reasons why cookies may be blocked.
     return CookieBehavior.ACCEPT_ALL;
   }
+
+  // Cookie Banner Handling feature.
+
+  public static class CookieBannerMode {
+    /** Do not enable handling cookie banners. */
+    public static final int COOKIE_BANNER_MODE_DISABLED = 0;
+
+    /** Only handle banners where selecting "reject all" is possible. */
+    public static final int COOKIE_BANNER_MODE_REJECT = 1;
+
+    /** Reject cookies when possible otherwise accept the cookies. */
+    public static final int COOKIE_BANNER_MODE_REJECT_OR_ACCEPT = 2;
+
+    protected CookieBannerMode() {}
+  }
+
+  @Retention(RetentionPolicy.SOURCE)
+  @IntDef({
+    CookieBannerMode.COOKIE_BANNER_MODE_DISABLED,
+    CookieBannerMode.COOKIE_BANNER_MODE_REJECT,
+    CookieBannerMode.COOKIE_BANNER_MODE_REJECT_OR_ACCEPT,
+  })
+  public @interface CBCookieBannerMode {}
 }
