@@ -23,6 +23,7 @@ import android.os.Parcel;
 import android.os.Parcelable;
 import android.os.Process;
 import android.provider.Settings;
+import android.text.format.DateFormat;
 import android.util.Log;
 import androidx.annotation.AnyThread;
 import androidx.annotation.NonNull;
@@ -39,6 +40,7 @@ import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.util.List;
 import java.util.Map;
+import org.mozilla.gecko.Clipboard;
 import org.mozilla.gecko.EventDispatcher;
 import org.mozilla.gecko.GeckoAppShell;
 import org.mozilla.gecko.GeckoNetworkManager;
@@ -104,6 +106,14 @@ public final class GeckoRuntime implements Parcelable {
    */
   public static final String EXTRA_CRASH_PROCESS_TYPE = "processType";
 
+  /**
+   * This is a key for extra data sent with {@link #ACTION_CRASHED}. The value is a String
+   * containing the content process type, which might not be available even for child processes.
+   *
+   * @see GeckoSession.ContentDelegate#onCrash(GeckoSession)
+   */
+  public static final String EXTRA_CRASH_REMOTE_TYPE = "remoteType";
+
   /**
    * Value for {@link #EXTRA_CRASH_PROCESS_TYPE} indicating the main application process was
    * affected by the crash, which is therefore fatal.
@@ -157,20 +167,43 @@ public final class GeckoRuntime implements Parcelable {
         // Do not trigger the first onResume event because it breaks nsAppShell::sPauseCount counter
         // thresholds.
         GeckoThread.onResume();
+      } else {
+        // Notify Gecko when the application has been moved in the foreground for the first time
+        // after being created and started (used by the ExtensionProcessCrashObserver on the Gecko
+        // side to adjust the appIsForeground property when the application-foreground or
+        // application-background topics are not notified).
+        EventDispatcher.getInstance().dispatch("GeckoView:InitialForeground", null);
       }
       mPaused = false;
+      // Can resume location services, checks if was in use before going to background
+      GeckoAppShell.resumeLocation();
       // Monitor network status and send change notifications to Gecko
       // while active.
       GeckoNetworkManager.getInstance().start(GeckoAppShell.getApplicationContext());
+
+      // Set settings that may have changed between last app opening
+      GeckoAppShell.setIs24HourFormat(
+          DateFormat.is24HourFormat(GeckoAppShell.getApplicationContext()));
+
+      // OnPrimaryClipChangedListener() won’t be triggered for a background
+      // application, so update the clipboard sequence number once the
+      // application returns to the foreground.
+      ThreadUtils.sGeckoHandler.post(
+          () -> {
+            Clipboard.updateSequenceNumber(GeckoAppShell.getApplicationContext());
+          });
     }
 
     @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
     void onPause() {
       Log.d(LOGTAG, "Lifecycle: onPause");
       mPaused = true;
+      // Pause listening for locations when in background
+      GeckoAppShell.pauseLocation();
       // Stop monitoring network status while inactive.
       GeckoNetworkManager.getInstance().stop();
       GeckoThread.onPause();
+      Clipboard.onPause();
     }
   }
 
@@ -220,11 +253,7 @@ public final class GeckoRuntime implements Parcelable {
     mContentBlockingController = new ContentBlockingController();
     mAutocompleteStorageProxy = new Autocomplete.StorageProxy();
     mProfilerController = new ProfilerController();
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
-      mScreenChangeListener = new GeckoScreenChangeListener();
-    } else {
-      mScreenChangeListener = null;
-    }
+    mScreenChangeListener = new GeckoScreenChangeListener();
 
     if (sRuntime != null) {
       throw new IllegalStateException("Only one GeckoRuntime instance is allowed");
@@ -247,34 +276,15 @@ public final class GeckoRuntime implements Parcelable {
    * @return SessionID to use for the request.
    */
   @SuppressLint("WrongThread") // for .isOpen() which is called on the UI thread
-  @WrapForJNI(calledFrom = "gecko")
-  private static @NonNull GeckoResult<String> serviceWorkerOpenWindow(final @NonNull String url) {
+  @UiThread
+  private static @NonNull GeckoResult<GeckoSession> serviceWorkerOpenWindow(
+      final @NonNull String url) {
     if (sRuntime != null && sRuntime.mServiceWorkerDelegate != null) {
-      final GeckoResult<String> result = new GeckoResult<>();
-      // perform the onOpenWindow call in the UI thread
-      ThreadUtils.runOnUiThread(
-          () -> {
-            sRuntime
-                .mServiceWorkerDelegate
-                .onOpenWindow(url)
-                .accept(
-                    session -> {
-                      if (session != null) {
-                        if (!session.isOpen()) {
-                          session.open(sRuntime);
-                        }
-                        session.loadUri(url);
-                        result.complete(session.getId());
-                      } else {
-                        result.complete(null);
-                      }
-                    });
-          });
-      return result;
-    } else {
-      return GeckoResult.fromException(
-          new java.lang.RuntimeException("No available Service Worker delegate."));
+      ThreadUtils.assertOnUiThread();
+      return sRuntime.mServiceWorkerDelegate.onOpenWindow(url);
     }
+    return GeckoResult.fromException(
+        new java.lang.RuntimeException("No available Service Worker delegate."));
   }
 
   /**
@@ -305,18 +315,67 @@ public final class GeckoRuntime implements Parcelable {
             mDelegate.onShutdown();
             EventDispatcher.getInstance()
                 .unregisterUiThreadListener(mEventListener, "Gecko:Exited");
+          } else if ("GeckoView:Test:NewTab".equals(event)) {
+            final String url = message.getString("url", "about:blank");
+            serviceWorkerOpenWindow(url)
+                .then(
+                    (GeckoResult.OnValueListener<GeckoSession, Void>)
+                        session -> {
+                          if (session == null) {
+                            callback.sendSuccess(null);
+                            return null;
+                          }
+                          if (!session.isOpen()) {
+                            session.open(sRuntime);
+                          }
+                          callback.sendSuccess(session.getId());
+                          return null;
+                        })
+                .exceptionally(
+                    (GeckoResult.OnExceptionListener<Void>)
+                        error -> {
+                          callback.sendError(error + " Could not open tab.");
+                          return null;
+                        });
           } else if ("GeckoView:ChildCrashReport".equals(event) && crashHandler != null) {
             final Context context = GeckoAppShell.getApplicationContext();
             final Intent i = new Intent(ACTION_CRASHED, null, context, crashHandler);
             i.putExtra(EXTRA_MINIDUMP_PATH, message.getString(EXTRA_MINIDUMP_PATH));
             i.putExtra(EXTRA_EXTRAS_PATH, message.getString(EXTRA_EXTRAS_PATH));
             i.putExtra(EXTRA_CRASH_PROCESS_TYPE, message.getString(EXTRA_CRASH_PROCESS_TYPE));
+            i.putExtra(EXTRA_CRASH_REMOTE_TYPE, message.getString(EXTRA_CRASH_REMOTE_TYPE));
 
             if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
               context.startForegroundService(i);
             } else {
               context.startService(i);
             }
+          } else if ("GeckoView:ServiceWorkerOpenWindow".equals(event)) {
+            final String url = message.getString("url", "about:blank");
+            serviceWorkerOpenWindow(url)
+                .then(
+                    (GeckoResult.OnValueListener<GeckoSession, Void>)
+                        session -> {
+                          if (session == null) {
+                            callback.sendSuccess(null);
+                            return null;
+                          }
+                          final boolean isOpen = session.isOpen();
+                          if (!isOpen) {
+                            session.open(sRuntime);
+                          }
+                          final GeckoBundle bundle = new GeckoBundle();
+                          bundle.putBoolean("isOpen", isOpen);
+                          bundle.putString("sessionId", session.getId());
+                          callback.sendSuccess(bundle);
+                          return null;
+                        })
+                .exceptionally(
+                    (GeckoResult.OnExceptionListener<Void>)
+                        error -> {
+                          callback.sendError(error + " Could not open tab.");
+                          return null;
+                        });
           }
         }
       };
@@ -379,29 +438,27 @@ public final class GeckoRuntime implements Parcelable {
     Map<String, Object> prefs = settings.getPrefsMap();
 
     // Older versions have problems with SnakeYaml
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
-      String configFilePath = settings.getConfigFilePath();
-      if (configFilePath == null) {
-        // Default to /data/local/tmp/$PACKAGE-geckoview-config.yaml if android:debuggable="true"
-        // or if this application is the current Android "debug_app", and to not read configuration
-        // from a file otherwise.
-        if (isApplicationDebuggable(context) || isApplicationCurrentDebugApp(context)) {
-          configFilePath =
-              String.format(CONFIG_FILE_PATH_TEMPLATE, context.getApplicationInfo().packageName);
-        }
+    String configFilePath = settings.getConfigFilePath();
+    if (configFilePath == null) {
+      // Default to /data/local/tmp/$PACKAGE-geckoview-config.yaml if android:debuggable="true"
+      // or if this application is the current Android "debug_app", and to not read configuration
+      // from a file otherwise.
+      if (isApplicationDebuggable(context) || isApplicationCurrentDebugApp(context)) {
+        configFilePath =
+            String.format(CONFIG_FILE_PATH_TEMPLATE, context.getApplicationInfo().packageName);
       }
+    }
 
-      if (configFilePath != null && !configFilePath.isEmpty()) {
-        try {
-          final DebugConfig debugConfig = DebugConfig.fromFile(new File(configFilePath));
-          Log.i(LOGTAG, "Adding debug configuration from: " + configFilePath);
-          prefs = debugConfig.mergeIntoPrefs(prefs);
-          args = debugConfig.mergeIntoArgs(args);
-          extras = debugConfig.mergeIntoExtras(extras);
-        } catch (final DebugConfig.ConfigException e) {
-          Log.w(LOGTAG, "Failed to add debug configuration from: " + configFilePath, e);
-        } catch (final FileNotFoundException e) {
-        }
+    if (configFilePath != null && !configFilePath.isEmpty()) {
+      try {
+        final DebugConfig debugConfig = DebugConfig.fromFile(new File(configFilePath));
+        Log.i(LOGTAG, "Adding debug configuration from: " + configFilePath);
+        prefs = debugConfig.mergeIntoPrefs(prefs);
+        args = debugConfig.mergeIntoArgs(args);
+        extras = debugConfig.mergeIntoExtras(extras);
+      } catch (final DebugConfig.ConfigException e) {
+        Log.w(LOGTAG, "Failed to add debug configuration from: " + configFilePath, e);
+      } catch (final FileNotFoundException e) {
       }
     }
 
@@ -438,7 +495,12 @@ public final class GeckoRuntime implements Parcelable {
     mSettings = settings;
 
     // Bug 1453062 -- the EventDispatcher should really live here (or in GeckoThread)
-    EventDispatcher.getInstance().registerUiThreadListener(mEventListener, "Gecko:Exited");
+    EventDispatcher.getInstance()
+        .registerUiThreadListener(
+            mEventListener,
+            "Gecko:Exited",
+            "GeckoView:Test:NewTab",
+            "GeckoView:ServiceWorkerOpenWindow");
 
     // Attach and commit settings.
     mSettings.attachTo(this);
@@ -467,14 +529,8 @@ public final class GeckoRuntime implements Parcelable {
   private boolean isApplicationCurrentDebugApp(final @NonNull Context context) {
     final ApplicationInfo applicationInfo = context.getApplicationInfo();
 
-    final String currentDebugApp;
-    if (Build.VERSION.SDK_INT >= 17) {
-      currentDebugApp =
-          Settings.Global.getString(context.getContentResolver(), Settings.Global.DEBUG_APP);
-    } else {
-      currentDebugApp =
-          Settings.System.getString(context.getContentResolver(), Settings.System.DEBUG_APP);
-    }
+    final String currentDebugApp =
+        Settings.Global.getString(context.getContentResolver(), Settings.Global.DEBUG_APP);
     return applicationInfo.packageName.equals(currentDebugApp);
   }
 
@@ -683,6 +739,11 @@ public final class GeckoRuntime implements Parcelable {
     mNotificationDelegate = delegate;
   }
 
+  @WrapForJNI
+  /* package */ float textScaleFactor() {
+    return getSettings().getFontSizeFactor();
+  }
+
   @WrapForJNI
   /* package */ boolean usesDarkTheme() {
     switch (getSettings().getPreferredColorScheme()) {
@@ -1022,4 +1083,17 @@ public final class GeckoRuntime implements Parcelable {
           return new GeckoRuntime[size];
         }
       };
+
+  /**
+   * Whether the default `interactive-widget` is `resizes-visual`.
+   *
+   * @return True the default `interactive-widget` is `resizes-visual`, false otherwise.
+   */
+  @AnyThread
+  public boolean isInteractiveWidgetDefaultResizesVisual() {
+    if (!GeckoThread.isStateAtLeast(GeckoThread.State.JNI_READY)) {
+      return false;
+    }
+    return GeckoAppShell.isInteractiveWidgetDefaultResizesVisual();
+  }
 }
