@@ -15,7 +15,9 @@
 
 #include "mozilla/Atomics.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/MozPromise.h"
 #include "mozilla/StaticPtr.h"
+#include "mozilla/ipc/SharedMemoryHandle.h"
 #include "nsCOMPtr.h"
 #include "nsIObserver.h"
 #include "nsIPrefBranch.h"
@@ -97,6 +99,8 @@ class Preferences final : public nsIPrefService,
   friend class ::nsPrefBranch;
 
  public:
+  using WritePrefFilePromise = MozPromise<bool, nsresult, false>;
+
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIPREFSERVICE
   NS_FORWARD_NSIPREFBRANCH(mRootBranch->)
@@ -258,9 +262,11 @@ class Preferences final : public nsIPrefService,
   // Note: All preference strings *must* be statically-allocated string
   // literals.
   static nsresult AddStrongObservers(nsIObserver* aObserver,
-                                     const char** aPrefs);
-  static nsresult AddWeakObservers(nsIObserver* aObserver, const char** aPrefs);
-  static nsresult RemoveObservers(nsIObserver* aObserver, const char** aPrefs);
+                                     const char* const* aPrefs);
+  static nsresult AddWeakObservers(nsIObserver* aObserver,
+                                   const char* const* aPrefs);
+  static nsresult RemoveObservers(nsIObserver* aObserver,
+                                  const char* const* aPrefs);
 
   // Registers/Unregisters the callback function for the aPref.
   template <typename T = void>
@@ -324,28 +330,28 @@ class Preferences final : public nsIPrefService,
   // Unregister call as was passed to the Register call.
   template <typename T = void>
   static nsresult RegisterCallbacks(PrefChangedFunc aCallback,
-                                    const char** aPrefs,
+                                    const char* const* aPrefs,
                                     T* aClosure = nullptr) {
     return RegisterCallbacks(aCallback, aPrefs, aClosure, ExactMatch);
   }
   static nsresult RegisterCallbacksAndCall(PrefChangedFunc aCallback,
-                                           const char** aPrefs,
+                                           const char* const* aPrefs,
                                            void* aClosure = nullptr);
   template <typename T = void>
   static nsresult UnregisterCallbacks(PrefChangedFunc aCallback,
-                                      const char** aPrefs,
+                                      const char* const* aPrefs,
                                       T* aClosure = nullptr) {
     return UnregisterCallbacks(aCallback, aPrefs, aClosure, ExactMatch);
   }
   template <typename T = void>
   static nsresult RegisterPrefixCallbacks(PrefChangedFunc aCallback,
-                                          const char** aPrefs,
+                                          const char* const* aPrefs,
                                           T* aClosure = nullptr) {
     return RegisterCallbacks(aCallback, aPrefs, aClosure, PrefixMatch);
   }
   template <typename T = void>
   static nsresult UnregisterPrefixCallbacks(PrefChangedFunc aCallback,
-                                            const char** aPrefs,
+                                            const char* const* aPrefs,
                                             T* aClosure = nullptr) {
     return UnregisterCallbacks(aCallback, aPrefs, aClosure, PrefixMatch);
   }
@@ -402,10 +408,10 @@ class Preferences final : public nsIPrefService,
   // prefs in bulk from the parent process, via shared memory.
   static void SerializePreferences(nsCString& aStr,
                                    bool aIsDestinationWebContentProcess);
-  static void DeserializePreferences(char* aStr, size_t aPrefsLen);
+  static void DeserializePreferences(const char* aStr, size_t aPrefsLen);
 
-  static mozilla::ipc::FileDescriptor EnsureSnapshot(size_t* aSize);
-  static void InitSnapshot(const mozilla::ipc::FileDescriptor&, size_t aSize);
+  static mozilla::ipc::ReadOnlySharedMemoryHandle EnsureSnapshot();
+  static void InitSnapshot(const mozilla::ipc::ReadOnlySharedMemoryHandle&);
 
   // When a single pref is changed in the parent process, these methods are
   // used to pass the update to content processes.
@@ -452,7 +458,9 @@ class Preferences final : public nsIPrefService,
 
   // Off main thread is only respected for the default aFile value (nullptr).
   nsresult SavePrefFileInternal(nsIFile* aFile, SaveMethod aSaveMethod);
-  nsresult WritePrefFile(nsIFile* aFile, SaveMethod aSaveMethod);
+  nsresult WritePrefFile(
+      nsIFile* aFile, SaveMethod aSaveMethod,
+      UniquePtr<MozPromiseHolder<WritePrefFilePromise>> aPromise = nullptr);
 
   nsresult ResetUserPrefs();
 
@@ -483,10 +491,10 @@ class Preferences final : public nsIPrefService,
                                           void* aClosure, MatchKind aMatchKind);
 
   static nsresult RegisterCallbacks(PrefChangedFunc aCallback,
-                                    const char** aPrefs, void* aClosure,
+                                    const char* const* aPrefs, void* aClosure,
                                     MatchKind aMatchKind);
   static nsresult UnregisterCallbacks(PrefChangedFunc aCallback,
-                                      const char** aPrefs, void* aClosure,
+                                      const char* const* aPrefs, void* aClosure,
                                       MatchKind aMatchKind);
 
   template <typename T>
@@ -518,6 +526,8 @@ class Preferences final : public nsIPrefService,
 
  private:
   nsCOMPtr<nsIFile> mCurrentFile;
+  // Time since unix epoch in ms (JS Date compatible)
+  PRTime mUserPrefsFileLastModifiedAtStartup = 0;
   bool mDirty = false;
   bool mProfileShutdown = false;
   // We wait a bit after prefs are dirty before writing them. In this period,
@@ -537,7 +547,14 @@ class Preferences final : public nsIPrefService,
 extern Atomic<bool, Relaxed> sOmitBlocklistedPrefValues;
 extern Atomic<bool, Relaxed> sCrashOnBlocklistedPref;
 
-bool ShouldSanitizePreference(const char* aPref, bool aIsDestWebContentProcess);
+bool IsPreferenceSanitized(const char* aPref);
+
+const char kFissionEnforceBlockList[] =
+    "fission.enforceBlocklistedPrefsInSubprocesses";
+const char kFissionOmitBlockListValues[] =
+    "fission.omitBlocklistedPrefsInSubprocesses";
+
+void OnFissionBlocklistPrefChange(const char* aPref, void* aData);
 
 }  // namespace mozilla
 
