@@ -2,39 +2,22 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, # You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import (
-    absolute_import,
-    print_function,
-    unicode_literals,
-)
-
 import argparse
 import json
 import os
+import platform
 import re
 import shutil
 import subprocess
 import sys
 import tempfile
-
 from collections import OrderedDict
 
-from six import iteritems
-
-from mach.decorators import (
-    Command,
-    CommandArgument,
-    SubCommand,
-)
-
-from mozbuild.base import (
-    MozbuildObject,
-    BinaryNotFoundException,
-)
-from mozbuild import nodeutil
 import mozlog
 import mozprofile
-
+from mach.decorators import Command, CommandArgument, SubCommand
+from mozbuild import nodeutil
+from mozbuild.base import BinaryNotFoundException, MozbuildObject
 
 EX_CONFIG = 78
 EX_SOFTWARE = 70
@@ -67,8 +50,8 @@ def remote(command_context):
 @CommandArgument(
     "--repository",
     metavar="REPO",
-    required=True,
-    help="The (possibly remote) repository to clone from.",
+    default="https://github.com/puppeteer/puppeteer.git",
+    help="The (possibly local) repository to clone from.",
 )
 @CommandArgument(
     "--commitish",
@@ -89,13 +72,16 @@ def vendor_puppeteer(command_context, repository, commitish, install):
     # Preserve our custom mocha reporter
     shutil.move(
         os.path.join(puppeteer_dir, "json-mocha-reporter.js"),
-        remotedir(command_context),
+        os.path.join(remotedir(command_context), "json-mocha-reporter.js"),
     )
+
+    print("Removing folders for current Puppeteer version…")
     shutil.rmtree(puppeteer_dir, ignore_errors=True)
     os.makedirs(puppeteer_dir)
+
     with TemporaryDirectory() as tmpdir:
-        git("clone", "-q", repository, tmpdir)
-        git("checkout", commitish, worktree=tmpdir)
+        print(f'Fetching commitish "{commitish}" from {repository}…')
+        git("clone", "--depth", "1", "--branch", commitish, repository, tmpdir)
         git(
             "checkout-index",
             "-a",
@@ -150,8 +136,22 @@ def vendor_puppeteer(command_context, repository, commitish, install):
         )
 
     if install:
-        env = {"HUSKY": "0", "PUPPETEER_SKIP_DOWNLOAD": "1"}
-        npm(
+        env = {
+            "CI": "1",  # Force the quiet logger of wireit
+            "HUSKY": "0",  # Disable any hook checks
+            "PUPPETEER_SKIP_DOWNLOAD": "1",  # Don't download any build
+        }
+
+        print("Cleaning up and installing new version of Puppeteer…")
+        run_npm(
+            "run",
+            "clean",
+            cwd=puppeteer_dir,
+            env=env,
+            exit_on_fail=False,
+        )
+
+        run_npm(
             "install",
             cwd=os.path.join(command_context.topsrcdir, puppeteer_dir),
             env=env,
@@ -188,43 +188,42 @@ def git(*args, **kwargs):
     return out
 
 
-def npm(*args, **kwargs):
-    from mozprocess import processhandler
+def run_npm(*args, **kwargs):
+    from mozprocess import run_and_wait
 
-    env = None
+    def output_timeout_handler(proc):
+        # In some cases, we wait longer for a mocha timeout
+        print(
+            "Timed out after {} seconds of no output".format(kwargs["output_timeout"])
+        )
+
+    env = os.environ.copy()
     npm, _ = nodeutil.find_npm_executable()
     if kwargs.get("env"):
-        env = os.environ.copy()
         env.update(kwargs["env"])
 
-    proc_kwargs = {}
-    if "processOutputLine" in kwargs:
-        proc_kwargs["processOutputLine"] = kwargs["processOutputLine"]
+    proc_kwargs = {"output_timeout_handler": output_timeout_handler}
+    for kw in ["output_line_handler", "output_timeout"]:
+        if kw in kwargs:
+            proc_kwargs[kw] = kwargs[kw]
 
-    p = processhandler.ProcessHandler(
-        cmd=npm,
-        args=list(args),
+    cmd = [npm]
+    cmd.extend(list(args))
+
+    p = run_and_wait(
+        args=cmd,
         cwd=kwargs.get("cwd"),
         env=env,
-        universal_newlines=True,
-        **proc_kwargs
+        text=True,
+        **proc_kwargs,
     )
-    if not kwargs.get("wait", True):
-        return p
-
-    wait_proc(p, cmd=npm, exit_on_fail=kwargs.get("exit_on_fail", True))
+    post_wait_proc(p, cmd=npm, exit_on_fail=kwargs.get("exit_on_fail", True))
 
     return p.returncode
 
 
-def wait_proc(p, cmd=None, exit_on_fail=True, output_timeout=None):
-    try:
-        p.run(outputTimeout=output_timeout)
-        p.wait()
-        if p.timedOut:
-            # In some cases, we wait longer for a mocha timeout
-            print("Timed out after {} seconds of no output".format(output_timeout))
-    finally:
+def post_wait_proc(p, cmd=None, exit_on_fail=True):
+    if p.poll() is None:
         p.kill()
     if exit_on_fail and p.returncode > 0:
         msg = (
@@ -260,7 +259,9 @@ class MochaOutputHandler(object):
     def pid(self):
         return self.proc and self.proc.pid
 
-    def __call__(self, line):
+    def __call__(self, proc, line):
+        self.proc = proc
+        line = line.rstrip("\r\n")
         event = None
         try:
             if line.startswith("[") and line.endswith("]"):
@@ -271,6 +272,14 @@ class MochaOutputHandler(object):
         finally:
             self.logger.process_output(self.pid, line, command="npm")
 
+    def testExpectation(self, testIdPattern, expected_name):
+        if testIdPattern.find("*") == -1:
+            return expected_name == testIdPattern
+        else:
+            return re.compile(re.escape(testIdPattern).replace(r"\*", ".*")).search(
+                expected_name
+            )
+
     def process_event(self, event):
         if isinstance(event, list) and len(event) > 1:
             status = self.status_map.get(event[0])
@@ -278,8 +287,10 @@ class MochaOutputHandler(object):
             if not status and not test_start:
                 return
             test_info = event[1]
-            test_name = test_info.get("fullTitle", "")
+            test_full_title = test_info.get("fullTitle", "")
+            test_name = test_full_title
             test_path = test_info.get("file", "")
+            test_file_name = os.path.basename(test_path).replace(".js", "")
             test_err = test_info.get("err")
             if status == "FAIL" and test_err:
                 if "timeout" in test_err.lower():
@@ -293,7 +304,19 @@ class MochaOutputHandler(object):
             if test_start:
                 self.logger.test_start(test_name)
                 return
-            expected = self.expected.get(test_name, ["PASS"])
+            expected_name = "[{}] {}".format(test_file_name, test_full_title)
+            expected_item = next(
+                (
+                    expectation
+                    for expectation in reversed(list(self.expected))
+                    if self.testExpectation(expectation["testIdPattern"], expected_name)
+                ),
+                None,
+            )
+            if expected_item is None:
+                expected = ["PASS"]
+            else:
+                expected = expected_item["expectations"]
             # mozlog doesn't really allow unexpected skip,
             # so if a test is disabled just expect that and note the unexpected skip
             # Also, mocha doesn't log test-start for skipped tests
@@ -325,34 +348,7 @@ class MochaOutputHandler(object):
                 known_intermittent=known_intermittent,
             )
 
-    def new_expected(self):
-        new_expected = OrderedDict()
-        for test_name, status in iteritems(self.test_results):
-            if test_name not in self.expected:
-                new_status = [status]
-            else:
-                if status in self.expected[test_name]:
-                    new_status = self.expected[test_name]
-                else:
-                    new_status = [status]
-            new_expected[test_name] = new_status
-        return new_expected
-
-    def after_end(self, subset=False):
-        if not subset:
-            missing = set(self.expected) - set(self.test_results)
-            extra = set(self.test_results) - set(self.expected)
-            if missing:
-                self.has_unexpected = True
-                for test_name in missing:
-                    self.logger.error("TEST-UNEXPECTED-MISSING %s" % (test_name,))
-            if self.expected and extra:
-                self.has_unexpected = True
-                for test_name in extra:
-                    self.logger.error(
-                        "TEST-UNEXPECTED-MISSING Unknown new test %s" % (test_name,)
-                    )
-
+    def after_end(self):
         if self.unexpected_skips:
             self.has_unexpected = True
             for test_name in self.unexpected_skips:
@@ -409,25 +405,15 @@ class PuppeteerRunner(MozbuildObject):
           before invoking npm.  Overrides default preferences.
         `enable_webrender`:
           Boolean to indicate whether to enable WebRender compositor in Gecko.
-        `write_results`:
-          Path to write the results json file
-        `subset`
-          Indicates only a subset of tests are being run, so we should
-          skip the check for missing results
         """
         setup()
 
-        binary = params.get("binary") or self.get_binary_path()
+        binary = params.get("binary")
+        headless = params.get("headless", False)
         product = params.get("product", "firefox")
+        this_chunk = params.get("this_chunk", "1")
+        total_chunks = params.get("total_chunks", "1")
 
-        env = {
-            # Print browser process ouptut
-            "DUMPIO": "1",
-            # Checked by Puppeteer's custom mocha config
-            "CI": "1",
-            # Causes some tests to be skipped due to assumptions about install
-            "PUPPETEER_ALT_INSTALL": "1",
-        }
         extra_options = {}
         for k, v in params.get("extra_launcher_options", {}).items():
             extra_options[k] = json.loads(v)
@@ -442,16 +428,59 @@ class PuppeteerRunner(MozbuildObject):
             "--timeout",
             "20000",
             "--no-parallel",
+            "--no-coverage",
         ]
+
+        env = {
+            # Checked by Puppeteer's custom mocha config
+            "CI": "1",
+            # Print browser process ouptut
+            "DUMPIO": "1",
+            # Run in headless mode if trueish, otherwise use headful
+            "HEADLESS": str(headless),
+            # Causes some tests to be skipped due to assumptions about install
+            "PUPPETEER_ALT_INSTALL": "1",
+        }
+
         if product == "firefox":
-            env["BINARY"] = binary
+            env["BINARY"] = binary or self.get_binary_path()
             env["PUPPETEER_PRODUCT"] = "firefox"
-
             env["MOZ_WEBRENDER"] = "%d" % params.get("enable_webrender", False)
+        else:
+            if binary:
+                env["BINARY"] = binary
+            env["PUPPETEER_CACHE_DIR"] = os.path.join(
+                self.topobjdir,
+                "_tests",
+                "remote",
+                "test",
+                "puppeteer",
+                ".cache",
+            )
 
-        command = ["run", "unit", "--"] + mocha_options
+        if product == "chrome":
+            if not headless:
+                raise Exception(
+                    "Chrome doesn't support headful mode with the WebDriver BiDi protocol"
+                )
+            test_command = "chrome-bidi"
+        elif product == "firefox":
+            if headless:
+                test_command = "firefox-headless"
+            else:
+                test_command = "firefox-headful"
+        else:
+            test_command = product
 
-        env["HEADLESS"] = str(params.get("headless", False))
+        command = [
+            "run",
+            "test",
+            "--",
+            "--shard",
+            f"{this_chunk}-{total_chunks}",
+            "--test-suite",
+            test_command,
+        ] + mocha_options
 
         prefs = {}
         for k, v in params.get("extra_prefs", {}).items():
@@ -465,44 +494,50 @@ class PuppeteerRunner(MozbuildObject):
             env["EXTRA_LAUNCH_OPTIONS"] = json.dumps(extra_options)
 
         expected_path = os.path.join(
-            os.path.dirname(__file__), "test", "puppeteer-expected.json"
+            os.path.dirname(__file__),
+            "test",
+            "puppeteer",
+            "test",
+            "TestExpectations.json",
         )
-        if product == "firefox" and os.path.exists(expected_path):
+        if os.path.exists(expected_path):
             with open(expected_path) as f:
                 expected_data = json.load(f)
         else:
-            expected_data = {}
+            expected_data = []
 
-        output_handler = MochaOutputHandler(logger, expected_data)
-        proc = npm(
+        expected_platform = platform.uname().system.lower()
+        if expected_platform == "windows":
+            expected_platform = "win32"
+
+        # Filter expectation data for the selected browser,
+        # headless or headful mode, the operating system,
+        # run in BiDi mode or not.
+        expectations = [
+            expectation
+            for expectation in expected_data
+            if is_relevant_expectation(
+                expectation, product, env["HEADLESS"], expected_platform
+            )
+        ]
+
+        output_handler = MochaOutputHandler(logger, expectations)
+        run_npm(
             *command,
             cwd=self.puppeteer_dir,
             env=env,
-            processOutputLine=output_handler,
-            wait=False
+            output_line_handler=output_handler,
+            # Puppeteer unit tests don't always clean-up child processes in case of
+            # failure, so use an output_timeout as a fallback
+            output_timeout=60,
+            exit_on_fail=True,
         )
-        output_handler.proc = proc
 
-        # Puppeteer unit tests don't always clean-up child processes in case of
-        # failure, so use an output_timeout as a fallback
-        wait_proc(proc, "npm", output_timeout=60, exit_on_fail=False)
-
-        output_handler.after_end(params.get("subset", False))
-
-        # Non-zero return codes are non-fatal for now since we have some
-        # issues with unresolved promises that shouldn't otherwise block
-        # running the tests
-        if proc.returncode != 0:
-            logger.warning("npm exited with code %s" % proc.returncode)
-
-        if params["write_results"]:
-            with open(params["write_results"], "w") as f:
-                json.dump(
-                    output_handler.new_expected(), f, indent=2, separators=(",", ": ")
-                )
+        output_handler.after_end()
 
         if output_handler.has_unexpected:
-            exit(1, "Got unexpected results")
+            logger.error("Got unexpected results")
+            exit(1)
 
 
 def create_parser_puppeteer():
@@ -549,6 +584,18 @@ def create_parser_puppeteer():
         metavar="<option>=<value>",
         help="Defines additional options for `puppeteer.launch`.",
     )
+    p.add_argument(
+        "--this-chunk",
+        type=str,
+        default="1",
+        help="Defines a current chunk to run.",
+    )
+    p.add_argument(
+        "--total-chunks",
+        type=str,
+        default="1",
+        help="Defines a total amount of chunks to run.",
+    )
     p.add_argument(
         "-v",
         dest="verbosity",
@@ -558,30 +605,40 @@ def create_parser_puppeteer():
         "debug level messages with -v, trace messages with -vv,"
         "and to not truncate long trace messages with -vvv",
     )
-    p.add_argument(
-        "--write-results",
-        action="store",
-        nargs="?",
-        default=None,
-        const=os.path.join(
-            os.path.dirname(__file__), "test", "puppeteer-expected.json"
-        ),
-        help="Path to write updated results to (defaults to the "
-        "expectations file if the argument is provided but "
-        "no path is passed)",
-    )
-    p.add_argument(
-        "--subset",
-        action="store_true",
-        default=False,
-        help="Indicate that only a subset of the tests are running, "
-        "so checks for missing tests should be skipped",
-    )
     p.add_argument("tests", nargs="*")
     mozlog.commandline.add_logging_group(p)
     return p
 
 
+def is_relevant_expectation(
+    expectation, expected_product, is_headless, expected_platform
+):
+    parameters = expectation["parameters"]
+
+    if expected_product == "firefox":
+        is_expected_product = (
+            "chrome" not in parameters and "chrome-headless-shell" not in parameters
+        )
+    else:
+        is_expected_product = "firefox" not in parameters
+
+    is_expected_protocol = "cdp" not in parameters
+
+    if is_headless == "True":
+        is_expected_mode = "headful" not in parameters
+    else:
+        is_expected_mode = "headless" not in parameters
+
+    is_expected_platform = expected_platform in expectation["platforms"]
+
+    return (
+        is_expected_product
+        and is_expected_protocol
+        and is_expected_mode
+        and is_expected_platform
+    )
+
+
 @Command(
     "puppeteer-test",
     category="testing",
@@ -608,11 +665,10 @@ def puppeteer_test(
     verbosity=0,
     tests=None,
     product="firefox",
-    write_results=None,
-    subset=False,
-    **kwargs
+    this_chunk="1",
+    total_chunks="1",
+    **kwargs,
 ):
-
     logger = mozlog.commandline.setup_logging(
         "puppeteer-test", kwargs, {"mach": sys.stdout}
     )
@@ -668,8 +724,8 @@ def puppeteer_test(
         "extra_prefs": prefs,
         "product": product,
         "extra_launcher_options": options,
-        "write_results": write_results,
-        "subset": subset,
+        "this_chunk": this_chunk,
+        "total_chunks": total_chunks,
     }
     puppeteer = command_context._spawn(PuppeteerRunner)
     try:
@@ -684,26 +740,46 @@ def puppeteer_test(
 
 def install_puppeteer(command_context, product, ci):
     setup()
-    env = {"HUSKY": "0"}
-    from mozversioncontrol import get_repository_object
 
-    repo = get_repository_object(command_context.topsrcdir)
+    env = {
+        "CI": "1",  # Force the quiet logger of wireit
+        "HUSKY": "0",  # Disable any hook checks
+    }
+
     puppeteer_dir = os.path.join("remote", "test", "puppeteer")
-    changed_files = False
-    for f in repo.get_changed_files():
-        if f.startswith(puppeteer_dir) and f.endswith(".ts"):
-            changed_files = True
-            break
+    puppeteer_dir_full_path = os.path.join(command_context.topsrcdir, puppeteer_dir)
+    puppeteer_test_dir = os.path.join(puppeteer_dir, "test")
 
-    if product != "chrome":
+    if product == "chrome":
+        env["PUPPETEER_PRODUCT"] = "chrome"
+        env["PUPPETEER_CACHE_DIR"] = os.path.join(
+            command_context.topobjdir, "_tests", puppeteer_dir, ".cache"
+        )
+    else:
         env["PUPPETEER_SKIP_DOWNLOAD"] = "1"
-    lib_dir = os.path.join(command_context.topsrcdir, puppeteer_dir, "lib")
-    if changed_files and os.path.isdir(lib_dir):
-        # clobber lib to force `tsc compile` step
-        shutil.rmtree(lib_dir)
 
-    command = "ci" if ci else "install"
-    npm(command, cwd=os.path.join(command_context.topsrcdir, puppeteer_dir), env=env)
+    if not ci:
+        run_npm(
+            "run",
+            "clean",
+            cwd=puppeteer_dir_full_path,
+            env=env,
+            exit_on_fail=False,
+        )
+
+    # Always use the `ci` command to not get updated sub-dependencies installed.
+    run_npm("ci", cwd=puppeteer_dir_full_path, env=env)
+
+    # Build Puppeteer and the code to download browsers.
+    run_npm(
+        "run",
+        "build",
+        cwd=os.path.join(command_context.topsrcdir, puppeteer_test_dir),
+        env=env,
+    )
+
+    # Run post install steps, including downloading the Chrome browser if requested
+    run_npm("run", "postinstall", cwd=puppeteer_dir_full_path, env=env)
 
 
 def exit(code, error=None):
