@@ -2,20 +2,32 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { setTimeout } = ChromeUtils.import("resource://gre/modules/Timer.jsm");
+const { setTimeout } = ChromeUtils.importESModule(
+  "resource://gre/modules/Timer.sys.mjs"
+);
 
 const {
+  DEFAULT_UNLOAD_TIMEOUT,
+  getUnloadTimeoutMultiplier,
   ProgressListener,
   waitForInitialNavigationCompleted,
-} = ChromeUtils.import("chrome://remote/content/shared/Navigate.jsm");
+} = ChromeUtils.importESModule(
+  "chrome://remote/content/shared/Navigate.sys.mjs"
+);
+
+const LOAD_FLAG_ERROR_PAGE = 0x10000;
 
 const CURRENT_URI = Services.io.newURI("http://foo.bar/");
 const INITIAL_URI = Services.io.newURI("about:blank");
 const TARGET_URI = Services.io.newURI("http://foo.cheese/");
-const TARGET_URI_IS_ERROR_PAGE = Services.io.newURI("doesnotexist://");
+const TARGET_URI_ERROR_PAGE = Services.io.newURI("doesnotexist://");
 const TARGET_URI_WITH_HASH = Services.io.newURI("http://foo.cheese/#foo");
 
+function wait(time) {
+  // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+  return new Promise(resolve => setTimeout(resolve, time));
+}
+
 class MockRequest {
   constructor(uri) {
     this.originalURI = uri;
@@ -33,6 +45,7 @@ class MockWebProgress {
     this.documentRequest = null;
     this.isLoadingDocument = false;
     this.listener = null;
+    this.loadType = 0;
     this.progressListenerRemoved = false;
   }
 
@@ -59,15 +72,15 @@ class MockWebProgress {
     this.documentRequest = null;
 
     if (flag & Ci.nsIWebProgressListener.LOCATION_CHANGE_SAME_DOCUMENT) {
-      this.browsingContext.currentURI = TARGET_URI_WITH_HASH;
+      this.browsingContext.updateURI(TARGET_URI_WITH_HASH);
     } else if (flag & Ci.nsIWebProgressListener.LOCATION_CHANGE_ERROR_PAGE) {
-      this.browsingContext.currentURI = TARGET_URI_IS_ERROR_PAGE;
+      this.browsingContext.updateURI(TARGET_URI_ERROR_PAGE, true);
     }
 
     this.listener?.onLocationChange(
       this,
       this.documentRequest,
-      TARGET_URI_WITH_HASH,
+      this.browsingContext.currentURI,
       flag
     );
 
@@ -88,6 +101,7 @@ class MockWebProgress {
     this.browsingContext.currentWindowGlobal.isInitialDocument = isInitial;
 
     this.isLoadingDocument = true;
+    this.loadType = 0;
     const uri = isInitial ? INITIAL_URI : TARGET_URI;
     this.documentRequest = new MockRequest(uri);
 
@@ -95,14 +109,14 @@ class MockWebProgress {
       this,
       this.documentRequest,
       Ci.nsIWebProgressListener.STATE_START,
-      null
+      0
     );
 
     return new Promise(executeSoon);
   }
 
   sendStopState(options = {}) {
-    const { errorFlag = 0 } = options;
+    const { flag = 0, loadType = 0 } = options;
 
     this.browsingContext.currentURI = this.documentRequest.originalURI;
 
@@ -113,9 +127,16 @@ class MockWebProgress {
       this,
       this.documentRequest,
       Ci.nsIWebProgressListener.STATE_STOP,
-      errorFlag
+      flag
     );
 
+    if (loadType & LOAD_FLAG_ERROR_PAGE) {
+      this.loadType = 0x10000;
+      return this.sendLocationChange({
+        flag: Ci.nsIWebProgressListener.LOCATION_CHANGE_ERROR_PAGE,
+      });
+    }
+
     return new Promise(executeSoon);
   }
 }
@@ -123,30 +144,26 @@ class MockWebProgress {
 class MockTopContext {
   constructor(webProgress = null) {
     this.currentURI = CURRENT_URI;
-    this.currentWindowGlobal = { isInitialDocument: true };
+    this.currentWindowGlobal = {
+      isInitialDocument: true,
+      documentURI: CURRENT_URI,
+    };
     this.id = 7;
     this.top = this;
     this.webProgress = webProgress || new MockWebProgress(this);
   }
+
+  updateURI(uri, isError = false) {
+    this.currentURI = uri;
+    if (isError) {
+      this.currentWindowGlobal.documentURI = "about:neterror?e=errorMessage";
+    } else {
+      this.currentWindowGlobal.documentURI = uri;
+    }
+  }
 }
 
-const hasPromiseResolved = async function(promise) {
-  let resolved = false;
-  promise.finally(() => (resolved = true));
-  // Make sure microtasks have time to run.
-  await new Promise(resolve => Services.tm.dispatchToMainThread(resolve));
-  return resolved;
-};
-
-const hasPromiseRejected = async function(promise) {
-  let rejected = false;
-  promise.catch(() => (rejected = true));
-  // Make sure microtasks have time to run.
-  await new Promise(resolve => Services.tm.dispatchToMainThread(resolve));
-  return rejected;
-};
-
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentNoWindowGlobal() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -178,12 +195,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, INITIAL_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentNotLoaded() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -213,12 +228,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, INITIAL_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentLoadingAndNoAdditionalLoad() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -247,12 +260,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, INITIAL_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentFinishedLoadingNoAdditionalLoad() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -282,12 +293,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, INITIAL_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentLoadingAndAdditionalLoad() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -305,8 +314,7 @@ add_test(
 
     await webProgress.sendStopState();
 
-    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
-    await new Promise(resolve => setTimeout(resolve, 100));
+    await wait(100);
 
     await webProgress.sendStartState({ isInitial: false });
     await webProgress.sendStopState();
@@ -324,12 +332,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_initialDocumentFinishedLoadingAndAdditionalLoad() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -346,8 +352,7 @@ add_test(
       "waitForInitialNavigationCompleted has not resolved yet"
     );
 
-    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
-    await new Promise(resolve => setTimeout(resolve, 100));
+    await wait(100);
 
     await webProgress.sendStartState({ isInitial: false });
     await webProgress.sendStopState();
@@ -365,12 +370,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_notInitialDocumentNotLoading() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -399,12 +402,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_notInitialDocumentAlreadyLoading() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -433,12 +434,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(
+add_task(
   async function test_waitForInitialNavigation_notInitialDocumentFinishedLoading() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -448,9 +447,8 @@ add_test(
 
     ok(!webProgress.isLoadingDocument, "Document is not loading");
 
-    const { currentURI, targetURI } = await waitForInitialNavigationCompleted(
-      webProgress
-    );
+    const { currentURI, targetURI } =
+      await waitForInitialNavigationCompleted(webProgress);
 
     ok(!webProgress.isLoadingDocument, "Document is not loading");
     ok(
@@ -463,12 +461,10 @@ add_test(
       "Expected current URI has been set"
     );
     equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-    run_next_test();
   }
 );
 
-add_test(async function test_waitForInitialNavigation_resolveWhenStarted() {
+add_task(async function test_waitForInitialNavigation_resolveWhenStarted() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
@@ -489,11 +485,9 @@ add_test(async function test_waitForInitialNavigation_resolveWhenStarted() {
   );
   equal(currentURI.spec, CURRENT_URI.spec, "Expected current URI has been set");
   equal(targetURI.spec, INITIAL_URI.spec, "Expected target URI has been set");
-
-  run_next_test();
 });
 
-add_test(async function test_waitForInitialNavigation_crossOrigin() {
+add_task(async function test_waitForInitialNavigation_crossOrigin() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
@@ -522,11 +516,83 @@ add_test(async function test_waitForInitialNavigation_crossOrigin() {
   );
   equal(currentURI.spec, TARGET_URI.spec, "Expected current URI has been set");
   equal(targetURI.spec, TARGET_URI.spec, "Expected target URI has been set");
-
-  run_next_test();
 });
 
-add_test(async function test_ProgressListener_expectNavigation() {
+add_task(async function test_waitForInitialNavigation_unloadTimeout_default() {
+  const browsingContext = new MockTopContext();
+  const webProgress = browsingContext.webProgress;
+
+  // Stop the navigation on an initial page which is not loading anymore.
+  // This situation happens with new tabs on Android, even though they are on
+  // the initial document, they will not start another navigation on their own.
+  await webProgress.sendStartState({ isInitial: true });
+  await webProgress.sendStopState();
+
+  ok(!webProgress.isLoadingDocument, "Document is not loading");
+
+  const navigated = waitForInitialNavigationCompleted(webProgress);
+
+  // Start a timer longer than the timeout which will be used by
+  // waitForInitialNavigationCompleted, and check that navigated resolves first.
+  const waitForMoreThanDefaultTimeout = wait(
+    DEFAULT_UNLOAD_TIMEOUT * 1.5 * getUnloadTimeoutMultiplier()
+  );
+  await Promise.race([navigated, waitForMoreThanDefaultTimeout]);
+
+  ok(
+    await hasPromiseResolved(navigated),
+    "waitForInitialNavigationCompleted has resolved"
+  );
+
+  ok(!webProgress.isLoadingDocument, "Document is not loading");
+  ok(
+    webProgress.browsingContext.currentWindowGlobal.isInitialDocument,
+    "Document is still on the initial document"
+  );
+});
+
+add_task(async function test_waitForInitialNavigation_unloadTimeout_longer() {
+  const browsingContext = new MockTopContext();
+  const webProgress = browsingContext.webProgress;
+
+  // Stop the navigation on an initial page which is not loading anymore.
+  // This situation happens with new tabs on Android, even though they are on
+  // the initial document, they will not start another navigation on their own.
+  await webProgress.sendStartState({ isInitial: true });
+  await webProgress.sendStopState();
+
+  ok(!webProgress.isLoadingDocument, "Document is not loading");
+
+  const navigated = waitForInitialNavigationCompleted(webProgress, {
+    unloadTimeout: DEFAULT_UNLOAD_TIMEOUT * 3,
+  });
+
+  // Start a timer longer than the default timeout of the Navigate module.
+  // However here we used a custom timeout, so we expect that the navigation
+  // will not be done yet by the time this timer is done.
+  const waitForMoreThanDefaultTimeout = wait(
+    DEFAULT_UNLOAD_TIMEOUT * 1.5 * getUnloadTimeoutMultiplier()
+  );
+  await Promise.race([navigated, waitForMoreThanDefaultTimeout]);
+
+  // The promise should not have resolved because we didn't reached the custom
+  // timeout which is 3 times the default one.
+  ok(
+    !(await hasPromiseResolved(navigated)),
+    "waitForInitialNavigationCompleted has not resolved yet"
+  );
+
+  // The navigation should eventually resolve once we reach the custom timeout.
+  await navigated;
+
+  ok(!webProgress.isLoadingDocument, "Document is not loading");
+  ok(
+    webProgress.browsingContext.currentWindowGlobal.isInitialDocument,
+    "Document is still on the initial document"
+  );
+});
+
+add_task(async function test_ProgressListener_expectNavigation() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
@@ -537,8 +603,7 @@ add_test(async function test_ProgressListener_expectNavigation() {
   const navigated = progressListener.start();
 
   // Wait for unloadTimeout to finish in case it started
-  // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
-  await new Promise(resolve => setTimeout(resolve, 30));
+  await wait(30);
 
   ok(!(await hasPromiseResolved(navigated)), "Listener has not resolved yet");
 
@@ -546,11 +611,9 @@ add_test(async function test_ProgressListener_expectNavigation() {
   await webProgress.sendStopState();
 
   ok(await hasPromiseResolved(navigated), "Listener has resolved");
-
-  run_next_test();
 });
 
-add_test(
+add_task(
   async function test_ProgressListener_expectNavigation_initialDocumentFinishedLoading() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -567,8 +630,7 @@ add_test(
     await webProgress.sendStopState();
 
     // Wait for unloadTimeout to finish in case it started
-    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
-    await new Promise(resolve => setTimeout(resolve, 30));
+    await wait(30);
 
     ok(!(await hasPromiseResolved(navigated)), "Listener has not resolved yet");
 
@@ -576,12 +638,10 @@ add_test(
     await webProgress.sendStopState();
 
     ok(await hasPromiseResolved(navigated), "Listener has resolved");
-
-    run_next_test();
   }
 );
 
-add_test(async function test_ProgressListener_isStarted() {
+add_task(async function test_ProgressListener_isStarted() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
@@ -593,11 +653,9 @@ add_test(async function test_ProgressListener_isStarted() {
 
   progressListener.stop();
   ok(!progressListener.isStarted);
-
-  run_next_test();
 });
 
-add_test(async function test_ProgressListener_notWaitForExplicitStart() {
+add_task(async function test_ProgressListener_notWaitForExplicitStart() {
   // Create a webprogress and start it before creating the progress listener.
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
@@ -615,11 +673,9 @@ add_test(async function test_ProgressListener_notWaitForExplicitStart() {
     await hasPromiseResolved(navigated),
     "Listener has resolved after initial navigation"
   );
-
-  run_next_test();
 });
 
-add_test(async function test_ProgressListener_waitForExplicitStart() {
+add_task(async function test_ProgressListener_waitForExplicitStart() {
   // Create a webprogress and start it before creating the progress listener.
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
@@ -651,11 +707,39 @@ add_test(async function test_ProgressListener_waitForExplicitStart() {
     await hasPromiseResolved(navigated),
     "Listener resolved after finishing the new navigation"
   );
-
-  run_next_test();
 });
 
-add_test(
+add_task(
+  async function test_ProgressListener_waitForExplicitStartAndResolveWhenStarted() {
+    // Create a webprogress and start it before creating the progress listener.
+    const browsingContext = new MockTopContext();
+    const webProgress = browsingContext.webProgress;
+    await webProgress.sendStartState();
+
+    // Create the progress listener for a webprogress already in a navigation.
+    const progressListener = new ProgressListener(webProgress, {
+      resolveWhenStarted: true,
+      waitForExplicitStart: true,
+    });
+    const navigated = progressListener.start();
+
+    // Send stop state to complete the initial navigation
+    await webProgress.sendStopState();
+    ok(
+      !(await hasPromiseResolved(navigated)),
+      "Listener has not resolved after initial navigation"
+    );
+
+    // Start a new navigation
+    await webProgress.sendStartState();
+    ok(
+      await hasPromiseResolved(navigated),
+      "Listener resolved after starting the new navigation"
+    );
+  }
+);
+
+add_task(
   async function test_ProgressListener_resolveWhenNavigatingInsideDocument() {
     const browsingContext = new MockTopContext();
     const webProgress = browsingContext.webProgress;
@@ -683,12 +767,27 @@ add_test(
       TARGET_URI_WITH_HASH.spec,
       "Expected target URI has been set"
     );
-
-    run_next_test();
   }
 );
 
-add_test(async function test_ProgressListener_navigationRejectedOnErrorPage() {
+add_task(async function test_ProgressListener_ignoreCacheError() {
+  const browsingContext = new MockTopContext();
+  const webProgress = browsingContext.webProgress;
+
+  const progressListener = new ProgressListener(webProgress);
+  const navigated = progressListener.start();
+
+  ok(!(await hasPromiseResolved(navigated)), "Listener has not resolved");
+
+  await webProgress.sendStartState();
+  await webProgress.sendStopState({
+    flag: Cr.NS_ERROR_PARSED_DATA_CACHED,
+  });
+
+  ok(await hasPromiseResolved(navigated), "Listener has resolved");
+});
+
+add_task(async function test_ProgressListener_navigationRejectedOnErrorPage() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
@@ -708,26 +807,106 @@ add_test(async function test_ProgressListener_navigationRejectedOnErrorPage() {
     await hasPromiseRejected(navigated),
     "Listener has rejected in location change for error page"
   );
-
-  run_next_test();
 });
 
-add_test(async function test_ProgressListener_navigationRejectedOnStopState() {
+add_task(
+  async function test_ProgressListener_navigationRejectedOnStopStateErrorPage() {
+    const browsingContext = new MockTopContext();
+    const webProgress = browsingContext.webProgress;
+
+    const progressListener = new ProgressListener(webProgress, {
+      waitForExplicitStart: false,
+    });
+    const navigated = progressListener.start();
+
+    await webProgress.sendStartState();
+    await webProgress.sendStopState({
+      flag: Cr.NS_ERROR_MALWARE_URI,
+      loadType: LOAD_FLAG_ERROR_PAGE,
+    });
+
+    ok(
+      await hasPromiseRejected(navigated),
+      "Listener has rejected in stop state for erroneous navigation"
+    );
+  }
+);
+
+add_task(
+  async function test_ProgressListener_navigationRejectedOnStopStateAndAborted() {
+    const browsingContext = new MockTopContext();
+    const webProgress = browsingContext.webProgress;
+
+    for (const flag of [Cr.NS_BINDING_ABORTED, Cr.NS_ERROR_ABORT]) {
+      const progressListener = new ProgressListener(webProgress, {
+        waitForExplicitStart: false,
+      });
+      const navigated = progressListener.start();
+
+      await webProgress.sendStartState();
+      await webProgress.sendStopState({ flag });
+
+      ok(
+        await hasPromiseRejected(navigated),
+        "Listener has rejected in stop state for erroneous navigation"
+      );
+    }
+  }
+);
+
+add_task(async function test_ProgressListener_stopIfStarted() {
   const browsingContext = new MockTopContext();
   const webProgress = browsingContext.webProgress;
 
+  const progressListener = new ProgressListener(webProgress);
+  const navigated = progressListener.start();
+
+  progressListener.stopIfStarted();
+  ok(!(await hasPromiseResolved(navigated)), "Listener has not resolved");
+
+  await webProgress.sendStartState();
+  progressListener.stopIfStarted();
+  ok(await hasPromiseResolved(navigated), "Listener has resolved");
+});
+
+add_task(async function test_ProgressListener_stopIfStarted_alreadyStarted() {
+  // Create an already navigating browsing context.
+  const browsingContext = new MockTopContext();
+  const webProgress = browsingContext.webProgress;
+  await webProgress.sendStartState();
+
+  // Create a progress listener which accepts already ongoing navigations.
   const progressListener = new ProgressListener(webProgress, {
     waitForExplicitStart: false,
   });
   const navigated = progressListener.start();
 
-  await webProgress.sendStartState();
-  await webProgress.sendStopState({ errorFlag: Cr.NS_BINDING_ABORTED });
-
-  ok(
-    await hasPromiseRejected(navigated),
-    "Listener has rejected in stop state for erroneous navigation"
-  );
-
-  run_next_test();
+  // stopIfStarted should stop the listener because of the ongoing navigation.
+  progressListener.stopIfStarted();
+  ok(await hasPromiseResolved(navigated), "Listener has resolved");
 });
+
+add_task(
+  async function test_ProgressListener_stopIfStarted_alreadyStarted_waitForExplicitStart() {
+    // Create an already navigating browsing context.
+    const browsingContext = new MockTopContext();
+    const webProgress = browsingContext.webProgress;
+    await webProgress.sendStartState();
+
+    // Create a progress listener which rejects already ongoing navigations.
+    const progressListener = new ProgressListener(webProgress, {
+      waitForExplicitStart: true,
+    });
+    const navigated = progressListener.start();
+
+    // stopIfStarted will not stop the listener for the existing navigation.
+    progressListener.stopIfStarted();
+    ok(!(await hasPromiseResolved(navigated)), "Listener has not resolved");
+
+    // stopIfStarted will stop the listener when called after starting a new
+    // navigation.
+    await webProgress.sendStartState();
+    progressListener.stopIfStarted();
+    ok(await hasPromiseResolved(navigated), "Listener has resolved");
+  }
+);
