@@ -8,51 +8,154 @@ function usage {
 Usage: $(basename "$0") -h # Displays this usage/help text
 Usage: $(basename "$0") -x # lists exit codes
 Usage: $(basename "$0") [-p product]
-           [-r existing_repo_dir]
            # Use mozilla-central builds to check HSTS & HPKP
            [--use-mozilla-central]
            # Use archive.m.o instead of the taskcluster index to get xpcshell
            [--use-ftp-builds]
+           # Use git rather than hg. Using git does not currently support cloning (use
+           # --skip-repo as well).
+           [--use-git]
            # One (or more) of the following actions must be specified.
-           --hsts | --hpkp | --remote-settings | --suffix-list
+           --hsts | --hpkp | --remote-settings | --suffix-list | --mobile-experiments | --ct-logs
            -b branch
+           # The name of top source directory to use for the repository clone.
+           [-t topsrcdir]
+           # Skips cloning of the repository.
+           [--skip-clone]
+           # Performs a dry run - no commits are created.
+           [-n]
+           # Skips pushing of the repository - create a commit but does not try
+           # to push it.
+           [--skip-push]
 
 EOF
 }
 
+# Defaults
 PRODUCT="firefox"
-BRANCH=""
-PLATFORM_EXT="tar.bz2"
-UNPACK_CMD="tar jxf"
+DRY_RUN=false
 CLOSED_TREE=false
 DONTBUILD=false
 APPROVAL=false
-COMMIT_AUTHOR='ffxbld <ffxbld@mozilla.com>'
-REPODIR=''
+
+DO_PRELOAD_PINSET=false
+DO_HSTS=false
+DO_HPKP=false
+DO_REMOTE_SETTINGS=false
+DO_SUFFIX_LIST=false
+DO_MOBILE_EXPERIMENTS=false
+DO_CT_LOGS=false
+
+CLONE_REPO=true
 HGHOST="hg.mozilla.org"
 STAGEHOST="archive.mozilla.org"
-WGET="wget -nv"
-UNTAR="tar -zxf"
-DIFF="$(command -v diff) -u"
-BASEDIR="${HOME}"
-
-SCRIPTDIR="$(realpath "$(dirname "$0")")"
-HG="$(command -v hg)"
-DATADIR="${BASEDIR}/data"
-mkdir -p "${DATADIR}"
 
 USE_MC=false
 USE_TC=true
+USE_GIT=false
+SKIP_PUSH=false
+
+# Parse our command-line options.
+while [ $# -gt 0 ]; do
+  case "$1" in
+    -h) usage; exit 0 ;;
+    -p) PRODUCT="$2"; shift ;;
+    -b) BRANCH="$2"; shift ;;
+    -n) DRY_RUN=true ;;
+    -c) CLOSED_TREE=true ;;
+    -d) DONTBUILD=true ;;
+    -a) APPROVAL=true ;;
+    --pinset) DO_PRELOAD_PINSET=true ;;
+    --hsts) DO_HSTS=true ;;
+    --hpkp) DO_HPKP=true ;;
+    --remote-settings) DO_REMOTE_SETTINGS=true ;;
+    --suffix-list) DO_SUFFIX_LIST=true ;;
+    --mobile-experiments) DO_MOBILE_EXPERIMENTS=true ;;
+    --ct-logs) DO_CT_LOGS=true ;;
+    --skip-clone) CLONE_REPO=false ;;
+    --skip-push) SKIP_PUSH=true ;;
+    -t) TOPSRCDIR="$2"; shift ;;
+    --use-mozilla-central) USE_MC=true ;;
+    --use-ftp-builds) USE_TC=false ;;
+    --use-git) USE_GIT=true ;;
+    -*) usage
+      exit 11 ;;
+    *)  break ;; # terminate while loop
+  esac
+  shift
+done
+
+# Must supply a code branch to work with.
+if [ "${BRANCH}" == "" ]; then
+  echo "Error: You must specify a branch with -b branchname." >&2
+  usage
+  exit 12
+fi
+
+# Must choose at least one update action.
+if [ "$DO_HSTS" == "false" ] && [ "$DO_HPKP" == "false" ] && [ "$DO_REMOTE_SETTINGS" == "false" ] && [ "$DO_SUFFIX_LIST" == "false" ] && [ "$DO_MOBILE_EXPERIMENTS" == false ] && [ "$DO_CT_LOGS" == false ]
+then
+  echo "Error: you must specify at least one action from: --hsts, --hpkp, --remote-settings, or --suffix-list" >&2
+  usage
+  exit 13
+fi
+
+# per-product constants
+case "${PRODUCT}" in
+  thunderbird)
+    COMMIT_AUTHOR="tbirdbld <tbirdbld@thunderbird.net>"
+    ;;
+  firefox)
+    ;;
+  *)
+    echo "Error: Invalid product specified"
+    usage
+    exit 14
+    ;;
+esac
+
+if [ "${TOPSRCDIR}" == "" ]; then
+  TOPSRCDIR="$(basename "${BRANCH}")"
+fi
+
+case "${BRANCH}" in
+  mozilla-central|comm-central|try )
+    HGREPO="https://${HGHOST}/${BRANCH}"
+    ;;
+  mozilla-*|comm-* )
+    HGREPO="https://${HGHOST}/releases/${BRANCH}"
+    ;;
+  * )
+    HGREPO="https://${HGHOST}/projects/${BRANCH}"
+    ;;
+esac
+
+BROWSER_ARCHIVE="target.tar.xz"
+TESTS_ARCHIVE="target.common.tests.tar.gz"
+
+UNPACK_CMD="tar Jxf"
+COMMIT_AUTHOR='ffxbld <ffxbld@mozilla.com>'
+WGET="wget -nv"
+UNTAR="tar -zxf"
+DIFF="$(command -v diff) -u"
 JQ="$(command -v jq)"
 
-DO_HSTS=false
+if [ "${USE_GIT}" == "true" ]; then
+  GIT="$(command -v git)"
+else
+  HG="$(command -v hg)"
+fi
+
+BASEDIR="${HOME}"
+SCRIPTDIR="$(realpath "$(dirname "$0")")"
+DATADIR="${BASEDIR}/data"
+
 HSTS_PRELOAD_SCRIPT="${SCRIPTDIR}/getHSTSPreloadList.js"
 HSTS_PRELOAD_ERRORS="nsSTSPreloadList.errors"
 HSTS_PRELOAD_INC_OLD="${DATADIR}/nsSTSPreloadList.inc"
 HSTS_PRELOAD_INC_NEW="${BASEDIR}/${PRODUCT}/nsSTSPreloadList.inc"
 HSTS_UPDATED=false
 
-DO_HPKP=false
 HPKP_PRELOAD_SCRIPT="${SCRIPTDIR}/genHPKPStaticPins.js"
 HPKP_PRELOAD_ERRORS="StaticHPKPins.errors"
 HPKP_PRELOAD_JSON="${DATADIR}/PreloadedHPKPins.json"
@@ -61,31 +164,43 @@ HPKP_PRELOAD_INPUT="${DATADIR}/${HPKP_PRELOAD_INC}"
 HPKP_PRELOAD_OUTPUT="${DATADIR}/${HPKP_PRELOAD_INC}.out"
 HPKP_UPDATED=false
 
-DO_REMOTE_SETTINGS=false
 REMOTE_SETTINGS_SERVER=''
-REMOTE_SETTINGS_INPUT="${DATADIR}/remote-settings.in"
-REMOTE_SETTINGS_OUTPUT="${DATADIR}/remote-settings.out"
-REMOTE_SETTINGS_DIR="/services/settings/dumps"
+REMOTE_SETTINGS_DIR="${TOPSRCDIR}/services/settings/dumps"
 REMOTE_SETTINGS_UPDATED=false
 
-DO_SUFFIX_LIST=false
-GITHUB_SUFFIX_URL="https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat"
-GITHUB_SUFFIX_LOCAL="public_suffix_list.dat"
+PUBLIC_SUFFIX_URL="https://publicsuffix.org/list/public_suffix_list.dat"
+PUBLIC_SUFFIX_LOCAL="public_suffix_list.dat"
 HG_SUFFIX_LOCAL="effective_tld_names.dat"
 HG_SUFFIX_PATH="/netwerk/dns/${HG_SUFFIX_LOCAL}"
 SUFFIX_LIST_UPDATED=false
 
+EXPERIMENTER_URL="https://experimenter.services.mozilla.com/api/v6/experiments-first-run/"
+FENIX_INITIAL_EXPERIMENTS="mobile/android/fenix/app/src/main/res/raw/initial_experiments.json"
+FOCUS_INITIAL_EXPERIMENTS="mobile/android/focus-android/app/src/main/res/raw/initial_experiments.json"
+MOBILE_EXPERIMENTS_UPDATED=false
+
+CT_LOG_UPDATE_SCRIPT="${SCRIPTDIR}/getCTKnownLogs.py"
+
 ARTIFACTS_DIR="${ARTIFACTS_DIR:-.}"
 # Defaults
 HSTS_DIFF_ARTIFACT="${ARTIFACTS_DIR}/${HSTS_DIFF_ARTIFACT:-"nsSTSPreloadList.diff"}"
 HPKP_DIFF_ARTIFACT="${ARTIFACTS_DIR}/${HPKP_DIFF_ARTIFACT:-"StaticHPKPins.h.diff"}"
 REMOTE_SETTINGS_DIFF_ARTIFACT="${ARTIFACTS_DIR}/${REMOTE_SETTINGS_DIFF_ARTIFACT:-"remote-settings.diff"}"
 SUFFIX_LIST_DIFF_ARTIFACT="${ARTIFACTS_DIR}/${SUFFIX_LIST_DIFF_ARTIFACT:-"effective_tld_names.diff"}"
+EXPERIMENTER_DIFF_ARTIFACT="${ARTIFACTS_DIR}/initial_experiments.diff"
 
 # duplicate the functionality of taskcluster-lib-urls, but in bash..
 queue_base="$TASKCLUSTER_ROOT_URL/api/queue/v1"
 index_base="$TASKCLUSTER_ROOT_URL/api/index/v1"
 
+function create_repo_diff() {
+  if [ "${USE_GIT}" == "true" ]; then
+    ${GIT} -C "${TOPSRCDIR}" diff -u "$1" > "$2"
+  else
+    ${HG} -R "${TOPSRCDIR}" diff "$1" > "$2"
+  fi
+}
+
 # Cleanup common artifacts.
 function preflight_cleanup {
   cd "${BASEDIR}"
@@ -97,7 +212,7 @@ function download_shared_artifacts_from_ftp {
 
   # Download everything we need to run js with xpcshell
   echo "INFO: Downloading all the necessary pieces from ${STAGEHOST}..."
-  ARTIFACT_DIR="nightly/latest-${REPODIR}"
+  ARTIFACT_DIR="nightly/latest-${BRANCH}"
   if [ "${USE_MC}" == "true" ]; then
     ARTIFACT_DIR="nightly/latest-mozilla-central"
   fi
@@ -117,7 +232,7 @@ function download_shared_artifacts_from_tc {
 
   # Download everything we need to run js with xpcshell
   echo "INFO: Downloading all the necessary pieces from the taskcluster index..."
-  TASKID_URL="$index_base/task/gecko.v2.${REPODIR}.shippable.latest.${PRODUCT}.linux64-opt"
+  TASKID_URL="$index_base/task/gecko.v2.${BRANCH}.shippable.latest.${PRODUCT}.linux64-opt"
   if [ "${USE_MC}" == "true" ]; then
     TASKID_URL="$index_base/task/gecko.v2.mozilla-central.shippable.latest.${PRODUCT}.linux64-opt"
   fi
@@ -268,16 +383,16 @@ function compare_suffix_lists {
   HG_SUFFIX_URL="${HGREPO}/raw-file/default/${HG_SUFFIX_PATH}"
   cd "${BASEDIR}"
 
-  echo "INFO: ${WGET} -O ${GITHUB_SUFFIX_LOCAL} ${GITHUB_SUFFIX_URL}"
-  rm -f "${GITHUB_SUFFIX_LOCAL}"
-  ${WGET} -O "${GITHUB_SUFFIX_LOCAL}" "${GITHUB_SUFFIX_URL}"
+  echo "INFO: ${WGET} -O ${PUBLIC_SUFFIX_LOCAL} ${PUBLIC_SUFFIX_URL}"
+  rm -f "${PUBLIC_SUFFIX_LOCAL}"
+  ${WGET} -O "${PUBLIC_SUFFIX_LOCAL}" "${PUBLIC_SUFFIX_URL}"
 
   echo "INFO: ${WGET} -O ${HG_SUFFIX_LOCAL} ${HG_SUFFIX_URL}"
   rm -f "${HG_SUFFIX_LOCAL}"
   ${WGET} -O "${HG_SUFFIX_LOCAL}" "${HG_SUFFIX_URL}"
 
-  echo "INFO: diffing in-tree suffix list against the suffix list from AMO..."
-  ${DIFF} ${GITHUB_SUFFIX_LOCAL} ${HG_SUFFIX_LOCAL} | tee "${SUFFIX_LIST_DIFF_ARTIFACT}"
+  echo "INFO: diffing in-tree suffix list against the suffix list from publicsuffix.org"
+  ${DIFF} ${PUBLIC_SUFFIX_LOCAL} ${HG_SUFFIX_LOCAL} | tee "${SUFFIX_LIST_DIFF_ARTIFACT}"
   if [ -s "${SUFFIX_LIST_DIFF_ARTIFACT}" ]
   then
     return 0
@@ -286,6 +401,8 @@ function compare_suffix_lists {
 }
 
 function compare_remote_settings_files {
+  # cd "${TOPSRCDIR}"
+
   REMOTE_SETTINGS_SERVER="https://firefox.settings.services.mozilla.com/v1"
 
   # 1. List remote settings collections from server.
@@ -295,27 +412,28 @@ function compare_remote_settings_files {
     # 2. For each entry ${bucket, collection, last_modified}
   while IFS="/" read -r bucket collection last_modified; do
 
-    # 3. Download the dump from HG into REMOTE_SETTINGS_INPUT folder
-    hg_dump_url="${HGREPO}/raw-file/default${REMOTE_SETTINGS_DIR}/${bucket}/${collection}.json"
-    local_location_input="$REMOTE_SETTINGS_INPUT/${bucket}/${collection}.json"
-    mkdir -p "$REMOTE_SETTINGS_INPUT/${bucket}"
-    ${WGET} -qO "$local_location_input" "$hg_dump_url"
-    if [ $? -eq 8 ]; then
-      # We don't keep any dump for this collection, skip it.
-      # Try to clean up in case no collection in this bucket has dump.
-      rmdir "$REMOTE_SETTINGS_INPUT/${bucket}" --ignore-fail-on-non-empty
+    # 3. Check to see if the collection exists in the dump directory of the repository,
+    #    if it does not then we aren't keeping the dump, and so we skip it.
+    local_dump_file="${REMOTE_SETTINGS_DIR}/${bucket}/${collection}.json"
+    if [ ! -r "${local_dump_file}" ]; then
       continue
     fi
 
-    # 4. Download server version into REMOTE_SETTINGS_OUTPUT folder
+    # 4. Download server version into REMOTE_SETTINGS_DIR folder
     remote_records_url="$REMOTE_SETTINGS_SERVER/buckets/${bucket}/collections/${collection}/changeset?_expected=${last_modified}"
-    local_location_output="$REMOTE_SETTINGS_OUTPUT/${bucket}/${collection}.json"
-    mkdir -p "$REMOTE_SETTINGS_OUTPUT/${bucket}"
-    ${WGET} -qO- "$remote_records_url" | ${JQ} '{"data": .changes, "timestamp": .timestamp}' > "${local_location_output}"
+    local_location_output="$REMOTE_SETTINGS_DIR/${bucket}/${collection}.json"
+
+    # We sort both the keys and the records in search-config-v2 to make it
+    # easier to read and to experiment with making changes via the dump file.
+    if [ "${collection}" = "search-config-v2" ]; then
+      ${WGET} -qO- "$remote_records_url" | ${JQ} --sort-keys '{"data": .changes | sort_by(.recordType, .identifier), "timestamp": .timestamp}' > "${local_location_output}"
+    else
+      ${WGET} -qO- "$remote_records_url" | ${JQ} '{"data": .changes, "timestamp": .timestamp}' > "${local_location_output}"
+    fi
 
     # 5. Download attachments if needed.
     if [ "${bucket}" = "blocklists" ] && [ "${collection}" = "addons-bloomfilters" ]; then
-      # Find the attachment with the most recent generation_time, like _updateMLBF in Blocklist.jsm.
+      # Find the attachment with the most recent generation_time, like _updateMLBF in Blocklist.sys.mjs.
       # The server should return one "bloomfilter-base" record, but in case it returns multiple,
       # return the most recent one. The server may send multiple entries if we ever decide to use
       # the "filter_expression" feature of Remote Settings to send different records to specific
@@ -324,14 +442,37 @@ function compare_remote_settings_files {
       # Note that "attachment_type" and "generation_time" are specific to addons-bloomfilters.
       update_remote_settings_attachment "${bucket}" "${collection}" addons-mlbf.bin \
         'map(select(.attachment_type == "bloomfilter-base")) | sort_by(.generation_time) | last'
+      update_remote_settings_attachment "${bucket}" "${collection}" softblocks-addons-mlbf.bin \
+        'map(select(.attachment_type == "softblocks-bloomfilter-base")) | sort_by(.generation_time) | last'
+    fi
+    # TODO: Bug 1873448. This cannot handle new/removed files currently, due to the
+    # build system making it difficult.
+    if [ "${bucket}" = "main" ] && [ "${collection}" = "search-config-icons" ]; then
+      ${JQ} -r '.data[] | .id' < "${local_location_output}" |\
+      while read -r id; do
+        # We do not want quotes around ${id}
+        # shellcheck disable=SC2086
+        update_remote_settings_attachment "${bucket}" "${collection}" ${id} ".[] | select(.id == \"${id}\")"
+      done
     fi
-    # Here is an example to download an attachment with record identifier "ID":
-    # update_remote_settings_attachment "${bucket}" "${collection}" ID '.[] | select(.id == "ID")'
     # NOTE: The downloaded data is not validated. xpcshell should be used for that.
+    
+    # bug 1959683: remote settings update can add untracked search-config-icons
+    # It is not safe to take these (see https://bugzilla.mozilla.org/show_bug.cgi?id=1873448)
+    # If they are around as untracked files when `arc diff` runs, that command will fail.
+    # (We explicitly don't want to use `arc diff --allow-untracked` to avoid accidentally
+    # missing files from other updates - we'd rather the job fail.)
+    if [ "${USE_GIT}" == "true" ]; then
+      ${GIT} -C "${TOPSRCDIR}" clean -f -d services/settings/dumps/main/search-config-icons
+    else
+      ${HG} --cwd "${TOPSRCDIR}" purge services/settings/dumps/main/search-config-icons
+    fi
   done
 
   echo "INFO: diffing old/new remote settings dumps..."
-  ${DIFF} -r "${REMOTE_SETTINGS_INPUT}" "${REMOTE_SETTINGS_OUTPUT}" > "${REMOTE_SETTINGS_DIFF_ARTIFACT}"
+  create_repo_diff "${REMOTE_SETTINGS_DIR}" "${REMOTE_SETTINGS_DIFF_ARTIFACT}"
+
+  # cd "${BASEDIR}"
   if [ -s "${REMOTE_SETTINGS_DIFF_ARTIFACT}" ]
   then
     return 0
@@ -341,7 +482,7 @@ function compare_remote_settings_files {
 
 # Helper for compare_remote_settings_files to download attachments from remote settings.
 # The format and location is documented at:
-# https://firefox-source-docs.mozilla.org/services/common/services/RemoteSettings.html#packaging-attachments
+# https://firefox-source-docs.mozilla.org/services/settings/index.html#services-packaging-attachments
 function update_remote_settings_attachment() {
   local bucket=$1
   local collection=$2
@@ -349,84 +490,107 @@ function update_remote_settings_attachment() {
   # $4 is a jq filter on the arrays that should return one record with the attachment
   local jq_attachment_selector=".data | map(select(.attachment)) | $4"
 
-  # These paths match _readAttachmentDump in services/settings/Attachments.jsm.
+  # These paths match _readAttachmentDump in services/settings/Attachments.sys.mjs.
   local path_to_attachment="${bucket}/${collection}/${attachment_id}"
   local path_to_meta="${bucket}/${collection}/${attachment_id}.meta.json"
-  local old_meta="$REMOTE_SETTINGS_INPUT/${path_to_meta}"
-  local new_meta="$REMOTE_SETTINGS_OUTPUT/${path_to_meta}"
+  local meta_file="${REMOTE_SETTINGS_DIR}/${path_to_meta}"
 
   # Those files should have been created by compare_remote_settings_files before the function call.
-  local local_location_input="$REMOTE_SETTINGS_INPUT/${bucket}/${collection}.json"
-  local local_location_output="$REMOTE_SETTINGS_OUTPUT/${bucket}/${collection}.json"
+  local source_collection_location="${REMOTE_SETTINGS_DIR}/${bucket}/${collection}.json"
 
-  # Compute the metadata based on already-downloaded records.
-  mkdir -p "$REMOTE_SETTINGS_INPUT/${bucket}/${collection}"
-  ${JQ} -cj <"$local_location_input" "${jq_attachment_selector}" > "${old_meta}"
-  mkdir -p "$REMOTE_SETTINGS_OUTPUT/${bucket}/${collection}"
-  ${JQ} -cj <"$local_location_output" "${jq_attachment_selector}" > "${new_meta}"
-
-  if cmp --silent "${old_meta}" "${new_meta}" ; then
+  # Exact the metadata for this attachment from the already downloaded collection,
+  # and compare with our current metadata to see if the attachment has changed or not.
+  # Uses cmp for fast compare (rather than repository tools).
+  if ${JQ} -cj "${jq_attachment_selector}" < "${source_collection_location}" | cmp --silent - "${meta_file}"; then
     # Metadata not changed, don't bother downloading the attachments themselves.
     return
   fi
   # Metadata changed. Download attachments.
 
+  # Save the metadata.
+  ${JQ} -cj <"${source_collection_location}" "${jq_attachment_selector}" > "${meta_file}"
+
   echo "INFO: Downloading updated remote settings dump: ${bucket}/${collection}/${attachment_id}"
 
-  # Overwrited old_meta with the actual file from the repo. The content should be equivalent,
-  # but can have minor differences (e.g. different line endings) if the checked in file was not
-  # generated by this script (e.g. manually checked in).
-  ${WGET} -qO "${old_meta}" "${HGREPO}/raw-file/default${REMOTE_SETTINGS_DIR}/${path_to_meta}"
-
-  ${WGET} -qO "${REMOTE_SETTINGS_INPUT}/${path_to_attachment}" "${HGREPO}/raw-file/default${REMOTE_SETTINGS_DIR}/${path_to_attachment}"
-
   if [ -z "${ATTACHMENT_BASE_URL}" ] ; then
     ATTACHMENT_BASE_URL=$(${WGET} -qO- "${REMOTE_SETTINGS_SERVER}" | ${JQ} -r .capabilities.attachments.base_url)
   fi
-  attachment_path_from_meta=$(${JQ} -r < "${new_meta}" .attachment.location)
-  ${WGET} -qO "${REMOTE_SETTINGS_OUTPUT}/${path_to_attachment}" "${ATTACHMENT_BASE_URL}${attachment_path_from_meta}"
+  attachment_path_from_meta=$(${JQ} -r < "${meta_file}" .attachment.location)
+  ${WGET} -qO "${REMOTE_SETTINGS_DIR}/${path_to_attachment}" "${ATTACHMENT_BASE_URL}${attachment_path_from_meta}"
+}
+
+function compare_mobile_experiments() {
+  echo "INFO ${WGET} ${EXPERIMENTER_URL}"
+  ${WGET} -O experiments.json "${EXPERIMENTER_URL}"
+  ${WGET} -O fenix-experiments-old.json "${HGREPO}/raw-file/default/${FENIX_INITIAL_EXPERIMENTS}"
+  ${WGET} -O focus-experiments-old.json "${HGREPO}/raw-file/default/${FOCUS_INITIAL_EXPERIMENTS}"
+
+  # shellcheck disable=SC2016
+  ${JQ} --arg APP_NAME fenix '{"data":map(select(.appName == $APP_NAME))}' < experiments.json > fenix-experiments-new.json
+  # shellcheck disable=SC2016
+  ${JQ} --arg APP_NAME focus_android '{"data":map(select(.appName == $APP_NAME))}' < experiments.json > focus-experiments-new.json
+
+  ( ${DIFF} fenix-experiments-old.json fenix-experiments-new.json; ${DIFF} focus-experiments-old.json focus-experiments-new.json ) > "${EXPERIMENTER_DIFF_ARTIFACT}"
+  if [ -s "${EXPERIMENTER_DIFF_ARTIFACT}" ]; then
+    return 0
+  else
+    # no change
+    return 1
+  fi
+}
+
+function update_ct_logs() {
+  echo "INFO: Updating CT logs..."
+  "${TOPSRCDIR}"/mach python "${CT_LOG_UPDATE_SCRIPT}"
 }
 
 # Clones an hg repo
 function clone_repo {
   cd "${BASEDIR}"
-  if [ ! -d "${REPODIR}" ]; then
-    ${HG} robustcheckout --sharebase /tmp/hg-store -b default "${HGREPO}" "${REPODIR}"
+  if [ ! -d "${TOPSRCDIR}" ]; then
+    ${HG} robustcheckout --sharebase /tmp/hg-store -b default "${HGREPO}" "${TOPSRCDIR}"
   fi
 
-  ${HG} -R "${REPODIR}" pull
-  ${HG} -R "${REPODIR}" update -C default
+  ${HG} -R "${TOPSRCDIR}" pull
+  ${HG} -R "${TOPSRCDIR}" update -C default
 }
 
 # Copies new HSTS files in place, and commits them.
 function stage_hsts_files {
   cd "${BASEDIR}"
-  cp -f "${HSTS_PRELOAD_INC_NEW}" "${REPODIR}/security/manager/ssl/"
+  cp -f "${HSTS_PRELOAD_INC_NEW}" "${TOPSRCDIR}/security/manager/ssl/"
 }
 
 function stage_hpkp_files {
   cd "${BASEDIR}"
-  cp -f "${HPKP_PRELOAD_OUTPUT}" "${REPODIR}/security/manager/ssl/${HPKP_PRELOAD_INC}"
-}
-
-function stage_remote_settings_files {
-  cd "${BASEDIR}"
-  cp -a "${REMOTE_SETTINGS_OUTPUT}"/* "${REPODIR}${REMOTE_SETTINGS_DIR}"
+  cp -f "${HPKP_PRELOAD_OUTPUT}" "${TOPSRCDIR}/security/manager/ssl/${HPKP_PRELOAD_INC}"
 }
 
 function stage_tld_suffix_files {
   cd "${BASEDIR}"
-  cp -a "${GITHUB_SUFFIX_LOCAL}" "${REPODIR}/${HG_SUFFIX_PATH}"
+  cp -a "${PUBLIC_SUFFIX_LOCAL}" "${TOPSRCDIR}/${HG_SUFFIX_PATH}"
+}
+
+function stage_mobile_experiments_files {
+  cd "${BASEDIR}"
+
+  cp fenix-experiments-new.json "${TOPSRCDIR}/${FENIX_INITIAL_EXPERIMENTS}"
+  cp focus-experiments-new.json "${TOPSRCDIR}/${FOCUS_INITIAL_EXPERIMENTS}"
 }
 
 # Push all pending commits to Phabricator
 function push_repo {
-  cd "${REPODIR}"
+  if [ "${SKIP_PUSH}" == "true" ]; then
+    echo "Skipping push due to --skip-push"
+    return 0
+  fi
+
+  cd "${TOPSRCDIR}"
   if [ ! -r "${HOME}/.arcrc" ]
   then
     return 1
   fi
-  if ! ARC=$(command -v arc)
+  if ! ARC=$(command -v arc) && ! ARC=$(command -v arcanist)
   then
     return 1
   fi
@@ -451,80 +615,17 @@ function push_repo {
 
 # Main
 
-# Parse our command-line options.
-while [ $# -gt 0 ]; do
-  case "$1" in
-    -h) usage; exit 0 ;;
-    -p) PRODUCT="$2"; shift ;;
-    -b) BRANCH="$2"; shift ;;
-    -n) DRY_RUN=true ;;
-    -c) CLOSED_TREE=true ;;
-    -d) DONTBUILD=true ;;
-    -a) APPROVAL=true ;;
-    --pinset) DO_PRELOAD_PINSET=true ;;
-    --hsts) DO_HSTS=true ;;
-    --hpkp) DO_HPKP=true ;;
-    --remote-settings) DO_REMOTE_SETTINGS=true ;;
-    --suffix-list) DO_SUFFIX_LIST=true ;;
-    -r) REPODIR="$2"; shift ;;
-    --use-mozilla-central) USE_MC=true ;;
-    --use-ftp-builds) USE_TC=false ;;
-    -*) usage
-      exit 11 ;;
-    *)  break ;; # terminate while loop
-  esac
-  shift
-done
-
-# Must supply a code branch to work with.
-if [ "${BRANCH}" == "" ]; then
-  echo "Error: You must specify a branch with -b branchname." >&2
-  usage
-  exit 12
-fi
-
-# Must choose at least one update action.
-if [ "$DO_HSTS" == "false" ] && [ "$DO_HPKP" == "false" ] && [ "$DO_REMOTE_SETTINGS" == "false" ] && [ "$DO_SUFFIX_LIST" == "false" ]
-then
-  echo "Error: you must specify at least one action from: --hsts, --hpkp, --remote-settings, or --suffix-list" >&2
-  usage
-  exit 13
-fi
-
-# per-product constants
-case "${PRODUCT}" in
-  thunderbird)
-    COMMIT_AUTHOR="tbirdbld <tbirdbld@thunderbird.net>"
-    ;;
-  firefox)
-    ;;
-  *)
-    echo "Error: Invalid product specified"
-    usage
-    exit 14
-    ;;
-esac
-
-if [ "${REPODIR}" == "" ]; then
-  REPODIR="$(basename "${BRANCH}")"
-fi
-
-case "${BRANCH}" in
-  mozilla-central|comm-central|try )
-    HGREPO="https://${HGHOST}/${BRANCH}"
-    ;;
-  mozilla-*|comm-* )
-    HGREPO="https://${HGHOST}/releases/${BRANCH}"
-    ;;
-  * )
-    HGREPO="https://${HGHOST}/projects/${BRANCH}"
-    ;;
-esac
-
-BROWSER_ARCHIVE="target.${PLATFORM_EXT}"
-TESTS_ARCHIVE="target.common.tests.tar.gz"
-
 preflight_cleanup
+
+mkdir -p "${DATADIR}"
+
+# Clone the repository here as some sections will use it for source data, and
+# we'll need it later anyway.
+if [ "${CLONE_REPO}" == "true" ]
+then
+  clone_repo
+fi
+
 if [ "${DO_HSTS}" == "true" ] || [ "${DO_HPKP}" == "true" ] || [ "${DO_PRELOAD_PINSET}" == "true" ]
 then
   if [ "${USE_TC}" == "true" ]; then
@@ -559,9 +660,18 @@ if [ "${DO_SUFFIX_LIST}" == "true" ]; then
     SUFFIX_LIST_UPDATED=true
   fi
 fi
+if [ "${DO_MOBILE_EXPERIMENTS}" == "true" ]; then
+  if compare_mobile_experiments
+  then
+    MOBILE_EXPERIMENTS_UPDATED=true
+  fi
+fi
+if [ "${DO_CT_LOGS}" == "true" ]; then
+  update_ct_logs
+fi
 
 
-if [ "${HSTS_UPDATED}" == "false" ] && [ "${HPKP_UPDATED}" == "false" ] && [ "${REMOTE_SETTINGS_UPDATED}" == "false" ] && [ "${SUFFIX_LIST_UPDATED}" == "false" ]; then
+if [ "${HSTS_UPDATED}" == "false" ] && [ "${HPKP_UPDATED}" == "false" ] && [ "${REMOTE_SETTINGS_UPDATED}" == "false" ] && [ "${SUFFIX_LIST_UPDATED}" == "false" ] && [ "${MOBILE_EXPERIMENTS_UPDATED}" == "false" ] && [ "${DO_CT_LOGS}" == "false" ]; then
   echo "INFO: no updates required. Exiting."
   exit 0
 else
@@ -571,8 +681,6 @@ else
   fi
 fi
 
-clone_repo
-
 COMMIT_MESSAGE="No Bug, ${BRANCH} repo-update"
 if [ "${HSTS_UPDATED}" == "true" ]
 then
@@ -588,7 +696,6 @@ fi
 
 if [ "${REMOTE_SETTINGS_UPDATED}" == "true" ]
 then
-  stage_remote_settings_files
   COMMIT_MESSAGE="${COMMIT_MESSAGE} remote-settings"
 fi
 
@@ -598,6 +705,18 @@ then
   COMMIT_MESSAGE="${COMMIT_MESSAGE} tld-suffixes"
 fi
 
+if [ "${MOBILE_EXPERIMENTS_UPDATED}" == "true" ]
+then
+  stage_mobile_experiments_files
+  COMMIT_MESSAGE="${COMMIT_MESSAGE} mobile-experiments"
+fi
+
+if [ "${DO_CT_LOGS}" == "true" ]
+then
+  # CT log files are already updated in-place in the tree, so
+  # there's no need to stage them.
+  COMMIT_MESSAGE="${COMMIT_MESSAGE} ct-logs"
+fi
 
 if [ ${DONTBUILD} == true ]; then
   COMMIT_MESSAGE="${COMMIT_MESSAGE} - (DONTBUILD)"
@@ -609,10 +728,16 @@ if [ ${APPROVAL} == true ]; then
   COMMIT_MESSAGE="${COMMIT_MESSAGE} - a=repo-update"
 fi
 
-
-if ${HG} -R "${REPODIR}" commit -u "${COMMIT_AUTHOR}" -m "${COMMIT_MESSAGE}"
-then
-  push_repo
+if [ "${USE_GIT}" == "true" ]; then
+  if ${GIT} -C "${TOPSRCDIR}" commit -a --author "${COMMIT_AUTHOR}" -m "${COMMIT_MESSAGE}"
+  then
+    push_repo
+  fi
+else
+  if ${HG} -R "${TOPSRCDIR}" commit -u "${COMMIT_AUTHOR}" -m "${COMMIT_MESSAGE}"
+  then
+    push_repo
+  fi
 fi
 
 echo "All done"
