@@ -128,6 +128,7 @@ def get_flavors(graph_config, param):
                     "Optional: an array of kinds to ignore from the previous "
                     "graph(s)."
                 ),
+                "default": graph_config["release-promotion"].get("rebuild-kinds", []),
                 "items": {
                     "type": "string",
                 },
@@ -391,9 +392,9 @@ def release_promotion_action(parameters, graph_config, input, task_group_id, tas
     if release_enable_emefree and not partner_url_config["release-eme-free-repack"]:
         raise Exception("Can't enable EMEfree repacks when no config url found")
     parameters["release_enable_partner_repack"] = release_enable_partner_repack
-    parameters[
-        "release_enable_partner_attribution"
-    ] = release_enable_partner_attribution
+    parameters["release_enable_partner_attribution"] = (
+        release_enable_partner_attribution
+    )
     parameters["release_enable_emefree"] = release_enable_emefree
 
     partner_config = input.get("release_partner_config")
