@@ -6,16 +6,16 @@
 import json
 import logging
 import os
-import requests
 import xml.etree.ElementTree as ET
-from copy import deepcopy
-from urllib.parse import urlencode
+from urllib.parse import quote, urlencode
 
+import requests
 import yaml
 from redo import retry
+from taskgraph.util.copy import deepcopy
+from taskgraph.util.schema import resolve_keyed_by
 
 from gecko_taskgraph.util.attributes import release_level
-from gecko_taskgraph.util.schema import resolve_keyed_by
 
 # Suppress chatty requests logging
 logging.getLogger("requests").setLevel(logging.WARNING)
@@ -199,7 +199,7 @@ def get_repo_params(repo):
     if repo.startswith("https"):
         # eg https://github.com/mozilla-partners/mozilla-EME-free
         return repo.rsplit("/", 2)[-2:]
-    elif repo.startswith("git@"):
+    if repo.startswith("git@"):
         # eg git@github.com:mozilla-partners/mailru.git
         repo = repo.replace(".git", "")
         return repo.split(":")[-1].split("/")
@@ -432,9 +432,9 @@ def fix_partner_config(orig_config):
                     if not subpartner_config:
                         continue
                     # Make sure our locale list is a subset of all_locales
-                    pc.setdefault(kind, {}).setdefault(partner, {})[
-                        subpartner
-                    ] = _fix_subpartner_locales(subpartner_config, all_locales)
+                    pc.setdefault(kind, {}).setdefault(partner, {})[subpartner] = (
+                        _fix_subpartner_locales(subpartner_config, all_locales)
+                    )
     return pc
 
 
@@ -442,18 +442,17 @@ def fix_partner_config(orig_config):
 def get_ftp_platform(platform):
     if platform.startswith("win32"):
         return "win32"
-    elif platform.startswith("win64-aarch64"):
+    if platform.startswith("win64-aarch64"):
         return "win64-aarch64"
-    elif platform.startswith("win64"):
+    if platform.startswith("win64"):
         return "win64"
-    elif platform.startswith("macosx"):
+    if platform.startswith("macosx"):
         return "mac"
-    elif platform.startswith("linux-"):
+    if platform.startswith("linux-"):
         return "linux-i686"
-    elif platform.startswith("linux64"):
+    if platform.startswith("linux64"):
         return "linux-x86_64"
-    else:
-        raise ValueError(f"Unimplemented platform {platform}")
+    raise ValueError(f"Unimplemented platform {platform}")
 
 
 # Ugh
@@ -509,13 +508,31 @@ def get_repack_ids_by_platform(config, build_platform):
 
 
 def get_partners_to_be_published(config):
-    # hardcoded kind because release-bouncer-aliases doesn't match otherwise
-    partner_config = get_partner_config_by_kind(config, "release-partner-repack")
+    return _get_repack_partners(config) + _get_attribution_partners(config)
+
+
+def _get_repack_partners(config):
+    repack_partner_config = get_partner_config_by_kind(config, "release-partner-repack")
     partners = []
-    for partner, subconfigs in partner_config.items():
+
+    for partner, subconfigs in repack_partner_config.items():
         for sub_config_name, sub_config in subconfigs.items():
             if sub_config.get("publish_to_releases"):
                 partners.append((partner, sub_config_name, sub_config["platforms"]))
+
+    return partners
+
+
+def _get_attribution_partners(config):
+    attribution_partner_config = get_partner_config_by_kind(
+        config, "release-partner-attribution"
+    )
+    partners = []
+
+    for entry in attribution_partner_config.get("configs", []):
+        if entry.get("publish_to_releases"):
+            partners.append((entry["campaign"], entry["content"], entry["platforms"]))
+
     return partners
 
 
@@ -524,7 +541,7 @@ def apply_partner_priority(config, jobs):
     # Reduce the priority of the partner repack jobs because they don't block QE. Meanwhile
     # leave EME-free jobs alone because they do, and they'll get the branch priority like the rest
     # of the release. Only bother with this in production, not on staging releases on try.
-    # medium is the same as mozilla-central, see taskcluster/ci/config.yml. ie higher than
+    # medium is the same as mozilla-central, see taskcluster/config.yml. ie higher than
     # integration branches because we don't want to wait a lot for the graph to be done, but
     # for multiple releases the partner tasks always wait for non-partner.
     if (
@@ -554,3 +571,48 @@ def generate_attribution_code(defaults, partner):
 
     code = urlencode(params)
     return code
+
+
+MACOS_ATTRIBUTION_SENTINEL = "__MOZCUSTOM__"
+
+
+def build_macos_attribution_dmg_command(dmg_app_path, attributions):
+    command = []
+    for a in attributions:
+        output_dir = os.path.dirname(os.path.abspath(a["output"]))
+        create_dir_command = "mkdir -p {}".format(output_dir)
+        if create_dir_command not in command:
+            command.append(create_dir_command)
+
+        command.append(
+            " ".join(
+                [
+                    dmg_app_path,
+                    "attribute",
+                    a["input"],
+                    a["output"],
+                    MACOS_ATTRIBUTION_SENTINEL,
+                    _build_macos_attribution_string(attribution_code=a["attribution"]),
+                ]
+            )
+        )
+    return " && ".join(command)
+
+
+def _build_macos_attribution_string(attribution_code):
+    quoted_attribution_code = quote(attribution_code)
+    attribution_string = "{}{}".format(
+        MACOS_ATTRIBUTION_SENTINEL, quoted_attribution_code
+    )
+    # Padding must happen after string is URL-quoted, otherwise the tabs themselves
+    # are quoted as well.
+    padded_attribution_string = _pad_macos_attribution_code(attribution_string)
+    return "'{}'".format(padded_attribution_string)
+
+
+def _pad_macos_attribution_code(attribution_string):
+    # Attribution length should be aligned with ATTR_CODE_MAX_LENGTH
+    #   from browser/components/attribution/AttributionCode.sys
+    while len(attribution_string) < 1010:
+        attribution_string += "\t"
+    return attribution_string
