@@ -16,20 +16,18 @@ happen on mozilla-beta and mozilla-release.
 Additional configuration is found in the :ref:`graph config <taskgraph-graph-config>`.
 """
 import functools
+import itertools
 import json
 import os
-import itertools
-from copy import deepcopy
 from datetime import datetime
 
 import jsone
-
 from mozbuild.util import memoize
+from taskgraph.util.copy import deepcopy
+from taskgraph.util.schema import resolve_keyed_by
 from taskgraph.util.taskcluster import get_artifact_prefix
 from taskgraph.util.yaml import load_yaml
 
-from .schema import resolve_keyed_by
-
 # constants {{{1
 """Map signing scope aliases to sets of projects.
 
@@ -52,7 +50,13 @@ SIGNING_SCOPE_ALIAS_TO_PROJECT = [
         {
             "mozilla-central",
             "comm-central",
-            "oak",
+            # bug 1845368: pine is a permanent project branch used for testing
+            # nightly updates
+            "pine",
+            # bug 1877483: larch has similar needs for nightlies
+            "larch",
+            # maple is also an L3 branch: https://phabricator.services.mozilla.com/D184833
+            "maple",
         },
     ],
     [
@@ -60,11 +64,12 @@ SIGNING_SCOPE_ALIAS_TO_PROJECT = [
         {
             "mozilla-beta",
             "mozilla-release",
-            "mozilla-esr91",
-            "mozilla-esr102",
+            "mozilla-esr115",
+            "mozilla-esr128",
             "comm-beta",
-            "comm-esr91",
-            "comm-esr102",
+            "comm-release",
+            "comm-esr115",
+            "comm-esr128",
         },
     ],
 ]
@@ -99,7 +104,11 @@ BEETMOVER_SCOPE_ALIAS_TO_PROJECT = [
         {
             "mozilla-central",
             "comm-central",
-            "oak",
+            # bug 1845368: pine is a permanent project branch used for testing
+            # nightly updates
+            "pine",
+            # bug 1877483: larch has similar needs for nightlies
+            "larch",
         },
     ],
     [
@@ -107,11 +116,12 @@ BEETMOVER_SCOPE_ALIAS_TO_PROJECT = [
         {
             "mozilla-beta",
             "mozilla-release",
-            "mozilla-esr91",
-            "mozilla-esr102",
+            "mozilla-esr115",
+            "mozilla-esr128",
             "comm-beta",
-            "comm-esr91",
-            "comm-esr102",
+            "comm-release",
+            "comm-esr115",
+            "comm-esr128",
         },
     ],
 ]
@@ -124,14 +134,33 @@ BEETMOVER_BUCKET_SCOPES = {
     "default": "beetmover:bucket:dep",
 }
 
+"""Map the beetmover scope aliases to the actual scopes.
+These are the scopes needed to import artifacts into the product delivery APT repos.
+"""
+BEETMOVER_APT_REPO_SCOPES = {
+    "all-release-branches": "beetmover:apt-repo:release",
+    "all-nightly-branches": "beetmover:apt-repo:nightly",
+    "default": "beetmover:apt-repo:dep",
+}
+
 """Map the beetmover tasks aliases to the actual action scopes.
 """
 BEETMOVER_ACTION_SCOPES = {
     "nightly": "beetmover:action:push-to-nightly",
-    "nightly-oak": "beetmover:action:push-to-nightly",
+    # bug 1845368: pine is a permanent project branch used for testing
+    # nightly updates
+    "nightly-pine": "beetmover:action:push-to-nightly",
+    # bug 1877483: larch has similar needs for nightlies
+    "nightly-larch": "beetmover:action:push-to-nightly",
     "default": "beetmover:action:push-to-candidates",
 }
 
+"""Map the beetmover tasks aliases to the actual action scopes.
+The action scopes are generic across different repo types.
+"""
+BEETMOVER_REPO_ACTION_SCOPES = {
+    "default": "beetmover:action:import-from-gcs-to-artifact-registry",
+}
 
 """Known balrog actions."""
 BALROG_ACTIONS = (
@@ -152,7 +181,11 @@ BALROG_SCOPE_ALIAS_TO_PROJECT = [
         {
             "mozilla-central",
             "comm-central",
-            "oak",
+            # bug 1845368: pine is a permanent project branch used for testing
+            # nightly updates
+            "pine",
+            # bug 1877483: larch has similar needs for nightlies
+            "larch",
         },
     ],
     [
@@ -166,20 +199,21 @@ BALROG_SCOPE_ALIAS_TO_PROJECT = [
         "release",
         {
             "mozilla-release",
-            "comm-esr91",
-            "comm-esr102",
+            "comm-release",
         },
     ],
     [
-        "esr91",
+        "esr115",
         {
-            "mozilla-esr91",
+            "mozilla-esr115",
+            "comm-esr115",
         },
     ],
     [
-        "esr102",
+        "esr128",
         {
-            "mozilla-esr102",
+            "mozilla-esr128",
+            "comm-esr128",
         },
     ],
 ]
@@ -191,8 +225,8 @@ BALROG_SERVER_SCOPES = {
     "aurora": "balrog:server:aurora",
     "beta": "balrog:server:beta",
     "release": "balrog:server:release",
-    "esr91": "balrog:server:esr",
-    "esr102": "balrog:server:esr",
+    "esr115": "balrog:server:esr",
+    "esr128": "balrog:server:esr",
     "default": "balrog:server:dep",
 }
 
@@ -238,8 +272,7 @@ def with_scope_prefix(f):
         scope_or_scopes = f(config, **kwargs)
         if isinstance(scope_or_scopes, list):
             return map(functools.partial(add_scope_prefix, config), scope_or_scopes)
-        else:
-            return add_scope_prefix(config, scope_or_scopes)
+        return add_scope_prefix(config, scope_or_scopes)
 
     return wrapper
 
@@ -319,6 +352,17 @@ get_beetmover_bucket_scope = functools.partial(
     alias_to_scope_map=BEETMOVER_BUCKET_SCOPES,
 )
 
+get_beetmover_apt_repo_scope = functools.partial(
+    get_scope_from_project,
+    alias_to_project_map=BEETMOVER_SCOPE_ALIAS_TO_PROJECT,
+    alias_to_scope_map=BEETMOVER_APT_REPO_SCOPES,
+)
+
+get_beetmover_repo_action_scope = functools.partial(
+    get_scope_from_release_type,
+    release_type_to_scope_map=BEETMOVER_REPO_ACTION_SCOPES,
+)
+
 get_beetmover_action_scope = functools.partial(
     get_scope_from_release_type,
     release_type_to_scope_map=BEETMOVER_ACTION_SCOPES,
@@ -378,10 +422,9 @@ def get_release_config(config):
 def get_signing_cert_scope_per_platform(build_platform, is_shippable, config):
     if "devedition" in build_platform:
         return get_devedition_signing_cert_scope(config)
-    elif is_shippable:
+    if is_shippable:
         return get_signing_cert_scope(config)
-    else:
-        return add_scope_prefix(config, "signing:cert:dep-signing")
+    return add_scope_prefix(config, "signing:cert:dep-signing")
 
 
 # generate_beetmover_upstream_artifacts {{{1
@@ -424,8 +467,6 @@ def generate_beetmover_upstream_artifacts(
     if not dependencies:
         if job.get("dependencies"):
             dependencies = job["dependencies"].keys()
-        elif job.get("primary-dependency"):
-            dependencies = [job["primary-dependency"].kind]
         else:
             raise Exception(f"Unsupported type of dependency. Got job: {job}")
 
@@ -433,6 +474,12 @@ def generate_beetmover_upstream_artifacts(
         paths = list()
 
         for filename in map_config["mapping"]:
+            resolve_keyed_by(
+                map_config["mapping"][filename],
+                "from",
+                f"beetmover filename {filename}",
+                platform=platform,
+            )
             if dep not in map_config["mapping"][filename]["from"]:
                 continue
             if locale != "en-US" and not map_config["mapping"][filename]["all_locales"]:
@@ -496,6 +543,26 @@ def generate_beetmover_upstream_artifacts(
     return upstream_artifacts
 
 
+def generate_artifact_registry_gcs_sources(dep):
+    gcs_sources = []
+    locale = dep.attributes.get("locale")
+    if not locale:
+        repackage_deb_reference = "<repackage-deb>"
+        repackage_deb_artifact = "public/build/target.deb"
+    else:
+        repackage_deb_reference = "<repackage-deb-l10n>"
+        repackage_deb_artifact = f"public/build/{locale}/target.langpack.deb"
+    for config in dep.task["payload"]["artifactMap"]:
+        if (
+            config["taskId"]["task-reference"] == repackage_deb_reference
+            and repackage_deb_artifact in config["paths"]
+        ):
+            gcs_sources.append(
+                config["paths"][repackage_deb_artifact]["destinations"][0]
+            )
+    return gcs_sources
+
+
 # generate_beetmover_artifact_map {{{1
 def generate_beetmover_artifact_map(config, job, **kwargs):
     """Generate the beetmover artifact map.
@@ -546,6 +613,9 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
         paths = dict()
         for filename in map_config["mapping"]:
             # Relevancy checks
+            resolve_keyed_by(
+                map_config["mapping"][filename], "from", "blah", platform=platform
+            )
             if dep not in map_config["mapping"][filename]["from"]:
                 # We don't get this file from this dependency.
                 continue
@@ -617,6 +687,10 @@ def generate_beetmover_artifact_map(config, job, **kwargs):
                 if file_config.get("balrog_format"):
                     paths[key]["balrog_format"] = file_config["balrog_format"]
 
+            # optional flag: expiry
+            if file_config.get("expiry"):
+                paths[key]["expiry"] = {"relative-datestamp": file_config["expiry"]}
+
         if not paths:
             # No files for this dependency/locale combination.
             continue
@@ -774,6 +848,12 @@ def generate_beetmover_partials_artifact_map(config, job, partials_info, **kwarg
                 if file_config.get("from_buildid"):
                     partials_paths[key]["from_buildid"] = file_config["from_buildid"]
 
+                # optional flag: expiry
+                if file_config.get("expiry"):
+                    partials_paths[key]["expiry"] = {
+                        "relative-datestamp": file_config["expiry"]
+                    }
+
                 # render buildid
                 kwargs.update(
                     {
