@@ -7,6 +7,7 @@ import logging
 import os
 import re
 import sys
+import warnings
 
 import attr
 from taskgraph.util.treeherder import join_symbol
@@ -211,7 +212,17 @@ def verify_routes_notification_filters(
     if task is None:
         return
     route_prefix = "notify."
-    valid_filters = ("on-any", "on-completed", "on-failed", "on-exception")
+    valid_filters = (
+        "on-any",
+        "on-completed",
+        "on-defined",
+        "on-failed",
+        "on-exception",
+        "on-pending",
+        "on-resolved",
+        "on-running",
+        "on-transition",
+    )
     task_dict = task.task
     routes = task_dict.get("routes", [])
 
@@ -225,6 +236,13 @@ def verify_routes_notification_filters(
                         task.label, route_filter
                     )
                 )
+            if route_filter == "on-any":
+                warnings.warn(
+                    DeprecationWarning(
+                        f"notification filter '{route_filter}' is deprecated. Use "
+                        "'on-transition' or 'on-resolved'."
+                    )
+                )
 
 
 @verifications.add("full_task_graph")
@@ -372,12 +390,19 @@ def verify_test_packaging(task, taskgraph, scratch_pad, graph_config, parameters
         missing_tests_allowed = any(
             (
                 # user specified `--target-kind`
-                parameters.get("target-kind") is not None,
+                bool(parameters.get("target-kinds")),
                 # manifest scheduling is enabled
                 parameters["test_manifest_loader"] != "default",
             )
         )
 
+        test_env = parameters["try_task_config"].get("env", {})
+        if test_env.get("MOZHARNESS_TEST_PATHS", "") or test_env.get(
+            "MOZHARNESS_TEST_TAG", ""
+        ):
+            # This is sort of a hack, as we are filtering, we might filter out all test jobs
+            missing_tests_allowed = True
+
         exceptions = []
         for task in taskgraph.tasks.values():
             if task.kind == "build" and not task.attributes.get(
