@@ -4,6 +4,13 @@ set -x -e -v
 artifact=$(basename $TOOLCHAIN_ARTIFACT)
 sysroot=${artifact%.tar.*}
 
+# Make the wasi compiler-rt available to clang.
+env UPLOAD_DIR= $GECKO_PATH/taskcluster/scripts/misc/repack-clang.sh
+
+if [ -n "$1" ]; then
+  patch -d $MOZ_FETCHES_DIR/wasi-sdk -p1 < $(dirname $0)/$1
+fi
+
 cd $MOZ_FETCHES_DIR/wasi-sdk
 LLVM_PROJ_DIR=$MOZ_FETCHES_DIR/llvm-project
 
@@ -13,22 +20,32 @@ mkdir -p build/install/wasi
 ln -s $MOZ_FETCHES_DIR/clang build/llvm
 touch build/llvm.BUILT
 
-# The wasi-sdk build system has a dependency on compiler-rt for libcxxabi,
-# but that's not actually necessary. Pretend it's already built
-touch build/compiler-rt.BUILT
-
 # The wasi-sdk build system wants a clang and an ar binary in
 # build/install/$PREFIX/bin
 ln -s $MOZ_FETCHES_DIR/clang/bin build/install/wasi/bin
 ln -s llvm-ar build/install/wasi/bin/ar
 
 # Build wasi-libc, libc++ and libc++abi.
-make \
-  LLVM_PROJ_DIR=$LLVM_PROJ_DIR \
-  PREFIX=/wasi \
-  build/wasi-libc.BUILT \
-  build/libcxx.BUILT \
-  -j$(nproc)
+do_make() {
+  make \
+    LLVM_PROJ_DIR=$LLVM_PROJ_DIR \
+    PREFIX=$(grep -q BUILD_PREFIX Makefile || echo $PWD/build/install)/wasi \
+    -j$(nproc) \
+    $1
+}
+
+do_make build/wasi-libc.BUILT
+
+# The wasi-sdk build system has a dependency on compiler-rt for libcxxabi,
+# but that's not actually necessary. Pretend it's already built.
+# Because compiler-rt has a dependency on wasi-libc, we can only do this
+# after wasi-libc is built.
+touch build/compiler-rt.BUILT
+
+do_make build/libcxx.BUILT
+if grep -q build/libcxxabi.BUILT Makefile; then
+    do_make build/libcxxabi.BUILT
+fi
 
 mv build/install/wasi/share/wasi-sysroot $sysroot
 tar --zstd -cf $artifact $sysroot
